/*
 * voodoo-capture.c
 *
 * Video capture routines for Voodoo3 3500 TV.
 *
 * $Id: voodoo-capture.c,v 1.1.1.1 2004/05/23 04:46:54 scottsman Exp $
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 */

#define __NO_VERSION__
#include <linux/kernel.h>
#include <linux/version.h>
#include <linux/module.h>
#include <linux/i2c.h>
#include <linux/videodev.h>
#include <asm/io.h>
#include "voodoo.h"

/* External variables: Data struct */
// extern struct voodoo_data *voodoo;

/* Module parameters */
extern int debug;
extern int model;
extern int ddc;

/* FIXME
 * Here's a loop that is somewhat CPU speed dependent.
 */
/* Make room (wait for it to become available) in the PCI FIFO buffer */
int v3tv_MakeRoom(int num)
{
	int i = 0;

	while ((v3tv_regread(V3REG_status) & 0x1F) < num) {
		if (i++ > 0x100000) {
			printk("v3tv: no room in voodoo3 FIFO: breakout!!!");
			return 0;
		}
	}
	return 1;
}

static int clip0Min;
static int clip0Max;
static int dstBaseAddr;
static int dstFormat;
static int commandExtra;
static int srcFormat;
static int srcSize;
static int srcXY;
static int dstSize;
static int dstXY;
static int command;
static int srcBaseAddr;

/* FIXME
 * This _will_ interfere with the X-server, at least on SMP boxes.
 */
void v3tv_2dreg(int i)
{
	if (i == VOODOO3_2D_SAVE) {
		clip0Min =	v3tv_regread(V3REG_clip0Min);
		clip0Max =	v3tv_regread(V3REG_clip0Max);
		dstBaseAddr =	v3tv_regread(V3REG_dstBaseAddr);
		dstFormat =	v3tv_regread(V3REG_dstFormat);
		commandExtra =	v3tv_regread(V3REG_commandExtra);
		srcFormat =	v3tv_regread(V3REG_srcFormat);
		srcSize =	v3tv_regread(V3REG_srcSize);
		srcXY =		v3tv_regread(V3REG_srcXY);
		dstSize =	v3tv_regread(V3REG_dstSize);
		dstXY =		v3tv_regread(V3REG_dstXY);
		command =	v3tv_regread(V3REG_command);
		srcBaseAddr =	v3tv_regread(V3REG_srcBaseAddr);
	} else if (i == VOODOO3_2D_RESTORE) {
		if (!v3tv_MakeRoom(12))
			printk("Error!!!\n");

		v3tv_regwrite(V3REG_clip0Min, clip0Min);
		v3tv_regwrite(V3REG_clip0Max, clip0Max);
		v3tv_regwrite(V3REG_dstBaseAddr, dstBaseAddr);
		v3tv_regwrite(V3REG_dstFormat, dstFormat);
		v3tv_regwrite(V3REG_commandExtra, commandExtra);
		v3tv_regwrite(V3REG_srcFormat, srcFormat);
		v3tv_regwrite(V3REG_srcSize, srcSize);
		v3tv_regwrite(V3REG_srcXY, srcXY);
		v3tv_regwrite(V3REG_dstSize, dstSize);
		v3tv_regwrite(V3REG_dstXY, dstXY);
		v3tv_regwrite(V3REG_command, command);
		v3tv_regwrite(V3REG_srcBaseAddr, srcBaseAddr);
	}
}

void v3tv_capture_frame(struct voodoo_data *voodoo_tv)
{
	int vidInStatus;
	int isOdd;
	u32 vidBuffer;
	int srcSize;
	int dstSize;
	int cmd;

	vidInStatus = v3tv_regread(V3REG_vidInStatusCurrentLine);
	vidInStatus >>= 16;
	vidInStatus &= 7;

	isOdd = vidInStatus & 4;

	/* Only use odd frames for now */

	switch (vidInStatus & 0x3) {
	case 0:
		/* Finished writing to vidInAddr0 */
//                      if(isOdd)
		vidBuffer = voodoo_tv->vidInAddr0;
//                      else
//                              vidBuffer = voodoo_tv->vidInAddr2;
		break;
	case 1:
		/* Finished writing to vidInAddr1 */
//                      if(isOdd)
		vidBuffer = voodoo_tv->vidInAddr1;
//                      else
//                              vidBuffer = voodoo_tv->vidInAddr0;
		break;
	case 2:
		/* Finished writing to vidInAddr2 */
//                      if(isOdd)
		vidBuffer = voodoo_tv->vidInAddr2;
//                      else
//                              vidBuffer = voodoo_tv->vidInAddr1;
		break;
	default:
		/* Error */
		vidBuffer = 0;
		break;
	}

	if (!vidBuffer)
		return;

	v3tv_2dreg(VOODOO3_2D_SAVE);

	if (!v3tv_MakeRoom(14)) {
		printk("Error2\n");
		return;
	}

	v3tv_set_palette(&voodoo_tv->picture_settings);
	printk("v3tv: capture frame, palette=%d, V3REG_status= %x\n",
	       voodoo_tv->picture_settings.palette, v3tv_regread(V3REG_status));

	v3tv_regwrite(V3REG_srcBaseAddr, (unsigned long)vidBuffer);
	v3tv_regwrite(V3REG_dstBaseAddr, (unsigned long)voodoo_tv->captureBuf);
//      v3tv_regwrite(V3REG_dstBaseAddr, 0);

	srcSize =
	    ((voodoo_tv->
	      vidOrigWidth) & 0xFFF) | (((voodoo_tv->
					  vidOrigHeight) << 16) & 0xFFF0000);
	dstSize =
	    ((voodoo_tv->vidOverlayEndX -
	      voodoo_tv->vidOverlayStartX) & 0xFFF) | (((voodoo_tv->vidOverlayEndY -
						      voodoo_tv->
						      vidOverlayStartY) << 16) &
						    0xFFF0000);

	/* [16:9] = 9 == packed 4:2:2 UYVY */
	v3tv_regwrite(V3REG_srcFormat,
			 (voodoo_tv->vidInStride & 0xFFF) | (9 << 16));

	v3tv_regwrite(V3REG_srcSize, srcSize);
	v3tv_regwrite(V3REG_dstSize, dstSize);
	v3tv_regwrite(V3REG_srcXY, 0);
	v3tv_regwrite(V3REG_dstXY, 0);

//      v3tv_regwrite(V3REG_dstFormat, (2048 & 0xFFF) | (3 << 16));
	v3tv_regwrite(V3REG_clip0Min, 0);
	v3tv_regwrite(V3REG_clip0Max, 0xFFFFFFFF);

	v3tv_regwrite(V3REG_commandExtra, 0);
	cmd = 0xCC000002;
	v3tv_regwrite(V3REG_command, cmd);	/* Command: screen to screen stretch blt */

	v3tv_regwrite(V3REG_launchArea, 0);
	v3tv_2dreg(VOODOO3_2D_RESTORE);
}

#if 0
static int fbuffer_alloc()
{
	if (!voodoo->fbuffer)
		voodoo->fbuffer =
		    (unsigned char *)rvmalloc(voodoo->gbuffers *
					      voodoo->gbufsize);
	else
		printk(KERN_ERR "v4l-v3tv: Double alloc of fbuffer!\n");

	if (!voodoo->fbuffer)
		return -ENOBUFS;
	return 0;
}
#endif

/*
 *	This maps the vmalloced and reserved fbuffer to user space.
 *
 *  FIXME: 
 *  - PAGE_READONLY should suffice!?
 *  - remap_page_range is kind of inefficient for page by page remapping.
 *    But e.g. pte_alloc() does not work in modules ... :-(
 */

#if 0
static int do_voodoo_mmap(const char *adr, unsigned long size)
{
	unsigned long start = (unsigned long)adr;
	unsigned long page, pos;

	if (size > voodoo->gbuffers * voodoo->gbufsize)
		return -EINVAL;

	if (!voodoo->fbuffer) {
		if (fbuffer_alloc())
			return -EINVAL;
	}
	pos = (unsigned long)voodoo->fbuffer;
	while (size > 0) {
		page = kvirt_to_pa(pos);
		if (remap_page_range(start, page, PAGE_SIZE, PAGE_SHARED))
			return -EAGAIN;
		start += PAGE_SIZE;
		pos += PAGE_SIZE;
		size -= PAGE_SIZE;
	}
	return 0;
}
#endif

/*
 * Overrides for Emacs so that we follow Linus's tabbing style.
 * ---------------------------------------------------------------------------
 * Local variables:
 * c-basic-offset: 8
 * End:
 */
