/*
 * voodoo-i2c.c - Part of v4l voodoo3 drivers
 * Copyright (c) 2000  Juha Valkama <jusva@yahoo.com>
 *
 * Based on work by (at least) the following people:
 * Frodo Looijaard <frodol@dds.nl>
 * Philip Edelbrock <phil@netroedge.com>
 * Ralph Metzler <rjkm@thp.uni-koeln.de>
 * Mark D. Studebaker <mdsxyz123@yahoo.com>
 *
 * $Id: voodoo-i2c.c,v 1.1.1.1 2004/05/23 04:46:52 scottsman Exp $
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 */

#define __NO_VERSION__
#include <linux/kernel.h>
#include <linux/version.h>
#include <linux/module.h>
#include <linux/i2c.h>
#include <linux/i2c-algo-bit.h>
#include <linux/videodev.h>
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
#include <media/tuner.h>
#else
#include "../linux/drivers/media/video/tuner.h"
#endif
#include "voodoo.h"

//#ifndef I2C_DRIVERID_VPX3224
//#include "vpx-common.h"
//#endif

/* External variables: Data struct */
extern struct voodoo_data *voodoo;

/* Module parameters */
extern int debug;
extern int model;

int v3tv_get_adapter(struct i2c_adapter *adapter)
{
//	if (I2C_ADAP_CLASS_TV_ANALOG != adapter->class)
//		return 0;
#ifdef I2C_ADAP_CLASS_TV_ANALOG
	if (!(adapter->class & I2C_ADAP_CLASS_TV_ANALOG))
		return 0;
#else
	switch (adapter->id) {
	case I2C_ALGO_BIT | I2C_HW_B_VOO:
		if (!strnicmp(adapter->name, "I2C\0", 3))
			break;
	default:
		return 0;
	}
#endif	
	if (debug)
		printk(KERN_INFO "v3tv: i2c_attach_client found adapter: %s\n",
			adapter->name);

	if (voodoo->voodoo_i2c_adapter == adapter) {
		if (debug)
			printk(KERN_INFO "v3tv: i2c_attach_client releasing adapter: %s\n",
			       adapter->name);
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
		module_put(voodoo->voodoo_i2c_adapter->owner);
#else
		__MOD_DEC_USE_COUNT(voodoo->voodoo_i2c_adapter->owner);
#endif
		voodoo->voodoo_i2c_adapter = NULL;
		return 0;
	}

	voodoo->voodoo_i2c_adapter = adapter;
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
	if (!try_module_get(adapter->owner)) {
		printk("v3tv: Unable to get module %s\n",
			       adapter->owner->name);
		return -ENODEV;
	}
#else
	__MOD_INC_USE_COUNT(voodoo->voodoo_i2c_adapter->owner);
#endif
	printk ("v3tv: i2c dummy driver registered: adapter: 0x%p\n",
		voodoo->voodoo_i2c_adapter);

	return 0;
}

struct i2c_driver voodoo_i2c_driver = {
	.owner		= THIS_MODULE,
	.name		= "v3tv i2c dummy driver",
	.id		= 0,
/* FIXME: the 2.6 kernel does not have _DUMMY, use .detach_adapter */
#ifdef I2C_DF_DUMMY
	.flags		= I2C_DF_DUMMY,
#else
	.flags		= I2C_DF_NOTIFY,
	.detach_adapter	= v3tv_get_adapter,
#endif
	.attach_adapter = v3tv_get_adapter,
	.detach_client = NULL,
	.command = NULL,
};

int v3tv_i2c_setup(int arg)
{
	if (arg) {		/* get i2c adapter pointer */
		i2c_add_driver(&voodoo_i2c_driver);
		if (NULL == voodoo->voodoo_i2c_adapter) {
			printk
			    ("v3tv: i2c registration failed, module not inserted\n");
			return -ENODEV;
		}
	} else {		/* release adapter */
		if (voodoo->voodoo_i2c_adapter) {
			i2c_del_driver(&voodoo_i2c_driver);
		}
	}
	return 0;
}

/*
 * Overrides for Emacs so that we follow Linus's tabbing style.
 * ---------------------------------------------------------------------------
 * Local variables:
 * c-basic-offset: 8
 * End:
 */
