/*
 * voodoo-overlay.c
 *
 * Video overlay handling routines for Voodoo3 3500 TV.
 *
 * $Id: voodoo-overlay.c,v 1.1.1.1 2004/05/23 04:46:54 scottsman Exp $
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 */

#define __NO_VERSION__
#include <linux/kernel.h>
#include <linux/version.h>
#include <linux/module.h>
#include <linux/slab.h>
//#include <linux/i2c.h>
//#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)
//#include "../linux/drivers/media/video/i2c-compat.h"
//#endif
#include <linux/videodev.h>
#include <linux/video_decoder.h>
#include "voodoo.h"

/* External variables: Data struct */
// extern struct voodoo_data *voodoo;

/* Module parameters */
extern int debug;
extern int model;

struct video_capture video_standard[] = {
/*         x,    y,   width, height, decimation, flags */
	{0, 0, 768, 288, 0, 0},	/* PAL */
	{0, 0, 640, 240, 0, 0},	/* NTSC */
	{0, 0, 768, 288, 0, 0},	/* SECAM */
	{0, 0, 768, 288, 0, 0}	/* AUTO *//* we should never get here */
//      { 32,   24,     768,    288, 0, 0 },    /* PAL */
//      { 32,   24,     640,    240, 0, 0 },    /* NTSC */
//      { 32,   24,     768,    288, 0, 0 },    /* SECAM */
//      {  0,    0,     768,    288, 0, 0 }     /* AUTO */ /* we should never get here */
};

/* FIXME: Todo stuff.
 * - Background thread to monitor Voodoo3 for setting changes.
 * - Reset video processor on resolution switch (must figure out how to reset first).
 * - Update overlay window position when the virtual desktop scrolls.
 */

/******************************
 * Overlay handling functions *
 *****************************/

/* FIXME:
 * The chromakey register has different formats, depending on the desktop color bits.
 * This means, that the chromakey will be interpreted differently in different video modes.
 */

// static struct conf_struct Chromakey_Disable[] =
// {
//      { V3REG_vidProcCfg, 0, 5, 5}
// };

void v3tv_set_chromakey(struct voodoo_data *voodoo)
{
	static struct conf_struct Chromakey_Enable[] =
	    { {V3REG_vidProcCfg, 1, 5, 5} };

	if (debug >= 2)
		printk("v3tv: set_chromakey (0x%lx)\n", voodoo->chroma);

	v3tv_conf(Chromakey_Enable, CONF_SIZE(Chromakey_Enable));

	v3tv_regwrite(V3REG_vidChromaMin, voodoo->chroma);
	v3tv_regwrite(V3REG_vidChromaMax, voodoo->chroma);
}

/* do a fraction a/b in numBits format is 0.numbits*/
long do_fraction(int numBits, unsigned a, unsigned b)
{
	/* This is _not_ the "official" algorithm from voodoo3_spec.pdf
	 * This should work in typical cases.
	 */

	u32 num;

	if ((0xFFFFFFFF << (32 - numBits)) & a) {
		printk("v3tv: Overflow in do_fraction (%d, %d, %d)\n",
		       numBits, a, b);
	}

	num = ((u32) a << numBits) / b;

	if (debug >= 2) {
		printk("v3tv: do_fraction (0x%lx) -> 0x%lx\n",
		       (unsigned long)a << numBits, (unsigned long)num);
	}
	return num;
}

/* Update overlay window size and strides
 * FIXME:
 * This could possibly be made simpler, if we used constantly sized input video.
 * I'll need to get back to it...
 */

void v3tv_calc_scale(struct voodoo_data *voodoo)
{
	if (voodoo->vidOrigWidth <= voodoo->overlayWidth) {
		if (voodoo->vidOrigWidth < voodoo->overlayWidth) {
			voodoo->dudx =
			    do_fraction(20, voodoo->vidOrigWidth,
					voodoo->overlayWidth);
			if (debug)
				printk
				    ("v3tv: Horizontal Magnification %d\n",
				     voodoo->dudx);
		}

		voodoo->vidOverlayStride = voodoo->vidInStride;
	} else {
		voodoo->XDecim = (voodoo->overlayWidth & 0xFFF) |
		    (((voodoo->vidOrigWidth -
		       voodoo->overlayWidth) & 0xFFF) << 16);
		if (debug)
			printk("v3tv: Horizontal Decimation %d\n",
			       voodoo->XDecim);
		voodoo->vidInStride = voodoo->overlayWidth * 2;
		voodoo->vidOverlayStride = voodoo->vidInStride;
	}

	if (voodoo->vidOrigHeight <= voodoo->overlayHeight) {
		if (voodoo->vidOrigHeight < voodoo->overlayHeight) {
			voodoo->dvdy =
			    do_fraction(20, voodoo->vidOrigHeight,
					voodoo->overlayHeight);
			if (debug)
				printk("v3tv: Vertical Magnification %d\n",
				       voodoo->dvdy);
		}
	} else {
		voodoo->YDecim = (voodoo->overlayHeight & 0xFFF) |
		    (((voodoo->vidOrigHeight -
		       voodoo->overlayHeight) & 0xFFF) << 16);
		if (debug)
			printk("v3tv: Vertical Decimation %d\n",
			       voodoo->YDecim);
	}
}

void v3tv_update_overlay_setup(struct voodoo_data *voodoo)
{
	unsigned int value = 0;
	voodoo->dudx = voodoo->dvdy = voodoo->XDecim = voodoo->YDecim = 0;

	if (!voodoo)
		return;

	voodoo->vidInStride = voodoo->vidOrigWidth * 2;
	voodoo->overlayWidth =
	    voodoo->vidOverlayEndX - voodoo->vidOverlayStartX;
	voodoo->overlayHeight =
	    voodoo->vidOverlayEndY - voodoo->vidOverlayStartY;

	voodoo->vidOverlayStride = voodoo->vidInStride;
	voodoo->vidScanlineBytes = voodoo->vidInStride;

	v3tv_calc_scale(voodoo);

	value = voodoo->vidOverlayStartX & 0xFFF;
	value |= ((voodoo->vidOverlayStartY << 12) & 0xFFF000);
	v3tv_regwrite(V3REG_vidOverlayStartCoords, value);

	value = voodoo->vidOverlayEndX & 0xFFF;
	value |= ((voodoo->vidOverlayEndY << 12) & 0xFFF000);
	v3tv_regwrite(V3REG_vidOverlayEndScreenCoord, value);

	value = v3tv_regread(V3REG_vidDesktopOverlayStride);
	value =
	    (value & 0x8000FFFF) | ((voodoo->vidOverlayStride << 16) &
				    0x7FFF0000);
	v3tv_regwrite(V3REG_vidDesktopOverlayStride, value);

	v3tv_regwrite(V3REG_vidInStride, voodoo->vidInStride & 0x7FFF);

	v3tv_regwrite(V3REG_vidOverlayDudxOffsetSrcWidth,
			 voodoo->vidScanlineBytes << 19);

	v3tv_regwrite(V3REG_vidOverlayDudx, voodoo->dudx);
	v3tv_regwrite(V3REG_vidOverlayDvdy, voodoo->dvdy);

	v3tv_regwrite(V3REG_vidInXDecimDeltas, voodoo->XDecim);
	v3tv_regwrite(V3REG_vidInYDecimDeltas, voodoo->YDecim);

	if (debug)
		printk
		    ("v3tv: overlay start x: %d y: %d end x: %d y: %d (W: %d H: %d)\n",
		     voodoo->vidOverlayStartX, voodoo->vidOverlayStartY,
		     voodoo->vidOverlayEndX, voodoo->vidOverlayEndY,
		     voodoo->overlayWidth, voodoo->overlayHeight);
	if (debug)
		printk
		    ("v3tv: Overlay Stride: %d vidInStride: %d Bytes: %d\n",
		     voodoo->vidOverlayStride, voodoo->vidInStride,
		     voodoo->vidScanlineBytes);

	if (voodoo->XDecim) {
		static struct conf_struct Horizontal_Decimation_Enable[] = 
			{ {V3REG_vidInFormat, 1, 20, 20} };	/* Horizontal Decimation enable */
		v3tv_conf(Horizontal_Decimation_Enable,
			     CONF_SIZE(Horizontal_Decimation_Enable));
	} else {
		static struct conf_struct Horizontal_Decimation_Disable[] = 
			{ {V3REG_vidInFormat, 0, 20, 20} };	/* Horizontal Decimation disable */
		v3tv_conf(Horizontal_Decimation_Disable,
			     CONF_SIZE(Horizontal_Decimation_Disable));
	}

	if (voodoo->YDecim) {
		static struct conf_struct Vertical_Decimation_Enable[] = 
			{ {V3REG_vidInFormat, 1, 21, 21} };	/* Vertical Decimation enable */
		v3tv_conf(Vertical_Decimation_Enable,
			     CONF_SIZE(Vertical_Decimation_Enable));
	} else {
		static struct conf_struct Vertical_Decimation_Disable[] = 
			{ {V3REG_vidInFormat, 0, 21, 21} };	/* Vertical Decimation disable */
		v3tv_conf(Vertical_Decimation_Disable,
			     CONF_SIZE(Vertical_Decimation_Disable));
	}

	if (voodoo->dudx) {
		static struct conf_struct Horizontal_Magnification_Enable[] = 
			{ {V3REG_vidProcCfg, 1, 14, 14} };	/* Horizontal Scaling enable */
		v3tv_conf(Horizontal_Magnification_Enable,
			     CONF_SIZE(Horizontal_Magnification_Enable));
	} else {
		static struct conf_struct Horizontal_Magnification_Disable[] = 
			{ {V3REG_vidProcCfg, 0, 14, 14} };	/* Horizontal Scaling disable */
		v3tv_conf(Horizontal_Magnification_Disable,
			     CONF_SIZE(Horizontal_Magnification_Disable));
	}

	if (voodoo->dvdy) {
		static struct conf_struct Vertical_Magnification_Enable[] = 
			{ {V3REG_vidProcCfg, 1, 15, 15} };	/* Vertical Scaling enable */
		v3tv_conf(Vertical_Magnification_Enable,
			     CONF_SIZE(Vertical_Magnification_Enable));
	} else {
		static struct conf_struct Vertical_Magnification_Disable[] = 
			{ {V3REG_vidProcCfg, 0, 15, 15} };	/* Vertical Scaling disable */
		v3tv_conf(Vertical_Magnification_Disable,
			     CONF_SIZE(Vertical_Magnification_Disable));
	}
}

void v3tv_set_overlay(struct voodoo_data *voodoo)
{
	voodoo->vidOverlayStartX = voodoo->capture_win.x;
	voodoo->vidOverlayStartY = voodoo->capture_win.y;
	voodoo->vidOverlayEndX =
	    voodoo->capture_win.x + voodoo->capture_win.width;
	voodoo->vidOverlayEndY =
	    voodoo->capture_win.y + voodoo->capture_win.height;
	voodoo->vidOrigWidth = video_standard[voodoo->norm].width;
	voodoo->vidOrigHeight = video_standard[voodoo->norm].height;
	v3tv_update_overlay_setup(voodoo);
}

void v3tv_overlay_setup(struct voodoo_data *voodoo)
{
	unsigned int value = 0;
	static int dstFormat;

	voodoo->vidInStride = video_standard[voodoo->norm].width * 2;

	value = v3tv_regread(V3REG_vidScreenSize);
	voodoo->desktopWidth = value & 0xFFF;
	voodoo->desktopHeight = (value >> 12) & 0xFFF;

	voodoo->desktopAddr =
	    v3tv_regread(V3REG_vidDesktopStartAddr) & 0xFFFFFF;
	voodoo->desktopStride =
	    v3tv_regread(V3REG_vidDesktopOverlayStride) & 0x7FFF;

	/*
	 * Allocate memory for video capture and triple buffering overlay
	 * Start from the top of video memory, but leave 4K left for the hw mouse cursor.
	 * FIXME: We need to make this more intelligent, because the X-server might use
	 * memory differently in the future...
	 */
	voodoo->vidInAddr2 =
	    0xFFF000 -
	    video_standard[voodoo->norm].width *
	    (video_standard[voodoo->norm].height * 2);
	voodoo->vidInAddr1 =
	    voodoo->vidInAddr2 -
	    video_standard[voodoo->norm].width *
	    (video_standard[voodoo->norm].height * 2);
	voodoo->vidInAddr0 =
	    voodoo->vidInAddr1 -
	    video_standard[voodoo->norm].width *
	    (video_standard[voodoo->norm].height * 2);
	voodoo->captureBuf =
	    voodoo->vidInAddr0 -
	    (video_standard[voodoo->norm].width * 2) *
	    (video_standard[voodoo->norm].height * 4) * 3;

	v3tv_regwrite(V3REG_vidInAddr0, voodoo->vidInAddr0);
	v3tv_regwrite(V3REG_vidInAddr1, voodoo->vidInAddr1);
	v3tv_regwrite(V3REG_vidInAddr2, voodoo->vidInAddr2);

	dstFormat = (v3tv_regread(V3REG_dstFormat) >> 16) & 0x07;

	if (debug) {
		printk
		    ("v3tv: %d x %d sized %sbpp desktop at vidmem address 0x%x\n",
		     voodoo->desktopWidth, voodoo->desktopHeight,
		     (dstFormat == 1) ? "8" : (dstFormat ==
					       3) ? "16" : (dstFormat ==
							    4) ? "24"
		     : (dstFormat == 5) ? "32" : "unknown-",
		     voodoo->desktopAddr);
		printk("v3tv: Desktop stride is %d bytes.\n",
		       voodoo->desktopStride);
		printk("v3tv: Video in stride is %d bytes.\n",
		       voodoo->vidInStride);
		printk("v3tv: Capture buffer at vidmem 0x%x\n",
		       voodoo->captureBuf);
		printk("v3tv: Triple buffers at 0x%x, 0x%x, 0x%x\n",
		       voodoo->vidInAddr0, voodoo->vidInAddr1,
		       voodoo->vidInAddr2);
	}

	/* FIXME: Sometimes, the OverlayStride is incorrect. Check for this */
	value = v3tv_regread(V3REG_vidDesktopOverlayStride);
	value =
	    (value & 0x8000FFFF) +
	    ((voodoo->vidInStride << 16) & 0x7FFF0000);

	v3tv_regwrite(V3REG_vidDesktopOverlayStride, value);
	v3tv_regwrite(V3REG_vidOverlayDudxOffsetSrcWidth,
			 (voodoo->vidInStride) << 19);

	v3tv_set_overlay(voodoo);
}

void v3tv_overlay(struct voodoo_data *voodoo, int i)
{
	int tempReg;
	if (i == 0) {
		static struct conf_struct Overlay_Disable[] =
		    { {V3REG_vidProcCfg, 0, 8, 8},
		{V3REG_vidProcCfg, 0, 9, 9}
		};
		v3tv_conf(Overlay_Disable, CONF_SIZE(Overlay_Disable));
//              voodoo3_conf(Disable_VMI_Interrupt, CONF_SIZE(Disable_VMI_Interrupt));

		tempReg = v3tv_regread(V3REG_intrCtrl);
		if (debug)
			printk("v3tv: intrCtrl: %x\n", tempReg);
		tempReg &= ~(1 << 21);
		if (debug)
			printk("v3tv: write back intrCtrl: %x "
			       "VMI interupts disabled\n", tempReg);
		v3tv_regwrite(V3REG_intrCtrl, tempReg);
	} else {
		static struct conf_struct Overlay_Enable[] = {
			{V3REG_vidProcCfg, -1, 5, 5},	/* Disable Chroma Key                   */
			{V3REG_vidProcCfg, 1, 8, 8},	/* Overlay surface enable               */
			{V3REG_vidProcCfg, 1, 9, 9},	/* Use vidInAddr as overlay buffer      */
			{V3REG_vidProcCfg, 1, 11, 11},	/* Overlay ramdac-clut bypass           */
			{V3REG_vidProcCfg, -1, 13, 13},	/* Overlay clut don't care              */
			{V3REG_vidProcCfg, -1, 14, 14},	/* overlay horizontal scaling           */
			{V3REG_vidProcCfg, -1, 15, 15},	/* overlay vertical scaling             */
			{V3REG_vidProcCfg, 3, 17, 16},	/* overlay filter mode                  */
			{V3REG_vidProcCfg, 6, 23, 21},	/* overlay pixel format (422 UYVY)      */
			{V3REG_vidProcCfg, 0, 25, 25},	/* overlay tile space (0=linear)        */
			{V3REG_vidProcCfg, -1, 30, 30},	/* Reserved                             */
			{V3REG_vidProcCfg, 0, 31, 31},	/* No overlay backend de-interlacing    */
			{V3REG_vidInFormat, 1, 14, 14},	/* VMI interface enable                 */
			{V3REG_vidSerialParallelPort, 1, 0, 0}	/* VMI host interface enable            */
		};
		v3tv_conf(Overlay_Enable, CONF_SIZE(Overlay_Enable));

		/* Overlay filtering must be turned off, if video processor
		 * is set to 2x mode.  This makes the overlay look bad,
		 * so we disable filtering only when really needed.
		 */
		if (v3tv_regread(V3REG_vidProcCfg) & (1 << 26)) {
			static struct conf_struct Overlay_Filter_Off[] = { {V3REG_vidProcCfg, 0, 17, 16} };	/* overlay filter OFF */
			v3tv_conf(Overlay_Filter_Off,
				     CONF_SIZE(Overlay_Filter_Off));
			if (debug)
				printk(KERN_WARNING
				       "v3tv: Overlay filter disabled\n");
		} else {
			static struct conf_struct Overlay_Filter_On[] = { {V3REG_vidProcCfg, 3, 17, 16} };	/* overlay filter ON */
			v3tv_conf(Overlay_Filter_On,
				     CONF_SIZE(Overlay_Filter_On));
			if (debug)
				printk(KERN_WARNING
				       "v3tv: Overlay filter enabled\n");
		}
//              voodoo3_conf(Enable_VMI_Interrupt, CONF_SIZE(Enable_VMI_Interrupt));
		tempReg = v3tv_regread(V3REG_intrCtrl);
		if (debug)
			printk("v3tv: intrCtrl: %x\n", tempReg);
		tempReg |= (1 << 21);
		tempReg |= (1 << 31);
		if (debug)
			printk("v3tv: write back intrCtrl: %x "
			       "VMI interupts enable; PCI interupt is inactive\n",
			       tempReg);
		v3tv_regwrite(V3REG_intrCtrl, tempReg);

	}
	if (debug)
		printk(KERN_INFO "v3tv: Overlay %s\n",
		       (i) ? "Enabled" : "Disabled");
}

#if 0
static struct conf_struct Enable_VMI_Interrupt[] =
    { {V3REG_intrCtrl, 1, 21, 21},
{V3REG_intrCtrl, 0, 23, 23}
};

static struct conf_struct Disable_VMI_Interrupt[] =
    { {V3REG_intrCtrl, 0, 21, 21},
{V3REG_intrCtrl, 0, 23, 23}
};

static struct conf_struct Clear_VMI_Interrupt[] =
    { {V3REG_intrCtrl, 0, 23, 23} };

static struct conf_struct Overlay_Setup[] = { {V3REG_vidProcCfg, 1, 8, 8},
{V3REG_vidProcCfg, 1, 9, 9}
};
#endif

/*
 * Overrides for Emacs so that we follow Linus's tabbing style.
 * ---------------------------------------------------------------------------
 * Local variables:
 * c-basic-offset: 8
 * End:
 */
