/*
 * Here's some pretty complicated stuff...
 * We probably need to start inserting spinlock-stuff here,
 * or in the functions that call these ones.
 *
 * $Id: voodoo-util.c,v 1.1.1.1 2004/05/23 04:46:54 scottsman Exp $
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 */

#define __NO_VERSION__
#include <linux/kernel.h>
#include <linux/version.h>
#include <linux/module.h>
#include <linux/i2c.h>
#include <linux/videodev.h>
#include <linux/video_decoder.h>
#include <linux/spinlock.h>
#include <linux/vmalloc.h>
#include <asm/io.h>
#include "voodoo.h"

/* External variables: Data struct */
extern struct voodoo_data *voodoo;

/* Module parameters */
extern int debug;
extern int model;

void v3tv_regwrite(unsigned int reg, unsigned int data)
{
	if (!voodoo || !voodoo->io_base)
		return;

	*((unsigned int *)(voodoo->io_base + reg)) = data;
}

/*
inline static void voodoo3_regor(unsigned int reg, unsigned int data, unsigned int mask)
{
	unsigned int *reg_ptr;

	if (!voodoo || !voodoo->io_base)
		return;
	
	reg_ptr = (unsigned int *) voodoo->io_base + reg;
	*reg_ptr = (*reg_ptr & ~mask) | (data & mask);
}
*/

unsigned int v3tv_regread(unsigned int reg)
{
	if (!voodoo || !voodoo->io_base)
		return 0;

	return *((unsigned int *)(voodoo->io_base + reg));
}

static inline unsigned int do_mask(struct conf_struct *conf)
{
	unsigned int mask = 0;
	int i;
	for (i = conf->startbit; i <= conf->endbit; i++)
		mask |= (1 << i);

	return mask;
}

static inline unsigned int do_value(unsigned int orig, struct conf_struct *conf)
{
	unsigned int mask = 0;

	mask = do_mask(conf);

	return ((conf->value << conf->startbit) & mask) | (orig & (~mask));
}

void v3tv_conf(struct conf_struct *conf, int num_entries)
{
	int i;
	int lastReg = 0;
	unsigned int cur_value = 0;
	spinlock_t v3conf_lock = SPIN_LOCK_UNLOCKED;

	if (num_entries < 1) {
		printk("v3tv: Empty config data in voodoo3_conf()\n");
		return;
	}

	lastReg = conf[0].reg;
	cur_value = v3tv_regread(lastReg);

	if (debug >= 2)
		printk("v3tv: register 0x%x: 0x%x", lastReg, cur_value);

	spin_lock(&v3conf_lock);

	for (i = 0; i < num_entries; i++) {
		if (lastReg != conf[i].reg) {
			/* Write Register with mask */
			v3tv_regwrite(lastReg, cur_value);
			if (debug >= 2)
				printk(" -> 0x%x\n", cur_value);

			/* Set lastreg to point to current reg */
			lastReg = conf[i].reg;
			cur_value = v3tv_regread(lastReg);
			if (debug >= 2)
				printk("v3tv: register 0x%x: 0x%x",
				       lastReg, cur_value);
		}

		if (conf[i].value != -1) {
			cur_value = do_value(cur_value, &conf[i]);
		}
	}

	v3tv_regwrite(lastReg, cur_value);

	spin_unlock(&v3conf_lock);

	if (debug >= 2)
		printk(" -> 0x%x\n", cur_value);
}

/*******************************/
/* Memory management functions */
/*	these are from bttv    */
/*******************************/

#define MDEBUG(x)	do { } while(0)	/* Debug memory management */

/* [DaveM] I've recoded most of this so that:
 * 1) It's easier to tell what is happening
 * 2) It's more portable, especially for translating things
 *    out of vmalloc mapped areas in the kernel.
 * 3) Less unnecessary translations happen.
 *
 * The code used to assume that the kernel vmalloc mappings
 * existed in the page tables of every process, this is simply
 * not guarenteed.  We now use pgd_offset_k which is the
 * defined way to get at the kernel page tables.
 */

/* Given PGD from the address space's page table, return the kernel
 * virtual mapping of the physical memory mapped at ADR.
 */
#if 0
static inline unsigned long uvirt_to_kva(pgd_t * pgd, unsigned long adr)
{
	unsigned long ret = 0UL;
	pmd_t *pmd;
	pte_t *ptep, pte;

	if (!pgd_none(*pgd)) {
		pmd = pmd_offset(pgd, adr);
		if (!pmd_none(*pmd)) {
			ptep = pte_offset_map(pmd, adr);
			pte = *ptep;
			if (pte_present(pte)) {
				ret =
				    (unsigned long)page_address(pte_page(pte));
				ret |= (adr & (PAGE_SIZE - 1));
			}
		}
	}
	MDEBUG(printk("v3tv: uv2kva(%lx-->%lx)", adr, ret));
	return ret;
}
#endif

static inline unsigned long uvirt_to_kva(pgd_t * pgd, unsigned long adr)
{
	unsigned long ret = 0UL;
	pmd_t *pmd;
	pte_t *ptep, pte;

	if (!pgd_none(*pgd)) {
		pmd = pmd_offset(pgd, adr);
		if (!pmd_none(*pmd)) {
#if defined (pte_offset_map)	/* 2.4-rmap or 2.5-2.6 */
			ptep = pte_offset_map(pmd, adr);
			pte = *ptep;
			pte_unmap(ptep);
#else
#if defined (pte_offset_atomic)	/* 2.4-aa */
			ptep = pte_offset_atomic(pmd, adr);
			pte = *ptep;
			pte_kunmap(ptep);
#else
			ptep = NULL;
			pte = *pte_offset(pmd, adr);
#endif
#endif
			if (pte_present(pte)) {
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0)
				ret =
				    (unsigned long)page_address(pte_page(pte));
#else
				ret = pte_page(pte);
#endif
				ret |= (adr & (PAGE_SIZE - 1));
			}
		}
	}
	MDEBUG(printk("v3tv: uv2kva(%lx-->%lx)", adr, ret));
	return ret;
}

static inline unsigned long uvirt_to_bus(unsigned long adr)
{
	unsigned long kva, ret;

	kva = uvirt_to_kva(pgd_offset(current->mm, adr), adr);
	ret = virt_to_bus((void *)kva);
	MDEBUG(printk("v3tv: uv2b(%lx-->%lx)", adr, ret));
	return ret;
}

static inline unsigned long kvirt_to_bus(unsigned long adr)
{
	unsigned long va, kva, ret;

	va = ((unsigned long) (adr));
	kva = uvirt_to_kva(pgd_offset_k(va), va);
	ret = virt_to_bus((void *)kva);
	MDEBUG(printk("v3tv: kv2b(%lx-->%lx)", adr, ret));
	return ret;
}

/* Here we want the physical address of the memory.
 * This is used when initializing the contents of the
 * area and marking the pages as reserved.
 */
inline unsigned long kvirt_to_pa(unsigned long adr)
{
	unsigned long va, kva, ret;

	va = ((unsigned long) (adr));
	kva = uvirt_to_kva(pgd_offset_k(va), va);
	ret = __pa(kva);
	MDEBUG(printk("v3tv: kv2pa(%lx-->%lx)", adr, ret));
	return ret;
}

#if LINUX_VERSION_CODE <= KERNEL_VERSION(2,5,0)
#include <linux/wrapper.h>
#else
#define mem_map_reserve(p)      set_bit(PG_reserved, &((p)->flags))
#define mem_map_unreserve(p)    clear_bit(PG_reserved, &((p)->flags))
#endif

void *rvmalloc(signed long size)
{
	void *mem;
	unsigned long adr, page;

	mem = vmalloc(size);
	if (mem) {
		memset(mem, 0, size);	/* Clear the ram out, no junk to the user */
		adr = (unsigned long)mem;
		while (size > 0) {
			page = kvirt_to_pa(adr);
 			mem_map_reserve(virt_to_page(__va(page)));
			adr += PAGE_SIZE;
			size -= PAGE_SIZE;
		}
	}
	return mem;
}

void rvfree(void *mem, signed long size)
{
	unsigned long adr, page;

	if (mem) {
		adr = (unsigned long)mem;
		while (size > 0) {
			page = kvirt_to_pa(adr);
 			mem_map_unreserve(virt_to_page(__va(page)));
			adr += PAGE_SIZE;
			size -= PAGE_SIZE;
		}
		vfree(mem);
	}
}

/*
 * Overrides for Emacs so that we follow Linus's tabbing style.
 * ---------------------------------------------------------------------------
 * Local variables:
 * c-basic-offset: 8
 * End:
 */
