/*
 * Video4Linux functions for Voodoo3 3500 TV drivers.
 *
 * $Id: voodoo-v4l2.c,v 1.1.1.1 2004/05/23 04:46:53 scottsman Exp $
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
  */

#define __NO_VERSION__
#include <linux/kernel.h>
#include <linux/version.h>
#include <linux/module.h>
#include <linux/interrupt.h>
#include <linux/init.h>
#include <linux/pci.h>
#include <linux/i2c.h>
#include <linux/videodev.h>
#include <linux/video_decoder.h>
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
#include <media/audiochip.h>
#include <media/tuner.h>
#else
#include "../linux/drivers/media/video/i2c-compat.h"
#include "../linux/drivers/media/video/audiochip.h"
#include "../linux/drivers/media/video/tuner.h"
#endif
#include "voodoo.h"

//#ifndef I2C_DRIVERID_VPX3224
//#include "vpx-common.h"
//#endif

#if LINUX_VERSION_CODE <= KERNEL_VERSION(2,5,0)
#define iminor(inode) minor(inode->i_rdev)
#endif

/* External variables: Data struct */
extern struct voodoo_data *voodoo;
extern struct video_capture video_standard[];

static struct video_device v3tv_vdev_radio;
static struct video_device v3tv_vdev_video;

static atomic_t users = {
      counter:0
};				/* Common for radio and TV, only one at a time possible */

/* Module parameters */
extern int debug;

extern int video_nr;
extern int radio_nr;
extern int vbi_nr;

static const char *v4l1_ioctls[] = {
	"?", "CGAP", "GCHAN", "SCHAN", "GTUNER", "STUNER", "GPICT", "SPICT",
	"CCAPTURE", "GWIN", "SWIN", "GFBUF", "SFBUF", "KEY", "GFREQ",
	"SFREQ", "GAUDIO", "SAUDIO", "SYNC", "MCAPTURE", "GMBUF", "GUNIT",
	"GCAPTURE", "SCAPTURE", "SPLAYMODE", "SWRITEMODE", "GPLAYINFO",
	"SMICROCODE", "GVBIFMT", "SVBIFMT" };
#define V4L1_IOCTLS ARRAY_SIZE(v4l1_ioctls)

/* ----------------------------------------------------------------------- */

static struct v4l2_input v3tv_radio_input = {
	.name	=	"Voodoo3 TV 3500 FM Tuner",
	.type	=	V4L2_INPUT_TYPE_TUNER,
	.audioset =	1,
	.tuner	=	1
};

/* ----------------------------------------------------------------------- */

static const struct v4l2_queryctrl no_ctl = {
	.name  = "42",
	.flags = V4L2_CTRL_FLAG_DISABLED,
};
static const struct v4l2_queryctrl v3tv_ctls[] = {
	{
		.id            = V4L2_CID_AUDIO_MUTE,
		.name          = "Mute",
		.minimum       = 0,
		.maximum       = 1,
		.type          = V4L2_CTRL_TYPE_BOOLEAN,
	},{
		.id            = V4L2_CID_AUDIO_VOLUME,
		.name          = "Volume",
		.minimum       = 0,
		.maximum       = 65535,
		.step          = 65535/100,
		.default_value = 65535,
		.type          = V4L2_CTRL_TYPE_INTEGER,
	},{
		.id            = V4L2_CID_AUDIO_BALANCE,
		.name          = "Balance",
		.minimum       = 0,
		.maximum       = 65535,
		.step          = 65535/100,
		.default_value = 32768,
		.type          = V4L2_CTRL_TYPE_INTEGER,
	},{
		.id            = V4L2_CID_AUDIO_BASS,
		.name          = "Bass",
		.minimum       = 0,
		.maximum       = 65535,
		.step          = 65535/100,
		.default_value = 32768,
		.type          = V4L2_CTRL_TYPE_INTEGER,
	},{
		.id            = V4L2_CID_AUDIO_TREBLE,
		.name          = "Treble",
		.minimum       = 0,
		.maximum       = 65535,
		.step          = 65535/100,
		.default_value = 32768,
		.type          = V4L2_CTRL_TYPE_INTEGER,
	}
};

const int V3TV_CTLS = ARRAY_SIZE(v3tv_ctls);

static int get_control(struct voodoo_data *voodoo, struct v4l2_control *c)
{
	struct video_audio va;
	int i;
	
	for (i = 0; i < V3TV_CTLS; i++)
		if (v3tv_ctls[i].id == c->id)
			break;
	if (i == V3TV_CTLS)
		return -EINVAL;
	if (i >= 4 && i <= 10) {
		memset(&va,0,sizeof(va));
		i2c_clients_command(voodoo->voodoo_i2c_adapter, VIDIOCGAUDIO, &va);
	}
	switch (c->id) {
/*
	case V4L2_CID_BRIGHTNESS:
		c->value = voodoo->bright;
		break;
	case V4L2_CID_HUE:
		c->value = voodoo->hue;
		break;
	case V4L2_CID_CONTRAST:
		c->value = voodoo->contrast;
		break;
	case V4L2_CID_SATURATION:
		c->value = voodoo->saturation;
		break;
*/
	case V4L2_CID_AUDIO_MUTE:
		c->value = (VIDEO_AUDIO_MUTE & va.flags) ? 1 : 0;
		break;
	case V4L2_CID_AUDIO_VOLUME:
		c->value = va.volume;
		break;
	case V4L2_CID_AUDIO_BALANCE:
		c->value = va.balance;
		break;
	case V4L2_CID_AUDIO_BASS:
		c->value = va.bass;
		break;
	case V4L2_CID_AUDIO_TREBLE:
		c->value = va.treble;
		break;
//	case V4L2_CID_AUDIO_LOUDNESS:
//		c->value = voodoo->loudness;
//		break;
	default:
		return -EINVAL;
	}
	return 0;
}

static int set_control(struct voodoo_data *voodoo, struct v4l2_control *c)
{
	struct video_audio va;
	int i;

	for (i = 0; i < V3TV_CTLS; i++)
		if (v3tv_ctls[i].id == c->id)
			break;
	if (i == V3TV_CTLS)
		return -EINVAL;
	if (i >= 4 && i <= 10) {
		memset(&va,0,sizeof(va));
		i2c_clients_command(voodoo->voodoo_i2c_adapter, VIDIOCGAUDIO, &va);
	}
	switch (c->id) {
/*
	case V4L2_CID_BRIGHTNESS:
		bt848_bright(voodoo,c->value);
		break;
	case V4L2_CID_HUE:
		bt848_hue(voodoo,c->value);
		break;
	case V4L2_CID_CONTRAST:
		bt848_contrast(voodoo,c->value);
		break;
	case V4L2_CID_SATURATION:
		bt848_sat(voodoo,c->value);
		break;
*/
	case V4L2_CID_AUDIO_MUTE:
		if (c->value)
			va.flags |= VIDEO_AUDIO_MUTE;
		else
			va.flags &= ~VIDEO_AUDIO_MUTE;
		break;

	case V4L2_CID_AUDIO_VOLUME:
		va.volume = c->value;
		break;
	case V4L2_CID_AUDIO_BALANCE:
		va.balance = c->value;
		break;
	case V4L2_CID_AUDIO_BASS:
		va.bass = c->value;
		break;
	case V4L2_CID_AUDIO_TREBLE:
		va.treble = c->value;
		break;
	default:
		return -EINVAL;
	}
	if (i >= 4 && i <= 10)
		i2c_clients_command(voodoo->voodoo_i2c_adapter,
					 VIDIOCSAUDIO, &va);
	return 0;
}

static void
i2c_vidiocschan(struct voodoo_data *voodoo)
{
	struct video_channel c;

	memset(&c,0,sizeof(c));
	c.norm    = voodoo->norm;
	c.channel = voodoo->input;
	i2c_clients_command(voodoo->voodoo_i2c_adapter, VIDIOCSCHAN, &c);
}

/* ----------------------------------------------------------------------- */

static int v3tv_radio_do_ioctl(struct inode *inode, struct file *file,
			      unsigned int cmd, void *arg)
{
	struct voodoo_data *voodoo_radio = file->private_data;

	switch (cmd) {
	case VIDIOC_ENUMINPUT:
	{
		struct v4l2_input *i = arg;
		
		printk("v3tv: VIDIOC_ENUMINPUT %d.\n",i->index);
		if( i->index != 0 ) {
			return -EINVAL;
		}
		memcpy(i, &v3tv_radio_input, sizeof(struct v4l2_input));

		return 0;
	}
	case VIDIOC_QUERYCTRL:
	{
		struct v4l2_queryctrl *c = arg;
		int i;

		printk("v3tv: VIDIOC_QUERYCTRL\n");
		if (c->id <  V4L2_CID_BASE ||
		     c->id >= V4L2_CID_LASTP1) /* 'if (( ...' &&
		    (c->id <  V4L2_CID_PRIVATE_BASE ||
		     c->id >= V4L2_CID_PRIVATE_LASTP1)) */
			return -EINVAL;
		for (i = 0; i < V3TV_CTLS; i++)
			if (v3tv_ctls[i].id == c->id)
				break;
		if (i == V3TV_CTLS) {
			*c = no_ctl;
			return 0;
		}
		*c = v3tv_ctls[i];
		if (i >= 4 && i <= 8) {
			struct video_audio va;
			memset(&va,0,sizeof(va));
			i2c_clients_command(voodoo_radio->voodoo_i2c_adapter,
						VIDIOCGAUDIO, &va);
			switch (v3tv_ctls[i].id) {
			case V4L2_CID_AUDIO_VOLUME:
				if (!(va.flags & VIDEO_AUDIO_VOLUME))
					*c = no_ctl;
				break;
			case V4L2_CID_AUDIO_BALANCE:
				if (!(va.flags & VIDEO_AUDIO_BALANCE))
					*c = no_ctl;
				break;
			case V4L2_CID_AUDIO_BASS:
				if (!(va.flags & VIDEO_AUDIO_BASS))
					*c = no_ctl;
				break;
			case V4L2_CID_AUDIO_TREBLE:
				if (!(va.flags & VIDEO_AUDIO_TREBLE))
					*c = no_ctl;
				break;
			}
		}
		return 0;
	}
	case VIDIOC_G_CTRL:
	{
		printk("v3tv: VIDIOC_G_CTRL\n");
		return get_control(voodoo_radio,arg);
	}
	case VIDIOC_S_CTRL:
	{
		printk("v3tv: VIDIOC_S_CTRL\n");
		return set_control(voodoo_radio,arg);
	}
	case VIDIOC_QUERYCAP:
	{
		printk("v3tv: VIDIOC_QUERYCAP\n");
		struct v4l2_capability *cap = arg;

                strcpy(cap->driver,"v3tv radio");
                strcpy(cap->card,"Voodoo3 TV 3500");
		strcpy(cap->bus_info,"AGP Slot 0");
		cap->version = KERNEL_VERSION(0,1,0);
		cap->capabilities =
			V4L2_CAP_TUNER |
			V4L2_CAP_RADIO;
		return 0;
	}
	case VIDIOC_G_TUNER:
	{
		printk("v3tv: VIDIOC_G_TUNER\n");
		struct video_tuner vt;
		struct video_audio va;
		struct v4l2_tuner *t = arg;

		if (0 != t->index)
			return -EINVAL;
//		down(&voodoo_radio->lock);
		memset(t, 0, sizeof(struct v4l2_tuner));
		if (VOODOO3_MODEL_NTSC == voodoo_radio->model)
			strcpy(t->name, "FM Radio");
		else
			strcpy(t->name, "TV Tweaked to FM Radio");
		t->type       = V4L2_TUNER_RADIO;
		/* japan:		76.0 MHz -  89.9 MHz
		 * western europe:	87.5 MHz - 108.0 MHz
		 * russia:		65.0 MHz - 108.0 MHz */
		t->rangelow  = 87 * 16;
		t->rangehigh  = 108 * 16;
		t->capability = //V4L2_TUNER_CAP_LOW |
				V4L2_TUNER_CAP_NORM |
				V4L2_TUNER_CAP_STEREO;
//		t->rxsubchans = V4L2_TUNER_SUB_MONO;
		i2c_clients_command(voodoo_radio->voodoo_i2c_adapter,
				    VIDIOCGTUNER, &vt);
		t->signal = vt.signal;

		/* Hmmm ... */
		/* FIXME: fill capability+audmode */
		memset(&va, 0, sizeof(struct video_audio));
		i2c_clients_command(voodoo_radio->voodoo_i2c_adapter,
				    VIDIOCGAUDIO, &va);
		if(va.mode & VIDEO_SOUND_STEREO) {
			t->audmode     = V4L2_TUNER_MODE_STEREO;
//			t->rxsubchans |= V4L2_TUNER_SUB_STEREO;
		}
		if(va.mode & VIDEO_SOUND_LANG1) {
			t->audmode    = V4L2_TUNER_MODE_LANG1;
//			t->rxsubchans = V4L2_TUNER_SUB_LANG1
//				| V4L2_TUNER_SUB_LANG2;
		}
//		up(&voodoo_radio->lock);
		return 0;
	}
	case VIDIOC_S_TUNER:
	{
		printk("v3tv: VIDIOC_S_TUNER\n");
		struct v4l2_tuner *t = arg;

		if (0 != t->index)
			return -EINVAL;
//		down(&voodoo_radio->lock);
		{
			struct video_audio va;
			memset(&va, 0, sizeof(struct video_audio));
			i2c_clients_command(voodoo_radio->voodoo_i2c_adapter,
						VIDIOCGAUDIO, &va);
			if (t->audmode == V4L2_TUNER_MODE_MONO)
				va.mode = VIDEO_SOUND_MONO;
			else if (t->audmode == V4L2_TUNER_MODE_STEREO)
				va.mode = VIDEO_SOUND_STEREO;
			else if (t->audmode == V4L2_TUNER_MODE_LANG1)
				va.mode = VIDEO_SOUND_LANG1;
			else if (t->audmode == V4L2_TUNER_MODE_LANG2)
				va.mode = VIDEO_SOUND_LANG2;
			i2c_clients_command(voodoo_radio->voodoo_i2c_adapter,
						VIDIOCSAUDIO, &va);
		}
//		up(&voodoo_radio->lock);
		return 0;
	}
	case VIDIOC_G_FREQUENCY:
	{
		printk("v3tv: VIDIOC_G_FREQUENCY\n");
		struct v4l2_frequency *f = arg;

		memset(f,0,sizeof(struct v4l2_frequency));
		f->type = V4L2_TUNER_RADIO;
		f->frequency = voodoo_radio->freq;
		if (copy_to_user(arg, &f, sizeof(struct v4l2_frequency)))
			return -EFAULT;
		return 0;
	}
	case VIDIOC_S_FREQUENCY:
	{
		printk("v3tv: VIDIOC_S_FREQUENCY\n");
		struct v4l2_frequency *f = arg;
		int zero = 0;

		if (unlikely(f->tuner != 0))
			return -EINVAL;
		if (unlikely(f->type != V4L2_TUNER_RADIO))
			return -EINVAL;
#if 0		
		if (f->frequency < 45 * 16000) {
//		if (f->frequency < 696000) {
			printk ("v3tv: Radio program does not support the VIDEO_TUNER_LOW flag\n");
			printk("v3tv: Multiplying by 1000 to get %d\n", 1000 * (f->frequency / 16));
			f->frequency *= 1000;
		}
		voodoo_radio->freq = f->frequency/1000;
#endif
		voodoo_radio->freq = f->frequency;
		printk("v3tv: radio frequency %ld\n", voodoo_radio->freq);

//		down(&voodoo_radio->lock);
		i2c_clients_command(voodoo_radio->voodoo_i2c_adapter,
				    AUDC_SET_RADIO, &zero);
		i2c_clients_command(voodoo_radio->voodoo_i2c_adapter,
				    VIDIOCSFREQ, &voodoo_radio->freq);

//		up(&voodoo_radio->lock);
		return 0;
	}
#if 0 // obsolete v4l1 ioctl's
	case VIDIOCGCAP:
		{
			struct video_capability vcap;
			vcap.type = v3tv_vdev_radio.type;
			vcap.channels = 1;
			vcap.audios = 1;
			vcap.maxwidth = 0;
			vcap.minwidth = 0;
			vcap.maxheight = 0;
			vcap.minheight = 0;
			strcpy(vcap.name, v3tv_vdev_radio.name);
			if (copy_to_user(arg, &vcap, sizeof(vcap)))
				return -EFAULT;
			if (debug)
				printk("v3tv: Getting radio capabilities.\n");
			break;
		}
	case VIDIOCGTUNER:
		{
			struct video_tuner vtune;
			if (copy_from_user(&vtune, arg, sizeof(vtune)))
				return -EFAULT;
			if (vtune.tuner)
				return -EINVAL;
			vtune.rangelow = 87 * 16000;
			vtune.rangehigh = 108 * 16000;
			vtune.flags = VIDEO_TUNER_LOW;
			vtune.mode = VIDEO_MODE_AUTO;
			vtune.signal = 0xFFFF;	/* FIXME: We should really check the signal */
			if (voodoo_radio->model == VOODOO3_MODEL_NTSC)
				strcpy(vtune.name, "FM");
			else
				strcpy(vtune.name, "TV Tweaked to FM");

			if (copy_to_user(arg, &vtune, sizeof(vtune)))
				return -EFAULT;
			if (debug)
				printk
				    ("v3tv: Getting radio tuner capabilities.\n");
			break;
		}
	case VIDIOCSTUNER:
		{
			struct video_tuner vtune;
			if (copy_from_user(&vtune, arg, sizeof(vtune)))
				return -EFAULT;
			if (vtune.tuner)
				return -EINVAL;
			if (debug)
				printk("v3tv: Selecting radio tuner 0.\n");
			break;
		}
	case VIDIOCGFREQ:
		{
			if (copy_to_user(arg, &voodoo_radio->freq, sizeof(u32)))
				return -EFAULT;
			break;
		}
	case VIDIOCSFREQ:
		{
			u32 freq;
			int zero = 0;

			if (copy_from_user(&freq, arg, sizeof(u32)))
				return -EFAULT;
			if (freq < 45 * 16000) {
				printk ("v3tv: Radio program does not support the VIDEO_TUNER_LOW flag\n");
				printk("v3tv: Multiplying by 1000 to get %d\n", 1000 * (freq / 16));
				freq *= 1000;
			}

			voodoo_radio->freq = freq/1000;
			/* FIXME. This isn't right!
			 * We should really use the radio mode of the MSP processor,
			 * but apparently the PAL version of the Voodoo3 doesn't
			 * support this. So we use mono FM TV-sound in stead.
			 *
			 * FIXME again. The radio mutes when we change frequency.
			 *
			 * FIXME x 3: Force tweaked radio to use FM mono 5.5. MHz.
			 */
			i2c_clients_command(voodoo->voodoo_i2c_adapter,
						AUDC_SET_RADIO, &zero);
			i2c_clients_command(voodoo->voodoo_i2c_adapter,
						VIDIOCSFREQ, &zero);

			i2c_clients_command(voodoo->voodoo_i2c_adapter,
						AUDC_SET_RADIO, &zero);
			i2c_clients_command(voodoo->voodoo_i2c_adapter, VIDIOCSFREQ,
						&voodoo_radio->freq);

			break;
		}
	case VIDIOCGAUDIO:
		{
			struct video_audio va;

			if (copy_from_user(&va, arg, sizeof(va)))
				return -EFAULT;
			if (va.audio) {
				if (debug)
					printk
					    ("v3tv: FIXME: VIDIOCGAUDIO: Audio=%d\n",
					     va.audio);
				va.audio = 0;
				// return -EINVAL;
			}
			i2c_clients_command(voodoo->voodoo_i2c_adapter,
						VIDIOCGAUDIO, &va);

			va.mode = VIDEO_SOUND_MONO;
			va.flags |= VIDEO_AUDIO_MUTABLE | VIDEO_AUDIO_VOLUME
			    | VIDEO_AUDIO_BASS | VIDEO_AUDIO_TREBLE;
			// FIXME: Add support for these.
			if (copy_to_user(arg, &va, sizeof(va)))
				return -EFAULT;

			break;
		}
	case VIDIOCSAUDIO:
		{
			struct video_audio va;

			if (copy_from_user(&va, arg, sizeof(va)))
				return -EFAULT;
			if (va.audio) {
				if (debug)
					printk
					    ("v3tv: FIXME: VIDIOSAUDIO: Audio=%d\n",
					     va.audio);
				// FIXME. This should really be checked. 
				// If only user programs respected it :-(
				return -EINVAL;
			}
			// va.mode = 0;
			// FIXME. Force msp3400 _not_ to change stereo mode.
			// FIXME. See that muting is handled properly.
			i2c_clients_command(voodoo->voodoo_i2c_adapter,
						VIDIOCSAUDIO, &va);

			break;
		}
#endif
	default:
	    if (debug > 1) {
		switch (_IOC_TYPE(cmd)) {
		case 'v':
			printk("v3tv: unsuported ioctl 0x%x (v4l1, VIDIOC%s)\n",
			       cmd, (_IOC_NR(cmd) < V4L1_IOCTLS) ?
			       v4l1_ioctls[_IOC_NR(cmd)] : "???");
			break;
		case 'V':
			printk("v3tv: unsupported ioctl 0x%x (v4l2, %s)\n",
			       cmd,  v4l2_ioctl_names[_IOC_NR(cmd)]);
			break;
		default:
			printk("v3tv: unknown ioctl 0x%x (\?\?\?)\n",
			       cmd);
		}
	    }
	    return -ENOIOCTLCMD;
	}
	return 0;
}

static int v3tv_radio_ioctl(struct inode *inode, struct file *file,
			   unsigned int cmd, unsigned long arg)
{
	return video_usercopy(inode, file, cmd, arg, v3tv_radio_do_ioctl);
}

static int v3tv_radio_open(struct inode *inode, struct file *file)
{
	int minor = iminor(inode);

	if (debug)
		printk(KERN_DEBUG "v3tv: open minor=%d\n",minor);

	if (voodoo->radio_dev->minor != minor)
		return -ENODEV;

	if (debug)
		printk(KERN_DEBUG "v3tv: open called (radio)\n");

//	down(&voodoo->lock);
	if (users.counter) {
		printk("v3tv: Radio device busy\n");
//		up(&voodoo->lock);
		return -EBUSY;
	}
	atomic_inc(&users);
	file->private_data = voodoo;

	voodoo->input = 0;
	i2c_vidiocschan(voodoo);

	i2c_clients_command(voodoo->voodoo_i2c_adapter,
				TUNER_SET_TYPE, &voodoo->tuner);
        i2c_clients_command(voodoo->voodoo_i2c_adapter,
				AUDC_SET_RADIO, &voodoo->tuner);
//	up(&voodoo->lock);

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
	try_module_get(THIS_MODULE);
#else
	MOD_INC_USE_COUNT;
#endif
	printk("v3tv: Radio device opened.\n");
	return 0;
}

static int v3tv_radio_release(struct inode *inode, struct file *file)
{
/*	Use these in i2c_clients_command() in future */
//      struct video_device *dev = video_devdata(file);
//      struct voodoo_data *voodoo_radio = dev->priv;

	atomic_dec(&users);

	file->private_data = NULL;

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
	module_put(THIS_MODULE);
#else
	MOD_DEC_USE_COUNT;
#endif

	printk("v3tv: Radio device closed.\n");

	return 0;
}

// static struct wait_queue *capture_wait;
static volatile int capture_ready = 0;

static volatile int irq_counter = 0;
static volatile int idles = 0;
static volatile int timeout = 0;

#define DEBUG_LPT 0x378
#include <asm/delay.h>

static void v3tv_irq_printk(long tempReg)
{
	/* 1111.0000.0000.0000.0001.1111.1011.1111 f0001fbf * page 188
	   1f    * [4]  x1f  = FIFO emptyi

	   0     * [5]  x20  = FIFO busy
	   1     * [6]  x40  = Vertical retrace inactive (1)
	   0     * [7]  x80  = FBI grph eng busy

	   0     * [8]  x100 = TREX busy
	   1     * [9]  x200 = Avenger busy
	   1     * [10] x400 = 2D busy
	   1     * [11] x800 = FIFO 0 busy

	   0     * [12] x1000 = FIFO 1 busy
	   * [27:13] reserved

	   * [30:28] Swap Buffers Pending
	   * [31]    PCI Interupt Generated */

	printk(KERN_WARNING "v3tv: IRQ: Timeout %d : %d : %d\n",
	       timeout, idles, irq_counter);
	printk(KERN_WARNING "v3tv: IRQ: V3REG_status: 0x%lx FIFO: 0x%lx%s%s\n",
	       tempReg, tempReg & 0x1f,
	       (tempReg & 0x20) ? "; FIFO busy" : "",
	       (tempReg & 0x40) ? "; Vertical retrace inactive" :
	       "; Vertical retrace active");
	printk(KERN_WARNING "v3tv: IRQ: %s%s%s%s%s%s\n",
	       (tempReg & 0x80) ? "; FBI grph eng busy" : "",
	       (tempReg & 0x100) ? "; TREX busy" : "",
	       (tempReg & 0x200) ? "; Avenger busy" : "",
	       (tempReg & 0x400) ? "; 2D busy" : "",
	       (tempReg & 0x800) ? "; FIFO 0 busy" : "",
	       (tempReg & 0x1000) ? "; FIFO 1 busy" : "");
	printk(KERN_WARNING "v3tv: IRQ: Swap Buffers Pending: 0x%lx; %s\n",
	       (tempReg >> 28) & 7,
	       (tempReg & 80000000) ? "PCI Interupt Generated" : "");
}

// 2.4.23       void (*handler)(int, void *, struct pt_regs *),
// 2.6.1        irqreturn_t (*handler)(int, void *, struct pt_regs *),

static irqreturn_t v3tv_video_irq(int irq, void *dev_id, struct pt_regs *regs)
{
	long tempReg;
	idles = 0;
	timeout = 0;

	capture_ready = 1;

	irq_counter++;
	if ((irq_counter % 10000) == 0)
		printk(KERN_WARNING "v3tv: IRQ: %d interrupts\n", irq_counter);

	v3tv_regwrite(V3REG_command, 0x100);	/* Immediate NOP */
//      voodoo3_regwrite (V3REG_launchArea, 0);
//      voodoo3_regwrite (V3REG_IO_BASE_2 + 0x120, 0);

	while (((tempReg = v3tv_regread(V3REG_status)) & 0x7BF) != 0x1F) {
		timeout++;
		udelay(1000);
		if (timeout < 10)
			continue;

		v3tv_irq_printk(tempReg);

		timeout = 0;
		idles++;
		if (idles < 3)
			continue;
		else {
/*
	The Voodoo3 may be in an unstable state, so we try to bailout gracefully.
	First disable the VMI Interupt, so we don't get here again.
*/
			capture_ready = 0;

			static struct conf_struct Disable_VMI_Interrupt[] =
			      { {V3REG_intrCtrl, 0, 21, 21}, /* VMI Interrupt disable */
				{V3REG_intrCtrl, 0, 23, 23}, /* clear VMI Interrupt generated */
				{V3REG_intrCtrl, 1, 31, 31} }; /* clear PCI interrupt active */
			v3tv_conf(Disable_VMI_Interrupt, CONF_SIZE(Disable_VMI_Interrupt));

/* We're not supposed to call free_irq from interrupt context, leave it till _close */
/*			free_irq(voodoo->dev->irq, voodoo->dev);
			voodoo->initialized &= ~I_irq;

			voodoo->decoder_enabled = 0;
		        i2c_clients_command(voodoo->voodoo_i2c_adapter, 
				DECODER_ENABLE_OUTPUT, &voodoo->decoder_enabled);

		        struct video_audio va;

		        i2c_clients_command(voodoo->voodoo_i2c_adapter, VIDIOCGAUDIO, &va);

		        va.volume = 0;
		        va.balance = 32768;
		        i2c_clients_command(voodoo->voodoo_i2c_adapter, VIDIOCSAUDIO, &va);
*/

/* Optionaly disable Video In. */ 
//			static struct conf_struct VMI_Disable[] =
//				{ {V3REG_vidInFormat, 0, 14, 14},
//				  {V3REG_vidSerialParallelPort, 0, 0, 0}
//				};
//			v3tv_conf(VMI_Disable, CONF_SIZE(VMI_Disable));
			break;
		}
	}

	tempReg = v3tv_regread(V3REG_intrCtrl);
	if ((tempReg & (1 << 23)) || (capture_ready == 0)) {
		tempReg &= ~(1 << 23);
		tempReg |= (1 << 31);
		v3tv_regwrite(V3REG_intrCtrl, tempReg);
	}
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,5,0)
	return capture_ready;
#else
	return;
#endif
}

void v3tv_set_palette(struct video_picture *picture_settings)
{
	switch ((v3tv_regread(V3REG_dstFormat) >> 16) & 0x07) {
	case 4:		/* 24bpp */
	case 5:		/* 32 bpp - tdfxfb runs at 32 ?? */
		{
			picture_settings->palette = VIDEO_PALETTE_RGB24;
			picture_settings->depth = 24;
			if (debug)
				printk
				    ("v3tv: palette: VIDEO_PALETTE_RGB24 = 24 bpp\n");
			break;
		}
	case 1:		/* 8bpp - we are making an asumption so things can go forward */
	case 3:		/* 16bpp */
	default:
		{
			picture_settings->palette = VIDEO_PALETTE_RGB565;
			picture_settings->depth = 16;
			if (debug)
				printk
				    ("v3tv: palette: VIDEO_PALETTE_RGB565 = 16 bpp\n");
			break;
		}
	}
}

static int v3tv_video_open(struct inode *inode, struct file *file)
{
	int result;
	int minor = iminor(inode);
	enum v4l2_buf_type type = 0;

	if (debug)
		printk(KERN_DEBUG "v3tv: open minor=%d\n",minor);

	if (voodoo->vbi_dev && 
	    voodoo->vbi_dev->minor == minor)
		type = V4L2_BUF_TYPE_VBI_CAPTURE;
	if (voodoo->video_dev &&
	    voodoo->video_dev->minor == minor)
		type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
	if (!type)
		return -ENODEV;

	if (debug)
		printk(KERN_DEBUG "v3tv: open called (type=%s)\n",
			v4l2_type_names[type]);

	/* Check if we are already in use */
	if (users.counter) {
		printk("v3tv: Video or radio device busy\n");
		return -EBUSY;
	}

	/* Get irq. Interrupt is shared and disabled while processing */
	result = request_irq(voodoo->dev->irq, v3tv_video_irq,
			     SA_SHIRQ | SA_INTERRUPT, "v3tv", voodoo->dev);

	if (result == -EINVAL) {
		printk(KERN_ERR "v3tv: Bad irq number or handler\n");
		return result;
	}

	if (result == -EBUSY) {
		printk(KERN_ERR
		       "v3tv: IRQ %d busy, change your PnP config in BIOS\n",
		       voodoo->dev->irq);
		return result;
	}

	if (result < 0) {
		printk(KERN_ERR
		       "v3tv: Unknown error, code %d when getting interrupt\n",
		       result);
		return -EINVAL;	/* Return something that the kernel knows about, or it might Oops */
	}

	atomic_inc(&users);
	voodoo->initialized |= I_irq;

	i2c_clients_command(voodoo->voodoo_i2c_adapter, TUNER_SET_TYPE, &voodoo->tuner);

//	struct video_audio va;

//	i2c_clients_command(voodoo->voodoo_i2c_adapter, VIDIOCGAUDIO, &va);
 
//	va.flags = VIDEO_AUDIO_MUTE;
//	va.volume = 65535;
//	va.balance = 32768;	/* FIXME. This shouldn't be necessary */
//	i2c_clients_command(voodoo->voodoo_i2c_adapter, VIDIOCSAUDIO, &va);

	static struct conf_struct VMI_Enable[] =
		{ {V3REG_vidInFormat, 1, 14, 14}, /* VMI interafce enable */
	{V3REG_vidSerialParallelPort, 1, 0, 0} }; /* VMI parallel host interface enable */
	v3tv_conf(VMI_Enable, CONF_SIZE(VMI_Enable));

	v3tv_set_palette(&voodoo->picture_settings);
	i2c_clients_command(voodoo->voodoo_i2c_adapter, DECODER_SET_PICTURE,
				&voodoo->picture_settings);

	voodoo->decoder_enabled = 1;
	i2c_clients_command(voodoo->voodoo_i2c_adapter, DECODER_ENABLE_OUTPUT,
				&voodoo->decoder_enabled);

	file->private_data = voodoo;
	if (debug)
		printk("v3tv: video device opened.\n");
	return 0;
}

static int v3tv_video_release(struct inode *inode, struct file *file)
{
	struct voodoo_data *voodoo_tv = file->private_data;

	if (voodoo_tv->initialized & I_irq) {
 	free_irq(voodoo_tv->dev->irq, voodoo_tv->dev);
		voodoo_tv->initialized &= ~I_irq;
	}

	voodoo_tv->decoder_enabled = 0;
	i2c_clients_command(voodoo_tv->voodoo_i2c_adapter, DECODER_ENABLE_OUTPUT,
				&voodoo_tv->decoder_enabled);

//	struct video_audio va;

//	i2c_clients_command(voodoo->voodoo_i2c_adapter, VIDIOCGAUDIO, &va);

//	va.volume = 0;
//	va.balance = 32768;	/* FIXME. This shouldn't be necessary */
//	i2c_clients_command(voodoo->voodoo_i2c_adapter, VIDIOCSAUDIO, &va);

	static struct conf_struct VMI_Disable[] =
		{ {V3REG_vidInFormat, 0, 14, 14}, /* VMI interafce disable */
		  {V3REG_vidSerialParallelPort, 0, 0, 0} }; /* VMI parallel host interface disable */

	v3tv_conf(VMI_Disable, CONF_SIZE(VMI_Disable));

	atomic_dec(&users);

	if (debug)
		printk("v3tv: video device closed\n");

	return 0;
}

/* No poll now 

static int v4l_voodoo_poll (struct video_device*dev,
		structfile *file, struct poll_table *wait)
{
	poll_wait(file, &capture_wait, wait);

	if(capture_read)
		return POLLIN|POLLRDNORM;
	return 0;
}
*/

static ssize_t v3tv_video_read(struct file *file, char *data,
			       size_t count, loff_t * ppos)
{
	struct voodoo_data *voodoo_tv = file->private_data;

	v3tv_capture_frame(voodoo_tv);
#if 0
	if(copy_to_user(buf, voodoo_2ddtv->fb_base +
			(unsigned long) voodoo_tv->captureBuf, count))

	if (debug)
		printk( "v3tv: read count=%d type=%s "
			"fb_base=0x%p captureBuf=0x%lx\n",
			(int)count, v4l2_type_names[fh->type], voodoo_tv->fb_base,
			(unsigned long) voodoo_tv->captureBuf);
	return count;
#endif
	return 0;
}

#ifdef JustForRef
static ssize_t bttv_read(struct file *file, char *data,
			 size_t count, loff_t *ppos)
{
	struct bttv_fh *fh = file->private_data;
	int retval = 0;

	if (fh->btv->errors)
		bttv_reinit_bt848(fh->btv);
	dprintk("bttv%d: read count=%d type=%s\n",
		fh->btv->c.nr,(int)count,v4l2_type_names[fh->type]);

	switch (fh->type) {
	case V4L2_BUF_TYPE_VIDEO_CAPTURE:
		if (locked_btres(fh->btv,RESOURCE_VIDEO))
			return -EBUSY;
		retval = videobuf_read_one(file, &fh->cap, data, count, ppos);
		break;
	case V4L2_BUF_TYPE_VBI_CAPTURE:
		if (!check_alloc_btres(fh->btv,fh,RESOURCE_VBI))
			return -EBUSY;
		retval = videobuf_read_stream(file, &fh->vbi, data, count, ppos, 1);
		break;
	default:
		BUG();
	}
	return retval;
}
#endif

/*
static long v3tv_voodoo_read(struct video_device *dev, char *buf,unsigned long count )
{
	struct wait_queue wait = { current, NULL};
	u8 *ptr;
	int len;
	int i;
	add_wait_queue(&capture_wait, &wait);
	while(!capture_ready)
	{
		if(file->flags&O_NDELAY)
		{
			remove_wait_queue(&capture_wait,&wait);
			current->state = TASK_RUNNING;
			return -EWOULDBLOCK;
		}
		if(signal_pending(current))
		{
			remove_wait_queue(&capture_wait,&wait);
			current->state = TASK_RUNNING;
			return -ERESTARTSYS;
		}
		schedule(); 
		current->state = TASK_INTERRUPTIBLE;
	}

	remove_wait_queue(&capture_wait, &wait);

	current->state = TASK_RUNNING;
     	capture_ready = 0;
	ptr = (u8 *)buf;
	len=capture_w * 3 * capture_h;

// 24bit RGB
	if(len>count)
		len = count;
// Doesn't all fit
	for(i=0; i<len; i++)
	{
		put_user(inb (io+IMAGE_DATA),ptr);
		ptr++;
	}
	hardware_restart_capture();
	return i;
}
*/

//int v3tv_voodoo_mmap(struct video_device *dev, const char *adr,
//                    unsigned long size)
int v3tv_voodoo_mmap(struct file *file, struct vm_area_struct *vma)
{
#if 0
	struct video_device *dev = video_devdata(file);
	struct voodoo_data *voodoo_tv = dev->priv;
	int r;

	down(&voodoo_tv->lock);
	r = do_voodoo_mmap(adr, size);
	up(&voodoo_tv->lock);
	return r;
#endif
	return -EINVAL;
}

char *desc_pict_palette[] = {
	[ VIDEO_PALETTE_GREY ]    = "GREY",
	[ VIDEO_PALETTE_HI240 ]   = "HI240",
	[ VIDEO_PALETTE_RGB565 ]  = "RGB565",
	[ VIDEO_PALETTE_RGB24 ]   = "RGB24",
	[ VIDEO_PALETTE_RGB32 ]   = "RGB32",
	[ VIDEO_PALETTE_RGB555 ]  = "RGB555",
	[ VIDEO_PALETTE_YUV422 ]  = "YUV422",
	[ VIDEO_PALETTE_YUYV ]    = "YUYV",
	[ VIDEO_PALETTE_UYVY ]    = "UYVY",
	[ VIDEO_PALETTE_YUV420 ]  = "YUV420",
	[ VIDEO_PALETTE_YUV411 ]  = "YUV411",
	[ VIDEO_PALETTE_RAW ]     = "RAW",
	[ VIDEO_PALETTE_YUV422P ] = "YUV422P",
	[ VIDEO_PALETTE_YUV411P ] = "YUV411P",
	[ VIDEO_PALETTE_YUV420P ] = "YUV420P",
	[ VIDEO_PALETTE_YUV410P ] = "YUV410P",
};

static struct video_channel voodoo_channels[3] = {
	{
		.channel = 0,
		.name = "TV Tuner",
		.tuners = 1,
		.flags = VIDEO_VC_TUNER | VIDEO_VC_AUDIO,
		.type = VIDEO_TYPE_TV
	},
	{
		.channel = 1,
		.name = "Composite input",
		.tuners = 0,
		.flags = VIDEO_VC_AUDIO,
		.type = VIDEO_TYPE_TV
	},
	{
		.channel = 2,
		.name = "S Video input",
		.tuners = 0,
		.flags = VIDEO_VC_AUDIO,
		.type = VIDEO_TYPE_TV
	}
};

static int v3tv_video_do_ioctl(struct inode *inode, struct file *file,
			       unsigned int cmd, void *arg)
{
	struct voodoo_data *voodoo_tv = file->private_data;

	switch (cmd) {
	case VIDIOCGCAP:
		{
			if (debug)
				printk("v3tv: VIDIOCGCAP\n");

			struct video_capability *vcap = arg;

			vcap->type = v3tv_vdev_video.type;
			vcap->channels = 3;
			vcap->audios = 0;	// FIXME: Add audio support.
			vcap->maxwidth = 1280;
			vcap->minwidth = 16;
			vcap->maxheight = 960;
			vcap->minheight = 16;
			strcpy(vcap->name, v3tv_vdev_video.name);

			return 0;
		}
	case VIDIOCGCHAN:
		{
			struct video_channel *vchan = arg;

			if (vchan->channel > 2)
				return -EINVAL;

//			vchan->channel	= voodoo_channels[vchan->channel].channel;
			vchan->tuners	= voodoo_channels[vchan->channel].tuners;
			vchan->flags	= voodoo_channels[vchan->channel].flags;
			vchan->type	= voodoo_channels[vchan->channel].type;
			vchan->norm	= voodoo_tv->norm;
			strcpy(vchan->name, voodoo_channels[vchan->channel].name);

			if (debug)
				printk("v3tv: VIDIOCGCHAN channel=%d %s\n",
					vchan->channel, vchan->name);
			return 0;
		}
	case VIDIOCSCHAN:
		{
			struct video_channel *vchan = arg;
			__u16 sarg = 0;

			if (debug)
				printk
				    ("v3tv: VIDIOCSCHAN: channel=%d, norm=%d\n",
				     vchan->channel, vchan->norm);
			if (vchan->channel > 2)
				return -EINVAL;
			if (vchan->channel == voodoo_tv->input &&
			    vchan->norm == voodoo_tv->norm)
				return 0;
			if ((vchan->norm != VIDEO_MODE_PAL) &&
			    (vchan->norm != VIDEO_MODE_NTSC) &&
			    (vchan->norm != VIDEO_MODE_SECAM))
				return -EINVAL;

			voodoo_tv->channel = vchan->channel;
			voodoo_tv->norm = vchan->norm;
			voodoo_channels[vchan->channel].norm = voodoo_tv->norm;

			i2c_clients_command(voodoo_tv->voodoo_i2c_adapter,
						DECODER_SET_INPUT,
						&vchan->channel);
			i2c_clients_command(voodoo_tv->voodoo_i2c_adapter,
						DECODER_SET_NORM,
						&voodoo_tv->norm);

			v3tv_overlay_setup(voodoo_tv);

			i2c_clients_command(voodoo_tv->voodoo_i2c_adapter,
						VIDIOCSCHAN, &vchan);

			sarg = (voodoo_tv->channel) ? AUDIO_EXTERN_2 : 
							AUDIO_TUNER;
			i2c_clients_command(voodoo_tv->voodoo_i2c_adapter,
						AUDC_SET_INPUT, &sarg);

			return 0;
		}
	case VIDIOCGPICT:
		{
			struct video_picture *pic = arg;

			if (debug)
				printk("v3tv: VIDIOCGPICT\n");

			/* Return picture information here */
			pic->brightness	= voodoo_tv->picture_settings.brightness;
			pic->hue	= voodoo_tv->picture_settings.hue;
			pic->colour	= voodoo_tv->picture_settings.contrast;
			pic->contrast	= voodoo_tv->picture_settings.colour;
			pic->whiteness	= voodoo_tv->picture_settings.whiteness;
			pic->depth	= voodoo_tv->picture_settings.depth;
			pic->palette	= voodoo_tv->picture_settings.palette;

			if (debug)
				printk(KERN_INFO
					"v3tv: VIDIOCGPICT, video_picture-> "
					"brightness: 0x%04x hue: 0x%04x "
					"contrast: 0x%04x colour: 0x%04x "
					"w: 0x%04x\n",
					voodoo_tv->picture_settings.brightness,
					voodoo_tv->picture_settings.hue, 
					voodoo_tv->picture_settings.contrast,
					voodoo_tv->picture_settings.colour,
					voodoo_tv->picture_settings.whiteness);
			if (debug)
				printk(KERN_INFO
				       "v3tv: VIDIOCGPICT, depth = %d, "
					"palette = %d %s\n",
				       voodoo_tv->picture_settings.depth,
				       voodoo_tv->picture_settings.palette,
				       desc_pict_palette[
					voodoo_tv->picture_settings.palette] );
			return 0;
		}
	case VIDIOCSPICT:
		{
			struct video_picture *pic = arg;

			if (debug)
				printk(KERN_INFO
					"v3tv: VIDIOCSPICT, video_picture-> "
					"brightness: 0x%04x hue: 0x%04x "
					"contrast: 0x%04x colour: 0x%04x "
					"w: 0x%04x\n",
					pic->brightness, pic->hue, pic->contrast,
					pic->colour, pic->whiteness);
			if (debug)
				printk(KERN_INFO
				       "v3tv: VIDIOCSPICT, depth = %d, "
					"palette = %d %s\n",
				       pic->depth, pic->palette,
				       desc_pict_palette[pic->palette]);
#if 1
			/* Set picture parameters here */
			if ((pic->palette != VIDEO_PALETTE_RGB565) &&
			    (pic->palette != VIDEO_PALETTE_RGB24)  &&
			    (pic->palette != VIDEO_PALETTE_RGB32)) 
				v3tv_set_palette(pic);
#else
			if (pic->palette != voodoo_tv->picture_settings.palette) {
				pic->palette = voodoo_tv->picture_settings.palette;
				printk(KERN_INFO
				       "v3tv: VIDIOCSPICT: pallete not supported\n");
				return -EINVAL;
			}
#endif
			voodoo_tv->picture_settings.brightness	= pic->brightness;
			voodoo_tv->picture_settings.hue		= pic->hue;
			voodoo_tv->picture_settings.contrast	= pic->contrast;
			voodoo_tv->picture_settings.colour	= pic->colour;
			voodoo_tv->picture_settings.whiteness	= pic->whiteness;
			voodoo_tv->picture_settings.depth	= pic->depth;
//			voodoo_tv->picture_settings.palette	= pic->palette;

			i2c_clients_command(voodoo_tv->voodoo_i2c_adapter,
						DECODER_SET_PICTURE,
						&voodoo_tv->picture_settings);
			return 0;
		}

	case VIDIOCGWIN:
		{
			/* return the current capture window here */
			struct video_window *win = arg;

			memset(win,0,sizeof(*win));
			if (debug)
				(printk("v3tv: VIDIOCGWIN: %d, %d, %d, %d\n",
					voodoo_tv->capture_win.x,
					voodoo_tv->capture_win.y,
					voodoo_tv->capture_win.width,
					voodoo_tv->capture_win.height));

			win->x      = voodoo_tv->capture_win.x;
			win->y      = voodoo_tv->capture_win.y;
			win->width  = voodoo_tv->capture_win.width;
			win->height = voodoo_tv->capture_win.height;
			return 0;
		}

		case VIDIOCSWIN:
		{
			struct video_window *win = arg;
			unsigned long chroma;
			unsigned long R, G, B;

			/* set the capture window here */

			if (win->width > 1280 || win->height > 960)
				return -EINVAL;
			if (win->width < 16 || win->height < 16)
				return -EINVAL;

			/* FIXME: an ugly hack to get chroma working */

			R = (win->chromakey & 0xFF0000) >> 16;
			G = (win->chromakey & 0x00FF00) >> 8;
			B = (win->chromakey & 0x0000FF);

			// Stop the seq_fault Divide by zero.
			if (voodoo_tv->fb.width == 0) {
				printk
				    ("v3tv: Frame buffer width is 0! divide by zero in VIDIOCSWIN");
				return -EINVAL;
			}
			switch (voodoo_tv->fb.bytesperline / voodoo_tv->fb.width) {
			case 1:
				R = R >> 5;
				G = G >> 5;
				B = B >> 6;

				chroma = 0;
				chroma |= R & 0x7;
				chroma |= (G << 3) & 0x38;
				chroma |= (B << 6) & 0xC0;

				break;
			case 2:
				R = R >> 3;
				G = G >> 2;
				B = B >> 3;

				chroma = 0;
				chroma |= ((R << 11) & 0xF800);
				chroma |= ((G << 5) & 0x7E0);
				chroma |= (B & 0x1F);

				break;
			default:
				chroma = win->chromakey;
				break;
			}

			voodoo_tv->capture_win.x	= win->x;
			voodoo_tv->capture_win.y	= win->y;
			voodoo_tv->capture_win.width	= win->width;
			voodoo_tv->capture_win.height	= win->height;
			voodoo_tv->capture_win.chromakey = win->chromakey;
			voodoo_tv->chroma = chroma;

			v3tv_set_chromakey(voodoo_tv);
			v3tv_set_overlay(voodoo_tv);

			if (debug)
				printk
				    ("v3tv: VIDIOCSWIN: %d, %d, %d, %d\n",
				     voodoo_tv->capture_win.x,
				     voodoo_tv->capture_win.y,
				     voodoo_tv->capture_win.width,
				     voodoo_tv->capture_win.height);

			return 0;
		}

	case VIDIOCCAPTURE:
		{
			/* turn on/off overlay here */
			int *on = arg;

			if (debug)
				printk("v3tv: VIDIOCCAPTURE %s\n",
				       (*on) ? "Enable" : "Disable");
/*			if (v)
				if(capture_fb.width == 0 || capture_w == 0)
					return -EINVAL;
*/
			v3tv_overlay(voodoo_tv, *on);
			return 0;
		}

	case VIDIOCSFBUF:
		{
			struct video_buffer *fbuf = arg;

			voodoo_tv->fb.base	= fbuf->base;
			voodoo_tv->fb.height	= fbuf->height;
			voodoo_tv->fb.width	= fbuf->width;
			voodoo_tv->fb.depth	= fbuf->depth;
			voodoo_tv->fb.bytesperline = fbuf->bytesperline;

			if (debug)
				printk
				    ("v3tv: VIDIOCSFBUF: base 0x%p height: %d "
				     "width %d depth %d bytesperline %d\n",
				     voodoo_tv->fb.base, voodoo_tv->fb.height,
				     voodoo_tv->fb.width, voodoo_tv->fb.depth,
				     voodoo_tv->fb.bytesperline);
			return 0;
		}
	case VIDIOCGFBUF:
		{
			struct video_buffer *fbuf = arg;

			fbuf->base	= voodoo_tv->fb.base;
			fbuf->height	= voodoo_tv->fb.height;
			fbuf->width	= voodoo_tv->fb.width;
			fbuf->depth	= voodoo_tv->fb.depth;
			fbuf->bytesperline = voodoo_tv->fb.bytesperline;

			return 0;
		}
	case VIDIOCGTUNER:
		{
			struct video_tuner *tuner_info = arg;
			if (debug)
				printk("v3tv: VIDIOCGTUNER\n");

			if (tuner_info->tuner != 0)
				return -EINVAL;

			strcpy(tuner_info->name, "TV Tuner");
			tuner_info->rangelow  = 0;
			tuner_info->rangehigh = 0x7FFFFFFF;
			tuner_info->flags =
				VIDEO_TUNER_PAL | VIDEO_TUNER_NTSC |
				VIDEO_TUNER_SECAM;
			/* XXX VIDEO_TUNER_STEREO_ON   Tuner is seeing stereo, what is RDS and MBS? */
			tuner_info->mode = voodoo_tv->norm;
			/* Get signal strength from tuner */
			i2c_clients_command(voodoo_tv->voodoo_i2c_adapter,
						VIDIOCGTUNER, tuner_info);

			return 0;
		}
	case VIDIOCSTUNER:
	        {
			struct video_tuner *tuner_info = arg;

			if (tuner_info->tuner) /* Only tuner 0 */
				return -EINVAL;
			voodoo_tv->norm = tuner_info->mode;
			/* Get signal strength from tuner */
			i2c_clients_command(voodoo_tv->voodoo_i2c_adapter,
						VIDIOCGTUNER, tuner_info);
			i2c_clients_command(voodoo_tv->voodoo_i2c_adapter,
						DECODER_SET_NORM,
						&voodoo_tv->norm);
			return 0;
	        }

	case VIDIOCGFREQ:
		{
			if (debug)
				printk("v3tv: VIDIOCGFREQ\n");

			/* get tuner frequency here */
			/* out: int tuner_frequency */
			if (put_user(arg, &voodoo_tv->freq)) {
				if (debug)
					printk("v3tv: VIDIOCGFREQ put_user failed!!\n");
				return -EFAULT;
			}
			return 0;
		}
	case VIDIOCSFREQ:
		{
			unsigned long *freq = arg;
			struct video_channel vchan;

			if (debug)
				printk("v3tv: VIDIOCSFREQ %ld\n", *freq);
			/* set tuner frequency here */
			/* in: int tuner_frequency */

			voodoo_tv->freq = *freq;
			voodoo_tv->channel = 0;	/* Force video input to tuner */
			vchan.channel = 0;

			i2c_clients_command(voodoo_tv->voodoo_i2c_adapter,
						VIDIOCSCHAN, &vchan);
			i2c_clients_command(voodoo_tv->voodoo_i2c_adapter,
						VIDIOCSFREQ, freq);
			return 0;
		}

	case VIDIOCGAUDIO:
		{
			if (debug)
				printk("v3tv: VIDIOCGAUDIO\n");
			i2c_clients_command(voodoo_tv->voodoo_i2c_adapter,
						VIDIOCGAUDIO, (int *)arg);

			return 0;
		}
	case VIDIOCSAUDIO:
		{
			if (debug)
				printk("v3tv: VIDIOCSAUDIO\n");
			i2c_clients_command(voodoo_tv->voodoo_i2c_adapter,
						VIDIOCSAUDIO, (int *)arg);

			return 0;
		}

	case VIDIOCSYNC:
		{
			if (debug)
				printk("v3tv: VIDIOCSYNC not implemented.\n");
			return -EFAULT;
//                      int i;
//                      DECLARE_WAITQUEUE(wait, current);
//
//                      if(copy_from_user((void *)&i,arg,sizeof(int))) {
//				printk("v3tv: VIDIOCSYNC couldn't copy from user.\n");
//                              return -EFAULT;
//			}
//
//                      if (i < 0 || i >= voodoo_tv->gbuffers)
//                              return -EINVAL;
//                      switch (voodoo_tv->gbuf[i].stat)
//                      {
//                              case GBUFFER_UNUSED:
//                                      ret = -EINVAL;
//                                      break;
//                              case GBUFFER_GRABBING:
//                                      add_wait_queue(&voodoo_tv->capq, &wait);
//                                      current->state = TASK_INTERRUPTIBLE;
//                                      while(voodoo_tv->gbuf[i].stat==GBUFFER_GRABBING) {
//                                              schedule();
//                                              if(signal_pending(current)) {
//                                                      remove_wait_queue(&voodoo_tv->capq, &wait);
//                                                      current->state = TASK_RUNNING;
//                                                      return -EINTR;
//                                              }
//                                      }
//                                      remove_wait_queue(&voodoo_tv->capq, &wait);
//                                      current->state = TASK_RUNNING;
//                              /* fall through */
//                              case GBUFFER_DONE:
//                              case GBUFFER_ERROR:
//                                      ret = (voodoo_tv->gbuf[i].stat == GBUFFER_ERROR) ? -EIO : 0;
//                                      voodoo_tv->gbuf[i].stat = GBUFFER_UNUSED;
//                      }
//                      return ret;
		}

	case VIDIOCMCAPTURE:
		{
			if (debug)
				printk
				    ("v3tv: VIDIOCMCAPTURE not implemented.\n");
			return -EFAULT;
//                      struct video_mmap vm;
//                      int ret;
//                      if(copy_from_user((void *) &vm, (void *) arg, sizeof(vm))) {
//				printk("v3tv: VIDIOCMCAPTURE couldn't copy from user.\n");
//                              return -EFAULT;
//			}

//                      down(&voodoo_tv->lock);
//                      ret = vgrab(voodoo_tv, &vm);
//                      up(&voodoo_tv->lock);
//                      return ret;
		}

	case VIDIOCGMBUF:
		{
			if (debug)
				printk("v3tv: VIDIOCMBUF not implemented.\n");
			return -EFAULT;
//                      struct video_mbuf vm;
//                      memset(&vm, 0 , sizeof(vm));

//                      vm.size=gbufsize*gbuffers;
//                      vm.frames=gbuffers;
//                      for (i = 0; i < gbuffers; i++)
//                              vm.offsets[i]=i*gbufsize;
//                      if(copy_to_user((void *)arg, (void *)&vm, sizeof(vm)))
//                              return -EFAULT;
//                      return 0;
		}

	default:
	    if (debug > 1) {
		switch (_IOC_TYPE(cmd)) {
		case 'v':
			printk("v3tv: unsuported ioctl 0x%x (v4l1, VIDIOC%s)\n",
			       cmd, (_IOC_NR(cmd) < V4L1_IOCTLS) ?
			       v4l1_ioctls[_IOC_NR(cmd)] : "???");
			break;
		case 'V':
			printk("v3tv: unsupported ioctl 0x%x (v4l2, %s)\n",
			       cmd,  v4l2_ioctl_names[_IOC_NR(cmd)]);
			break;
		default:
			printk("v3tv: unknown ioctl 0x%x (\?\?\?)\n",
			       cmd);
		}
	    }
	    return -ENOIOCTLCMD;
	}
	return 0;
}

static int v3tv_video_ioctl(struct inode *inode, struct file *file,
			    unsigned int cmd, unsigned long arg)
{
	return video_usercopy(inode, file, cmd, arg, v3tv_video_do_ioctl);
}

static struct file_operations v3tv_vdev_radio_fops = {
	.owner = THIS_MODULE,
	.open = v3tv_radio_open,
	.release = v3tv_radio_release,
	.ioctl = v3tv_radio_ioctl,
	.llseek = no_llseek
};

static struct video_device v3tv_vdev_radio = {
	.owner = THIS_MODULE,
	.name = "Voodoo3 3500 TV radio",
	.type = VID_TYPE_TUNER,
	.hardware = VID_HARDWARE_V3TV_RADIO,	/* FIXME: Define these in a .h file */
	.fops = &v3tv_vdev_radio_fops,
	.minor    = -1,
};

static struct file_operations v3tv_vdev_video_fops = {
	.owner = THIS_MODULE,
	.open = v3tv_video_open,
	.release = v3tv_video_release,
	.ioctl = v3tv_video_ioctl,
	.llseek = no_llseek,
	.read = v3tv_video_read,	/* experimental read */
//      .mmap     = v4l_voodoo_mmap,
//      .poll     = v4l_voodoo_poll,
};

static struct video_device v3tv_vdev_video = {
	.owner = THIS_MODULE,
	.name = "Voodoo3 3500 TV",
	.type =
	    VID_TYPE_CAPTURE | VID_TYPE_TUNER | VID_TYPE_OVERLAY |
	    VID_TYPE_CHROMAKEY | VID_TYPE_SCALES,
	.hardware = VID_HARDWARE_V3TV,	/* temporary id */
	.fops = &v3tv_vdev_video_fops,
	.minor    = -1,
};

static struct video_device v3tv_vdev_vbi = {
	.owner = THIS_MODULE,
	.name = "Voodoo3 3500 VBI Teletext",
	.type     = VID_TYPE_TUNER|VID_TYPE_TELETEXT,
	.hardware = VID_HARDWARE_V3TV,	/* temporary id */
	.fops = &v3tv_vdev_video_fops,
	.minor    = -1,
};

static void v3tv_unregister_video(struct voodoo_data *voodoo)
{
	if (voodoo->video_dev) {
		if (-1 != voodoo->video_dev->minor) {
			printk("v3tv: unregistering device video%d (%d)\n",
				voodoo->video_dev->minor & 0x1f, 
				voodoo->video_dev->minor);
			video_unregister_device(voodoo->video_dev);
		}
		voodoo->video_dev = NULL;
	}

	if (voodoo->vbi_dev) {
		if (-1 != voodoo->vbi_dev->minor) {
			printk("v3tv: unregistering device vbi%d (%d)\n",
				voodoo->vbi_dev->minor & 0x1f, 
				voodoo->vbi_dev->minor);
			video_unregister_device(voodoo->vbi_dev);
		}
		voodoo->vbi_dev = NULL;
	}

	if (voodoo->radio_dev) {
		if (-1 != voodoo->radio_dev->minor) {
			printk("v3tv: unregistering device radio%d (%d)\n",
				voodoo->radio_dev->minor & 0x1f, 
				voodoo->radio_dev->minor);
			video_unregister_device(voodoo->radio_dev);
		}
		voodoo->radio_dev = NULL;
	}
}

/* register video4linux devices */
static int __devinit v3tv_register_video(struct voodoo_data *voodoo)
{
	/* video */
	voodoo->video_dev = &v3tv_vdev_video;
	if (video_register_device(voodoo->video_dev,VFL_TYPE_GRABBER,video_nr)<0)
		goto err;
	printk(KERN_INFO "v3tv: registered device video%d (%d)\n",
	       voodoo->video_dev->minor & 0x1f, voodoo->video_dev->minor);

	/* vbi */
	voodoo->vbi_dev = &v3tv_vdev_vbi;
        if (video_register_device(voodoo->vbi_dev,VFL_TYPE_VBI,vbi_nr)<0)
		goto err;
	printk(KERN_INFO "v3tv: registered device vbi%d (%d)\n",
	       voodoo->vbi_dev->minor & 0x1f, voodoo->vbi_dev->minor);

	/* radio */
	voodoo->radio_dev = &v3tv_vdev_radio;
	if (video_register_device(voodoo->radio_dev, VFL_TYPE_RADIO,radio_nr)<0)
		goto err;
	printk(KERN_INFO "v3tv: registered device radio%d (%d)\n",
	       voodoo->radio_dev->minor & 0x1f, voodoo->radio_dev->minor);

	/* all done */
	return 0;

 err:
	v3tv_unregister_video(voodoo);
	return -1;
}

int v3tv_video_init(struct voodoo_data *voodoo)
{
//	int i;

	switch (voodoo->model) {
	case VOODOO3_MODEL_NTSC:
		printk(KERN_INFO "v3tv: NTSC model.\n");
		snprintf(voodoo->name, sizeof(voodoo->name), 
			"Voodoo3 TV 3500 %s", "NTSC");
		voodoo->norm = VIDEO_MODE_NTSC;
		voodoo->tuner = TUNER_PHILIPS_NTSC;	/* 2 */
		break;
	case VOODOO3_MODEL_PAL:
		printk(KERN_INFO "v3tv: PAL model.\n");
		snprintf(voodoo->name, sizeof(voodoo->name),
			"Voodoo3 TV 3500 %s", "PAL");
		voodoo->norm = VIDEO_MODE_PAL;
		voodoo->tuner = TUNER_Samsung_PAL_TCPM9091PD27;
		break;
	case VOODOO3_MODEL_SECAM:
		printk(KERN_INFO "v3tv: SECAM model.\n");
		snprintf(voodoo->name, sizeof(voodoo->name),
			"Voodoo3 TV 3500 %s", "SECAM");
		voodoo->norm = VIDEO_MODE_SECAM;
		voodoo->tuner = TUNER_PHILIPS_SECAM;	/* 3 */
		break;
	default:
		printk(KERN_INFO
		       "v3tv: Unknown model: 0x%x, defaulting to PAL.\n",
		       voodoo->model);
		snprintf(voodoo->name, sizeof(voodoo->name),
			"Voodoo3 TV 3500 model 0x%x", voodoo->model);
		voodoo->norm = VIDEO_MODE_PAL;
		voodoo->tuner = TUNER_Samsung_PAL_TCPM9091PD27;
		break;
	}

//	for (i=0;i<3;i++) {
//		voodoo_channels[i].norm = voodoo->norm;
//	}

	i2c_clients_command(voodoo->voodoo_i2c_adapter, TUNER_SET_TYPE, &voodoo->tuner);

	voodoo->picture_settings.brightness = 0x8000;
	voodoo->picture_settings.hue = 0x8000;
	voodoo->picture_settings.colour = 0x8000;
	voodoo->picture_settings.contrast = 0x8000;
	voodoo->picture_settings.whiteness = 0x8000;
	voodoo->picture_settings.depth = 0;
	voodoo->picture_settings.palette = 0;

	voodoo->vidOverlayStartX = 0;
	voodoo->vidOverlayStartY = 0;
	voodoo->vidOverlayEndX = video_standard[voodoo->norm].width;
	voodoo->vidOverlayEndY = video_standard[voodoo->norm].height * 2;
	voodoo->vidOrigWidth = video_standard[voodoo->norm].width;
	voodoo->vidOrigHeight = video_standard[voodoo->norm].height;

	voodoo->capture_win.x = 0;
	voodoo->capture_win.y = 0;
	voodoo->capture_win.width = video_standard[voodoo->norm].width;
	voodoo->capture_win.height = video_standard[voodoo->norm].height;

	v3tv_register_video(voodoo);

	return 0;
}

void v3tv_video_cleanup(struct voodoo_data *voodoo)
{
	v3tv_unregister_video(voodoo);

	voodoo->initialized &= ~I_v4l2;
}

/*
 * Overrides for Emacs so that we follow Linus's tabbing style.
 * ---------------------------------------------------------------------------
 * Local variables:
 * c-basic-offset: 8
 * End:
 */
