/*
 * Many declarations for v3tv drivers.
 *
 * $Id: voodoo.h,v 1.1.1.1 2004/05/23 04:46:51 scottsman Exp $
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 */

/* 3DFX defines */
#ifndef PCI_DEVICE_ID_3DFX_VOODOO3
#define PCI_DEVICE_ID_3DFX_VOODOO3 0x05
#endif

#ifndef VID_HARDWARE_V3TV
#define VID_HARDWARE_V3TV_RADIO 0x49
#define VID_HARDWARE_V3TV	0x50
#endif

/* Different versions of the Voodoo3 3500 TV */
#define VOODOO3_MODEL_NTSC  0x60
#define VOODOO3_MODEL_PAL   0x61
#define VOODOO3_MODEL_SECAM 0x62

#define V3_MAX_I2C_CLIENTS	10

#define VOODOO3_2D_SAVE 0
#define VOODOO3_2D_RESTORE 1

struct voodoo_data {
	/* locking */
//	spinlock_t s_lock;
//	struct semaphore lock;

	struct pci_dev *dev;

	unsigned char *io_base;	/* memory mapped registers */
	unsigned char *fb_base;	/* frame buffer memory     */
#ifdef INCLUDE_I2C
	struct i2c_client *i2c_clients[V3_MAX_I2C_CLIENTS];
#else
	struct i2c_adapter *voodoo_i2c_adapter;
#endif
	struct video_picture picture_settings;
	struct video_window capture_win;
	struct video_buffer fb;

	struct video_device *video_dev;
	struct video_device *radio_dev;
	struct video_device *vbi_dev;

	int buffering;		/* 1 = single, 2=double, 3=triple */

	u32 vidInAddr0;
	u32 vidInAddr1;
	u32 vidInAddr2;
	u32 captureBuf;
	u32 desktopAddr;

	int gbuffers;
	int gbufsize;
	unsigned char *fbuffer;

	int vidOverlayStartX;
	int vidOverlayStartY;
	int vidOverlayOffsetX;
	int vidOverlayOffsetY;

	int vidOverlayEndX;
	int vidOverlayEndY;

	int vidOrigWidth;
	int vidOrigHeight;

	int vidInStride;
	int vidOverlayStride;
	int vidScanlineBytes;

	int desktopStride;
	int desktopWidth;
	int desktopHeight;

	int overlayWidth, overlayHeight;
	int dudx;
	int dvdy;
	int XDecim;
	int YDecim;

	char name[32];
	int model;
	int norm;
	int type;
	int input;
	int tuner;
	int channel;
	unsigned long freq;
	unsigned long chroma;

	int initialized;
	int decoder_enabled;
};

struct conf_struct {
	unsigned int reg;
	int value;
	unsigned char endbit, startbit;
};

#define CONF_SIZE(a) (sizeof(a) / sizeof (struct conf_struct))

/* C function prototypes */
void v3tv_call_i2c_client(unsigned int DriverID, unsigned int cmd,
			     void *arg);
int v3tv_i2c_setup(int arg);
// static int v3tv_probe(void);
// int v3tv_init_module(void);
// void v3tv_cleanup_module(void);

// int v3tv_attach_adapter(struct i2c_adapter *adapter);
// int v3tv_detach_client(struct i2c_client *client);

void v3tv_set_chromakey(struct voodoo_data *voodoo);
// void v3tv_calc_scale(struct voodoo_data *voodoo)
// long do_fraction(int numBits, unsigned a, unsigned b);
// void v3tv_update_overlay_setup (struct voodoo_data *voodoo);
void v3tv_set_overlay(struct voodoo_data *voodoo);
void v3tv_overlay_setup(struct voodoo_data *voodoo);
void v3tv_overlay(struct voodoo_data *voodoo, int i);
// int v3tv_MakeRoom(int num);
// void v3tv_2dreg(int i);
void v3tv_capture_frame(struct voodoo_data *voodoo_tv);

// static int v3tv_radio_open (struct video_device *dev, int mode);
// static void v3tv_radio_close (struct video_device *dev);
// static int v3tv_radio_ioctl (struct video_device *dev, unsigned int cmd, void *arg);
// static irqreturn_t v3tv_voodoo_irq(int irq, void *dev_id, struct pt_regs *regs);
void v3tv_set_palette(struct video_picture *picture_settings);
// static int v3tv_voodoo_open(struct video_device *dev,int flags);
void v3tv_voodoo_close(struct video_device *dev);
int v3tv_voodoo_mmap(struct file *file, struct vm_area_struct *vma);
// static long v3tv_voodoo_read(struct video_device *dev, char *buf, unsigned long count, int nonblock);
// static int v3tv_voodoo_ioctl(struct video_device *dev, unsigned int cmd, void *arg);
int v3tv_video_init(struct voodoo_data *voodoo);
void v3tv_video_cleanup(struct voodoo_data *voodoo);

void v3tv_regwrite(unsigned int reg, unsigned int data);
unsigned int v3tv_regread(unsigned int reg);
// static inline unsigned int do_mask(struct conf_struct *conf);
// static inline unsigned int do_value(unsigned int orig, struct conf_struct *conf);
void v3tv_conf(struct conf_struct *conf, int num_entries);
// static inline unsigned long uvirt_to_kva(pgd_t *pgd, unsigned long adr);
// static inline unsigned long uvirt_to_bus(unsigned long adr);
// static inline unsigned long kvirt_to_bus(unsigned long adr);
inline unsigned long kvirt_to_pa(unsigned long adr);
void *rvmalloc(signed long size);
// void rvfree(void * mem, signed long size);

/* i2c defines */
/* bit locations in the register */
#define DDC_ENAB	0x00040000
#define DDC_SCL_OUT	0x00080000
#define DDC_SDA_OUT	0x00100000
#define DDC_SCL_IN	0x00200000
#define DDC_SDA_IN	0x00400000
#define I2C_ENAB	0x00800000
#define I2C_SCL_OUT	0x01000000
#define I2C_SDA_OUT	0x02000000
#define I2C_SCL_IN	0x04000000
#define I2C_SDA_IN	0x08000000

/* initialization states */
#define I_i2c	0x2		/* i2c-interface initialized */
#define I_ddc	0x4		/* ddc-interface initialized */
#define I_v4l2	0x8		/* v4l devices registered    */
#define I_irq	0x10		/* irq active */

/* delays */
#define CYCLE_DELAY	10
#define TIMEOUT		50

/* the only registers we use for i2c control*/
#define REG	0x78
#define REG2 	0x70

/* Voodoo avenger register offsets */
#define V3REG_vidTvOutBlankVCount 		0x3c	/* 31:0 R/W TV Out Vertical Active Start/End */
#define V3REG_vidMaxRgbDelta 			0x58	/* 23:0 R/W Maximum delta values for video filtering */
#define V3REG_vidProcCfg 			0x5c	/* 31:0 R/W Video Processor configuration */
#define V3REG_hwCurPatAddr 			0x60	/* 23:0 R/W Cursor Pattern Address */
#define V3REG_hwCurLoc 				0x64	/* 26:0 R/W X and Y location of HW cursor */
#define V3REG_hwCurC0 				0x68	/* 23:0 R/W Hw cursor color 0 */
#define V3REG_hwCurC1 				0x6c	/* 23:0 R/W Hw cursor color 1 */
#define V3REG_vidInFormat 			0x70	/* 31:0 R/W Video In Format */
#define V3REG_vidTvOutBlankHCount 		0x74	/* 31:0 R/W TV Out Horizontal Active Start/End */
#define V3REG_vidSerialParallelPort 		0x78	/* 31:0 R/W Serial and Parallel Ports */
#define V3REG_vidInXDecimDeltas 		0x7c	/* 31:0 R/W Video In horizontal decimation delta 1 & 2. */
#define V3REG_vidInDecimInitErrs 		0x80	/* 31:0 R/W Video In horizontal and vertical decimation initial error term */
#define V3REG_vidInYDecimDeltas 		0x84	/* 31:0 R/W Video In vertical decimation delta 1 & 2 */
#define V3REG_vidPixelBufThold 			0x88	/* 17:0 R/W Video Pixel Buffer Threshold */
#define V3REG_vidChromaMin 			0x8c	/* 31:0 R/W Chroma Key minimum value */
#define V3REG_vidChromaMax 			0x90	/* 31:0 R/W Chroma Key maximum value */
#define V3REG_vidInStatusCurrentLine 		0x94	/* 18:0 R Video In Status and Current Scan line */
#define V3REG_vidScreenSize 			0x98	/* 23:0 R/W Screen resolution */
#define V3REG_vidOverlayStartCoords 		0x9c	/* 31:0 R/W Start Surface Coordinates [31:28] Overlay Start Screen Coordinates */
#define V3REG_vidOverlayEndScreenCoord 		0xa0	/* 23:0 R/W Overlay End Screen Coordinates */
#define V3REG_vidOverlayDudx 			0xa4	/* 19:0 R/W Overlay horizontal magnification factor */
#define V3REG_vidOverlayDudxOffsetSrcWidth 	0xa8	/* 31:0 R/W Overlay horizontal magnification factor initial offset (bit 18:0) Overlay source surface width (bit 31:19) */
#define V3REG_vidOverlayDvdy 			0xac	/* 19:0 R/W Overlay vertical magnification factor */
#define V3REG_vidOverlayDvdyOffset 		0xe0	/* 18:0 R/W Overlay vertical magnification factor initial offset */
#define V3REG_vidDesktopStartAddr 		0xe4	/* 23:0 R/W Desktop start address */
#define V3REG_vidDesktopOverlayStride 		0xe8	/* 31:0 R/W Desktop and Overlay strides (linear or tile) */
#define V3REG_vidInAddr0 			0xec	/* 23:0 R/W Video In Buffer 0 starting address */
#define V3REG_vidInAddr1 			0xf0	/* 23:0 R/W Video In Buffer 1 starting address */
#define V3REG_vidInAddr2 			0xf4	/* 23:0 R/W Video In Buffer 2 starting address */
#define V3REG_vidInStride 			0xf8	/* 14:0 R/W Video In Buffer stride (linear or tile) */
#define V3REG_vidCurrOverlayStartAddr 		0xfc	/* 23:0 R Current overlay start address in use */

/* 2D registers */
#define V3REG_IO_BASE_1	0x100000

#define V3REG_status 		V3REG_IO_BASE_1 + 0x000	/* 31:0 R Avenger status register */
#define V3REG_intrCtrl		V3REG_IO_BASE_1 + 0x004	/* 31:0 R/W Interrupt control and status */
#define V3REG_clip0Min 		V3REG_IO_BASE_1 + 0x008	/* 28:0 R/W Min X & Y clip values when clip select is 0 */
#define V3REG_clip0Max 		V3REG_IO_BASE_1 + 0x00c	/* 28:0 R/W Max X & Y clip values when clip select is 0 */
#define V3REG_dstBaseAddr 	V3REG_IO_BASE_1 + 0x010	/* 23:0 R/W Destination base address */
#define V3REG_dstFormat 	V3REG_IO_BASE_1 + 0x014	/* 17:0 R/W Destination stride and bits per pixel */
#define V3REG_srcColorkeyMin 	V3REG_IO_BASE_1 + 0x018	/* 23:0 R/W Source Colorkey range (min) */
#define V3REG_srcColorkeyMax 	V3REG_IO_BASE_1 + 0x01c	/* 23:0 R/W Source Colorkey range (max) */
#define V3REG_dstColorkeyMin 	V3REG_IO_BASE_1 + 0x020	/* 23:0 R/W Destination Colorkey range (min) */
#define V3REG_dstColorkeyMax 	V3REG_IO_BASE_1 + 0x024	/* 23:0 R/W Destination Colorkey range (max) */
#define V3REG_bresError0 	V3REG_IO_BASE_1 + 0x028	/* 31:0 R/W Initial error for lines, right edges & stretch blt x */
#define V3REG_bresError1 	V3REG_IO_BASE_1 + 0x02c	/* 31:0 R/W Initial error for left poly edges & stretch blt y */
#define V3REG_rop 		V3REG_IO_BASE_1 + 0x030	/* 31:0 R/W 4 Ternary Raster operations */
#define V3REG_srcBaseAddr 	V3REG_IO_BASE_1 + 0x034	/* 23:0 R/W Source base address */
#define V3REG_commandExtra 	V3REG_IO_BASE_1 + 0x038	/* 31:0 R/W Extra control bits */
#define V3REG_lineStipple 	V3REG_IO_BASE_1 + 0x03c	/* 31:0 R/W Monochrome pattern for lines */
#define V3REG_lineStyle 	V3REG_IO_BASE_1 + 0x040	/* 28:0 R/W Style register for lines */
#define V3REG_pattern0Alias 	V3REG_IO_BASE_1 + 0x044	/* 31:0 R/W Alias to colorPattern(0) */
#define V3REG_pattern1Alias 	V3REG_IO_BASE_1 + 0x048	/* 31:0 R/W Alias to colorPattern(1) */
#define V3REG_clip1Min 		V3REG_IO_BASE_1 + 0x04c	/* 28:0 R/W Min X & Y clip values when clip select is 1 */
#define V3REG_clip1Max 		V3REG_IO_BASE_1 + 0x050	/* 28:0 R/W Max X & Y clip values when clip select is 1 */
#define V3REG_srcFormat 	V3REG_IO_BASE_1 + 0x054	/* 18:0 R/W Source stride and bits per pixel */
#define V3REG_srcSize 		V3REG_IO_BASE_1 + 0x058	/* 28:0 R/W Height and width of source for stretch blts */
#define V3REG_srcXY 		V3REG_IO_BASE_1 + 0x05c	/* 28:0 R/W Starting pixel of blt source data */
#define V3REG_colorBack 	V3REG_IO_BASE_1 + 0x060	/* 31:0 R/W Background color */
#define V3REG_colorFore 	V3REG_IO_BASE_1 + 0x064	/* 31:0 R/W Foreground color */
#define V3REG_dstSize 		V3REG_IO_BASE_1 + 0x068	/* 28:0 R/W Destination width and height for blts and rectangle fills */
#define V3REG_dstXY 		V3REG_IO_BASE_1 + 0x06c	/* 28:0 R/W Starting X and Y of destination for blts */
#define V3REG_command 		V3REG_IO_BASE_1 + 0x070	/* 31:0 R/W 2D command mode & control bits */
#define V3REG_launchArea 	V3REG_IO_BASE_1 + 0x080	/* 31:0 R Initiates 2D commands */

/* 3D registers */
#define V3REG_IO_BASE_2	0x200000

#define V3REG_swapbufferCMD	V3REG_IO_BASE_2 + 0x128	/*8:0 FBI W Yes/Yes Execute SWAPBUFFER command  */
#define V3REG_swapPending	V3REG_IO_BASE_2 + 0x24C	/*na FBI W No / No Swap buffer pending          */
#define V3REG_leftOverlayBuf 	V3REG_IO_BASE_2 + 0x250	/*31:0 FBI W No / Yes Left Overlay address      */
#define V3REG_rightOverlayBuf 	V3REG_IO_BASE_2 + 0x254	/*31:0 FBI W No / Yes Right Overlay address     */

/*
 * Overrides for Emacs so that we follow Linus's tabbing style.
 * ---------------------------------------------------------------------------
 * Local variables:
 * c-basic-offset: 8
 * End:
 */
