/*
 ****************************************************************************
 * Copyright 2001-2003 by NeTraverse, Inc.
 * This software is distributed under the terms of the GPL
 * which is supplied in the LICENSE file with this distribution
 ***************************************************************************
 * $Id: mki-adapter26.h,v 1.5 2004/12/03 02:59:15 lreiter Exp $
 ***************************************************************************
 */
#ifndef MKI_ADAPTER_H
#define MKI_ADAPTER_H

#ifndef PROT_USER
#define PROT_USER       0x8             /* page can not be accessed */
#endif

/* defines to explicitly set the linkage of functions (for CONFIG_REGPARM) */
#define MKIA_API	__attribute__ ((regparm(0)))

/**************/
/* structures */
/**************/
struct mergevma {
	unsigned long offset;
	unsigned long vm_start;
	unsigned long vm_end;
	unsigned long nbytes;
	unsigned long npages;
	unsigned long page_size;
	/* ... */
};

struct pt_regs;

/**************/
/* functions  */
/**************/
extern int MKIA_API mkia_process_owns_fpu(void *vm86p);

extern void * MKIA_API mkia_kmalloc(int size, int flags);
extern void * MKIA_API mkia_get_free_pages(int foo, int flags);
/* values for flags */
#define MKIA_SLEEP	0x0001
#define MKIA_NOSLEEP	0x0002
#define MKIA_DMA	0x0004
#define MKIA_ZERO	0x0100

extern void MKIA_API mkia_kfree(void *pointer);
extern void MKIA_API mkia_free_pages(unsigned long addr, int size);

extern void * MKIA_API mkia_vmalloc(int size);
extern void MKIA_API mkia_vfree(void *pointer);

extern int MKIA_API mkia_strncmp(char *s1, char *s2, int len);
extern int MKIA_API mkia_strcmp(char *s1, char *s2);
extern void * MKIA_API mkia_strncpy(char *s1, char *s2, int len);
extern void * MKIA_API mkia_strcpy(char *s1, char *s2);
extern void MKIA_API mkia_memset(void *mem, int val, int len);
extern void MKIA_API mkia_memcpy(void *mem, void *mem2, int len);


extern int MKIA_API mkia_register_chrdev(int maj, char * name, void *fops);
extern void MKIA_API mkia_unregister_chrdev(int major, char * name);

extern int MKIA_API mkia_get_fops_size(void);
extern void * MKIA_API
    mkia_alloc_file_ops(void *read, void *write, void *readdir, 
		    void *poll, void *ioctl, void *mmap, void *open, 
		    void *release);
extern void MKIA_API mkia_free_file_ops(void *fops);

extern void * MKIA_API mkia_mrgioctl_unlock_kernel(void);
extern void MKIA_API mkia_mrgioctl_lock_kernel(void *cookie);
extern int MKIA_API mkia_remap_page_range(unsigned long user_addr, 
						    unsigned long phys_addr, 
						    unsigned long size,
						    void *vma);
extern unsigned long MKIA_API mkia_get_vma_offset(void *vmap);
extern unsigned long MKIA_API mkia_get_vma_page_prot(void *vmap);
extern unsigned long MKIA_API mkia_get_vma_vm_start(void *vmap);
extern unsigned long MKIA_API mkia_get_vma_vm_end(void *vmap);
extern int MKIA_API mkia_get_inode_minor(void *inode);
extern int MKIA_API mkia_get_file_minor(void *file);

extern int MKIA_API mkia_install_hook(int id, int (*hook_fn)(void *));
extern void * MKIA_API mkia_alloc_waitqueuep(void);
extern void MKIA_API mkia_free_waitqueuep(void *wqp);
extern void * MKIA_API mkia_current(void);
extern int MKIA_API mkia_signal_pending(void *foo);
extern void * MKIA_API mkia_get_current_task(void);
extern int MKIA_API mkia_get_current_pid(void);
extern int MKIA_API mkia_get_current_task_index(void);
extern int MKIA_API mkia_call_svwait(void *wqp, volatile int *lockp, 
				   int interruptible);
extern void MKIA_API mkia_wake_up(void *wqp);
extern void MKIA_API mkia_poll_wait(void *file, void *wqp, 
					      void *wait);
extern void MKIA_API mkia_poll_wake(void *wqp); 
extern void MKIA_API mkia_adjust_esp0(int numlongs);
extern void MKIA_API mkia_mod_inc_use_count(void * module);
extern void MKIA_API mkia_mod_dec_use_count(void * module);
extern int MKIA_API mkia_request_irq(int irq,
		void (*handler)(int, void *, struct pt_regs *), 
		unsigned long foo, char *bar, void *baz);
extern void MKIA_API mkia_free_irq(int irq, void *baz);
extern void MKIA_API mkia_kill_proc(int procref, int sig, int flag);
extern void MKIA_API mkia_file_set_private_data(void * filp, void *data);
extern void * MKIA_API mkia_file_get_private_data(void * filp);
extern void * MKIA_API mkia_file_get_f_pos_addr(void * filp);
extern long MKIA_API mkia_file_get_f_mode(void * filp);
extern int MKIA_API mkia_is_file_nonblock(void *file);
extern int MKIA_API mkia_copy_to_user(void *dest, void *src, int len);
extern int MKIA_API mkia_copy_from_user(void *dest, void *src, int len);
extern int MKIA_API mkia_get_user(void *data, int len);
extern int MKIA_API mkia_put_user(unsigned long flags, void *data, int len);

extern int MKIA_API mkia_init_vnetint(void);
extern void MKIA_API mkia_cleanup_vnetint(void);
extern int MKIA_API mkia_populate_mrgvma(struct mergevma *mrgvma, void * vmap);

extern int MKIA_API timeout(void MKIA_API (*timeout_func)(void *), 
		void *timeout_arg, long ticks);
extern void MKIA_API untimeout(int id);

extern void MKIA_API mkia_post_event(void *cookie);
extern int  MKIA_API mkia_set_gdt_entry(unsigned short sel, unsigned long *new_entry);
extern int  MKIA_API mkia_set_ldt_entry(unsigned short sel, unsigned long *new_entry);
extern int  MKIA_API mkia_check_vm86(void);
extern void * MKIA_API mkia_get_vm86p(void);
extern void MKIA_API mkia_set_vm86p(void *vm86p);
extern void MKIA_API mkia_mark_vm86(void);
extern void MKIA_API mkia_clear_vm86(void);
extern void MKIA_API mkia_set_idt_entry(unsigned short vect_num, 
		unsigned long *new_entry, unsigned long *prev_entry);
extern void MKIA_API mkia_set_idt_dpl(void);
extern void MKIA_API mkia_enter_debugger(int reason, int error, void *regs);
extern void MKIA_API mkia_post_event(void *cookie);
extern void * MKIA_API mkia_getparm(int request, void *parm);
extern int  MKIA_API mkia_set_private_ldt(void *ldtp, size_t limit);
extern void MKIA_API mkia_pgfault_get_state(int *pfault_ok, void *fcstate);
extern void MKIA_API mkia_pgfault_restore_state(void *fcstate);
extern void MKIA_API mkia_yield(void);
extern void MKIA_API mkia_remove_hook(int id);

extern mkia_64bit_t MKIA_API mkia_virt_to_phys(void *kaddr);
extern mkia_64bit_t * MKIA_API mkia_get_pagedir(void);
extern int  MKIA_API mkia_remove_page_ref(unsigned long vaddr, mkia_64bit_t physpte, 
		void *file_cookie, off_t offset);
extern mkia_64bit_t MKIA_API mkia_add_page_ref(unsigned long vaddr);

extern void * MKIA_API mkia_get_file_cookie(int filedes);
extern void MKIA_API mkia_put_file_cookie(void *farg);
extern int  MKIA_API mkia_upageflt(unsigned long address, int error_code);
extern void * MKIA_API mkia_alloc_priv_tss(void);
extern unsigned long MKIA_API mkia_mmap_k(unsigned long, size_t, 
		int, int, int, off_t, int *);
extern unsigned long MKIA_API mkia_mprotect_k(unsigned long, size_t, int);
extern int MKIA_API mkia_munmap_k(unsigned long, size_t);
extern int MKIA_API mkia_ftruncate_k(int, off_t);
extern void MKIA_API mhia_switch(void *, void *);
extern void MKIA_API mhia_switch_to(void *, void *);
extern void MKIA_API mhia_switch_away(void *, void *);
extern void MKIA_API mhia_ret_user(void *, void *);
extern void MKIA_API mhia_exit(void *, void *);
extern void MKIA_API mhia_swap(void *, void *);

#endif /* MKI_ADAPTER_H */

