/*
 *  
 *  Copyright (C) 2004 Bell Kin bell_kin@pek.com.tw
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include <linux/types.h>
#include <linux/skbuff.h>
#include <linux/rtnetlink.h>
#include <linux/netdevice.h>
#include <linux/etherdevice.h>

#ifdef CONFIG_PRISM54_HOSTAPD

#include <asm/io.h>
#include <asm/system.h>
#include <linux/if_arp.h>

#include "prismcompat.h"
#include "isl_38xx.h"
#include "islpci_mgt.h"
#include "isl_oid.h"	/* additional types and defs for isl38xx fw */
#include "isl_ioctl.h"

#include <net/iw_handler.h>

#include "isl_ap.h"

#define DEBUG(x,args...)

/* PIMFOR netdev interface */

/* parent dev, interface name */
int
create_ap_dev(struct net_device *dev, char *aifname)
{
	int err = 0;
	int i = 0, j = 0;

	struct net_device *ap_dev;
	struct ap_net_local *ap_local;
	islpci_private *priv = dev->priv;

	DEBUG(DBG_FUNC_CALL, "create_ap_link \n");

	if ((dev == NULL) || (aifname == NULL)) {
		printk("create_ap_link(%lx, %lx) wrong parameter(s)!\n",
		       (unsigned long) dev, (unsigned long) aifname);
		return -1;
	}
	if (priv->ap_dev!=NULL) {
		printk("create_ap_link: already created\n");
		return -1;
	}
	if ((ap_dev = alloc_etherdev(sizeof(struct ap_net_local)))==NULL) {
		printk("create_ap_link: alloc_etherdev failed\n");
		return -ENOMEM;
	}
	printk("isl_ap: create %s\n",aifname);
	ap_local = (struct ap_net_local *)ap_dev->priv;
	memset(ap_dev->priv, 0, sizeof (struct ap_net_local));
	ap_local->parent_dev = dev;
	strcpy(ap_dev->name,aifname);

	ap_dev->mtu = MGMT_FRAME_SIZE;
	ap_dev->type = ARPHRD_VOID;
	ap_dev->addr_len = 4;
	ap_dev->hard_header_len = 8;
	ap_dev->tx_queue_len = 100;

	/* Override the network functions we need to use */
	ap_dev->open = ap_net_open;
	ap_dev->stop = ap_net_close;
	ap_dev->get_stats = ap_net_get_stats;
	ap_dev->hard_start_xmit = ap_net_send_packet;
	ap_dev->do_ioctl = ap_net_ioctl;
	ap_dev->rebuild_header = NULL;
	ap_dev->hard_header = NULL;
	ap_dev->flags |= IFF_NOARP;

	/* need to lock because double register_netdev calls */
	rtnl_lock();
	err = register_netdevice(ap_dev);
	rtnl_unlock();
	if (err) {
		printk("create_ap_link: register_netdev() failed!\n");
		kfree(ap_dev->priv);
		kfree(ap_dev);
		return err;
	}
	priv->ap_dev = ap_dev;
	return 0;
}

/* parent dev */
int
destroy_ap_dev(struct net_device *dev)
{
	int err = 0;
	int i = 0;

	struct net_device *ap_dev;

	DEBUG(DBG_FUNC_CALL, "destroy_ap_link \n");

	if (dev == NULL) {
		printk("destroy_ap_link(%lx) wrong parameter(s)!\n",
		       (unsigned long) dev);
		return -1;
	}
	/* need to lock because double register_netdev calls */
	rtnl_lock();
	ap_dev = ((islpci_private*)dev->priv)->ap_dev;
	err = dev_close(ap_dev);
	if (err) {
		printk("destroy_ap_link: dev_close() failed!\n");
		return err;
	}
	err = unregister_netdevice(ap_dev);
	rtnl_unlock();
	if (err != 0) {
		printk(
			"destroy_ap_link: %s could not be unregistered!\n",
			ap_dev->name);
		return err;
	}
	kfree(ap_dev->priv);
	kfree(ap_dev);
	return 0;
}


/* linux netdev functions */
int
ap_net_open(struct net_device *dev)
{
	u32 mlmeautolevel = DOT11_MLME_EXTENDED;
	struct ap_net_local* ap_local = (struct ap_net_local*)dev->priv;
	islpci_private *ppriv = ap_local->parent_dev->priv;

	DEBUG(DBG_FUNC_CALL, "ap_net_open \n");
	ppriv->ap_mode = PRISM54_AP_AA; /* auth and assoc */
	netif_start_queue(dev);
	return 0;
}

int
ap_net_send_packet(struct sk_buff *skb, struct net_device *dev)
{
	struct ap_net_frame_header *hdr;
	unsigned char op;
	unsigned long oid;
	int ret;
	struct ap_net_local *local = (struct ap_net_local*)dev->priv;
	islpci_private *priv = local->parent_dev->priv;
	DEFINE_WAIT(wait);
	if (down_interruptible(&priv->mgmt_sem)) {
		return -ERESTARTSYS;
	}
	prepare_to_wait(&priv->mgmt_wqueue, &wait, TASK_UNINTERRUPTIBLE);
	if(skb->len<sizeof(struct ap_net_frame_header)) {
		printk("ap_net: invalid packet\n");
		dev_kfree_skb(skb);
		return 0;
	}
	local = (struct ap_net_local*)dev->priv;
	hdr = (struct ap_net_frame_header *)skb->data;
	oid = ntohl(hdr->oid);
	op = ntohl(hdr->op);
	skb_pull(skb, sizeof(struct ap_net_frame_header));
	ret = islpci_mgt_transmit(local->parent_dev, op, oid, skb, PIMFOR_FLAG_APPLIC_ORIGIN);
	if(ret<0) {
		printk("ap_net: send failed\n");
	}
	finish_wait(&priv->mgmt_wqueue, &wait);
	up(&priv->mgmt_sem);
	return 0;
}

int
ap_net_close(struct net_device *dev)
{
	u32 mlmeautomode = CARD_DEFAULT_MLME_MODE;
	struct ap_net_local *ap_local = (struct ap_net_local *) dev->priv;
	islpci_private *ppriv = ap_local->parent_dev->priv;

	DEBUG(DBG_FUNC_CALL, "ap_net_close \n");
	ppriv->ap_mode = 0;
	/*mgt_set(ppriv, DOT11_OID_MLMEAUTOLEVEL, &mlmeautolevel);*/
	netif_stop_queue(dev);
	return 0;
}

struct net_device_stats *
ap_net_get_stats(struct net_device *dev)
{
	struct ap_net_local *ap_local = (struct ap_net_local *) dev->priv;
	struct net_device_stats *stats = &ap_local->stats;

	DEBUG(DBG_FUNC_CALL, "ap_net_get_stats \n");

	return stats;
}

int
ap_net_ioctl(struct net_device *dev, struct ifreq *ifr, int cmd)
{
	printk
	    ("ap_net_ioctl: Ioctls for AP devices are NOT supported in this driver.\n");
	return -EINVAL;
}

#endif /* CONFIG_PRISM54_HOSTAPD */
