/***************************************************************************
 * Video post-processing module for the W996[87]CF driver for Linux.       *
 *                                                                         *
 * Copyright (C) 2002-2005 by Luca Risolia <luca.risolia@studio.unibo.it>  *
 *                                                                         *
 * This program is free software; you can redistribute it and/or modify    *
 * it under the terms of the GNU General Public License as published by    *
 * the Free Software Foundation; either version 2 of the License, or       *
 * (at your option) any later version.                                     *
 *                                                                         *
 * This program is distributed in the hope that it will be useful,         *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 * GNU General Public License for more details.                            *
 *                                                                         *
 * You should have received a copy of the GNU General Public License       *
 * along with this program; if not, write to the Free Software             *
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.               *
 ***************************************************************************/

#include <linux/version.h>
#include <linux/init.h>

#include "w9968cf_vpp.h" /* interface */

#include "w9968cf_decoder.h"
#include "w9968cf_vppfuncs.h"


#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 4, 19)
#	error "Kernel 2.4.19 is the minimum for this driver."
#endif


MODULE_AUTHOR("Luca Risolia <luca.risolia@studio.unibo.it>");
MODULE_DESCRIPTION("Video post-processing module for the "
                   "W996[87]CF JPEG USB Dual Mode Camera Chip driver");
MODULE_SUPPORTED_DEVICE("Video");
MODULE_LICENSE("GPL");
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 4)
MODULE_VERSION(""); /* expands to a md5sum of the files in the dir */
#endif


static struct w9968cf_vpp_t w9968cf_vpp = {
	.owner = THIS_MODULE,
	.check_headers = &w9968cf_check_headers,
	.decode = &w9968cf_decode,
	.swap_yuvbytes = &w9968cf_swap_yuvbytes,
	.uyvy_to_rgbx = &w9968cf_uyvy_to_rgbx,
	.scale_up = &w9968cf_scale_up,
};


/****************************************************************************
 * Module init and exit                                                     *
 ****************************************************************************/

static int __init w9968cf_vpp_module_init(void)
{
	w9968cf_init_decoder();
	return w9968cf_vppmod_register(&w9968cf_vpp);
}

static void __exit w9968cf_vpp_module_exit(void)
{
	w9968cf_vppmod_deregister(&w9968cf_vpp);
}

module_init(w9968cf_vpp_module_init);
module_exit(w9968cf_vpp_module_exit);

#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 5, 0)
	EXPORT_NO_SYMBOLS;
#endif
