#include <linux/lids.h>
#include <linux/lidsext.h>
#include <linux/lidsif.h>

#ifdef CONFIG_LIDS_ALLOW_LFS
#define lids_process_switch() \
do { 		\
	if (lids_load != (lids_flag_raised(flags, LIDS_FLAGS_LIDS_ON) != 0)) { \
		lids_load = (lids_flag_raised(flags, LIDS_FLAGS_LIDS_ON) != 0);\
		lids_security_alert("LIDS switched to %d", lids_load);\
		if (lids_load) \
			lids_flag_raise(lids_flags, LIDS_FLAGS_LIDS_ON); \
		else								\
			lids_flag_lower(lids_flags, LIDS_FLAGS_LIDS_ON);	\
	}									\
	if (lids_local_on !=							\
	    (lids_flag_raised(flags, LIDS_FLAGS_LIDS_LOCAL_ON) != 0)) {		\
		lids_local_on = (lids_flag_raised(flags, LIDS_FLAGS_LIDS_LOCAL_ON) != 0); \
			/* XXX: Race condition here. We must first assign the PID */  \
		lids_security_alert("LIDS locally switched to %i", 		\
				    lids_local_on);				\
		if (lids_local_on) {						\
			lids_flag_raise(lids_flags, LIDS_FLAGS_LIDS_LOCAL_ON);	\
		} else {							\
			lids_local_pid = current->parent->pid;			\
										\
			if (lids_local_pid == 1) {	/* this doesn't apply to init */\
				printk						\
				    ("Can't give local lids deactivation to init!!\n"); \
				lids_flag_raise(lids_flags,			\
						LIDS_FLAGS_LIDS_LOCAL_ON);	\
				lids_local_on = 1;				\
			} else							\
				lids_flag_lower(lids_flags, LIDS_FLAGS_LIDS_LOCAL_ON);	\
		}								\
	}	\
}while(0);
#else
#define lids_process_switch() \
do { 		\
	if (!lids_flag_raised(flags, LIDS_FLAGS_LIDS_ON)) { \
		lids_security_alert					\
		    ("Attempt to switch LIDS off (feature disabled)");	\
		return -1;						\
	}	\
}while(0);
#endif

#ifdef CONFIG_LIDS_ALLOW_SWITCH

#define lids_process_password()  \
do{	\
	char lids_sig[LIDS_PW_LEN*2];					\
	if ((!lids_first_time) || (locks.passwd[0])) {			\
		lids_sha256(locks.passwd, LIDS_PW_LEN, lids_sig);	\
		memset((char *)locks.passwd, '\0', sizeof(passwd_t));	\
	}	\
	if (((lids_first_time) && (!locks.passwd[0])) ||	\
	    (!memcmp(lids_sig, lids_pw, LIDS_PW_LEN))) {	\
		/* access granted ! */				\
		number_failed = 0;				\
		if (lids_process_flags(locks.flags) == 0) {	\
			/* Seal the kernel,we can change the cap_set here */\
			if (lids_first_time ||			\
			    lids_flag_raised(locks.flags,	\
					     LIDS_FLAGS_RELOAD_CONF)\
			    || lids_flag_raised(locks.flags,	\
						LIDS_FLAGS_SHUTDOWN))\
				cap_bset = lids_cap_val;	\
			else					\
				cap_bset = locks.cap_bset;	\
			lids_security_alert			\
			    ("Changed: cap_bset=0x%x lids_flags=0x%x",\
			     cap_t(cap_bset), lids_flags);	\
		}	\
		lids_first_time = 0;	\
	} else {			\
		number_failed++;	\
		lids_security_alert	\
		    ("Give incorrect password (try #%d) with caps=0x%x and flags=0x%x",\
		     number_failed, cap_t(locks.cap_bset), locks.flags);	\
		if (number_failed >= LIDS_MAX_TRY) {	\
			wait_after_fail = 1;		\
			init_timer(&fail_timer);	\
			fail_timer.function = reenable_sysctl;	\
			fail_timer.data = (unsigned long)NULL;	\
			fail_timer.expires =		\
			    jiffies + LIDS_TTW_FAIL * HZ;	\
			add_timer(&fail_timer);		\
		}	\
	}		\
}while(0);
#else
#define lids_process_password()  \
do{ \
	if ((lids_first_time) && (!locks.passwd[0])) {	\
		/* access granted ! */			\
		number_failed = 0;			\
		if (lids_process_flags(locks.flags) == 0) {	\
			if (lids_first_time ||		\
			    lids_flag_raised(locks.flags,	\
					     LIDS_FLAGS_RELOAD_CONF)\
			    || lids_flag_raised(locks.flags,	\
						LIDS_FLAGS_SHUTDOWN))\
				cap_bset = lids_cap_val;	\
			else					\
				cap_bset = locks.cap_bset;	\
			lids_security_alert			\
			    ("Changed: cap_bset=0x%x lids_flags=0x%x",\
			     cap_t(cap_bset), lids_flags);	\
		}	\
		lids_first_time = 0;\
		lids_security_alert	\
		    ("Attempt %d to switch caps/flags with caps=0x%x and flags=0x%x (feature disabled)",\
		     number_failed, cap_t(locks.cap_bset), locks.flags);\
	}	\
}while(0);
#endif

#ifdef CONFIG_LIDS_ALLOW_SWITCH
char lids_pw[LIDS_PW_LEN+16];
int lids_read_pw()
{
	struct file *filp;
	char buffer[LIDS_PW_LEN];
	mm_segment_t oldfs;
	int bytes;
	int error = 0;

	filp = filp_open(LIDS_PW_FILE, O_RDONLY, O_RDONLY);
	if (IS_ERR(filp) || (filp == NULL)) {
		error = -1;
		printk("LIDS: Error opening passwd file " LIDS_PW_FILE
		       ". Does it exist?\n");
		return error;
	}

	if (filp->f_op->read == NULL) {
		fput(filp);
		error = -3;
		printk("LIDS: The file " LIDS_PW_FILE " can not be read\n");
		return error;
	}

	/* Now read LIDS_PW_LEN bytes from postion "StartPos" */
	filp->f_pos = 0;
	oldfs = get_fs();
	set_fs(KERNEL_DS);
	bytes = filp->f_op->read(filp, buffer, LIDS_PW_LEN, &filp->f_pos);
	set_fs(oldfs);

	if (bytes < LIDS_PW_LEN)
		return -1;

	memset(lids_pw,'\0',LIDS_PW_LEN);
	memcpy(lids_pw, buffer, LIDS_PW_LEN);
	/* Close the file */
	fput(filp);
	return error;
}
#else
int lids_read_pw()
{
	return 0;
}
#endif
