#ifndef LIDSEXT_H
#define LIDSEXT_H

/*
 * This file contains LIDS macros needed for logging and debugging,
 * used about everywhere in the kernel.
 *
 */

/* needed extern declarations */

#include <linux/config.h>

extern void lids_cap_log(int);
extern int lids_cap_time_checker(const int);
extern int lids_local_off(void);
extern int lids_reload_conf;
extern int lids_load;
extern int lids_local_on;
extern int lids_local_pid;
extern int lids_first_time;
extern int lids_state;

#ifdef CONFIG_LIDS_DEBUG
#define LIDS_DEBUG
#endif

#define LIDS_STR2(x) #x
#define LIDS_STR(X) LIDS_STR2(X)

#ifdef LIDS_DEBUG
#define LIDS_DBG(fmt, arg...)			\
	printk(KERN_DEBUG "LIDS: %s:%i: " fmt,	\
		__FUNCTION__, __LINE__, ## arg)
#else
#define LIDS_DBG(fmt, arg...)
#endif

#ifdef CONFIG_LIDS_RESTRICT_MODE_SWITCH

static inline int
lids_check_tty(struct tty_struct *tty)
{
	return (tty && !(0
#ifdef CONFIG_LIDS_MODE_SWITCH_CONSOLE
			 || tty->driver->type == TTY_DRIVER_TYPE_CONSOLE
#endif
#ifdef CONFIG_LIDS_MODE_SWITCH_SERIAL
			 || tty->driver->type == TTY_DRIVER_TYPE_SERIAL
#endif
#ifdef CONFIG_LIDS_MODE_SWITCH_PTY
			 || tty->driver->type == TTY_DRIVER_TYPE_PTY
#endif
		));
}
#else
static inline int
lids_check_tty(struct tty_struct *tty)
{
	return 0;
}
#endif				/* CONFIG_LIDS_RESTRICT_MODE_SWITCH */

extern void lids_log(int flood, const char *message, ...);

#define LIDS_TIMEOUT_AFTER_FLOOD 60

#ifdef CONFIG_LIDS_NO_FLOOD_LOG

#define lids_security_alert(message, args...)                                       \
do {                                                                       	    \
	if (lids_load && lids_local_load) {					    \
		static unsigned long warning_time = 0, no_flood_yet = 0;            \
		static spinlock_t lids_security_alert_lock = SPIN_LOCK_UNLOCKED;    \
									   	    \
		spin_lock(&lids_security_alert_lock);                               \
										    \
/* Make sure at least LIDS_TIMEOUT_AFTER_FLOOD 			   	    \
 * passed since the last warning logged 				   	    \
 */ 									   	    \
		if ((!warning_time) || 					            \
		    (jiffies-warning_time > LIDS_TIMEOUT_AFTER_FLOOD*HZ)) {  \
			warning_time = jiffies; no_flood_yet = 1;                   \
			lids_log(0, message , ## args);                             \
		} else if (no_flood_yet) {                                          \
			warning_time = jiffies; no_flood_yet = 0;                   \
			lids_log(1, message , ## args);                             \
		}                                                           	    \
		spin_unlock(&lids_security_alert_lock);                             \
	}									    \
} while(0)

#else				/* CONFIG_LIDS_NO_FLOOD_LOG */

#define lids_security_alert(message, args...)                                      \
do {                                                                       	   \
	if (lids_load && lids_local_load) {					   \
		static spinlock_t lids_security_alert_lock = SPIN_LOCK_UNLOCKED;   \
										   \
		spin_lock(&lids_security_alert_lock);                              \
		lids_log(0, message , ## args);                                    \
		spin_unlock(&lids_security_alert_lock);                            \
	}				   					   \
} while(0)

#endif				/* CONFIG_LIDS_NO_FLOOD_LOG */

//#ifdef CONFIG_LIDS_ALLOW_SWITCH
#if 1
#define lids_local_load ( lids_local_on || (!lids_local_off()) )
#else
#define lids_local_load 1
#endif				/* CONFIG_LIDS_ALLOW_SWITCH */

#endif				/* LIDSEXT_H */
