/*
 *  LIDS ACL functions 
 *
 *  Copyright (C) 2002,2004 Huagang Xie <xie@lids.org>
 *  Copyright (C) 2002,2003 Philippe Biondi <biondi@cartel-securite.fr>
 *
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2 of the License, or
 *	(at your option) any later version.
 */
/*
 *  Changes:
 *	
 *	[Oct 14 1999, Xie Huagang] initial creation 
 * 	[Sep 26 2000, Xie Huagang] Port to linux 2.4.0-test8
 * 	[Feb 23 2003, Xie Huagang] LSM support for 2.5.x
 */

#include <linux/mm.h>
#include <linux/proc_fs.h>
#include <linux/smp_lock.h>
#include <linux/quotaops.h>
#include <asm/uaccess.h>
#include <asm/unaligned.h>
#include <asm/semaphore.h>
#include <asm/page.h>
#include <asm/pgtable.h>
#include <asm/namei.h>
#include <linux/time.h>
#include <linux/fs.h>
#include <linux/kernel.h>
#include <linux/highmem.h>
#include <linux/slab.h>
#include <linux/file.h>
#include <linux/config.h>
#include <linux/tty.h>
#include <linux/sysctl.h>

#include <linux/version.h>
#include <linux/lids.h>
#include <linux/lidsext.h>
#include <linux/lidsif.h>

 /***********************************************************************
 * General variables
 ***********************************************************************/
int lids_load = 1;
int lids_local_on = 1;
int lids_acl_discovery = 0;	/* */
lids_flags_t lids_flags = 0;
int lids_local_pid = 0;
u32 lids_current = 0;
int lids_first_time = 1;

int lids_state = LIDS_STATE_BOOT;	/* initial state is boot */

void *b11;
static int lids_get_inode_security(struct dentry *o_dentry,
                                   struct inode *inode);

/*
 *	Free routine 
 *
 */
static void
lids_free_object_acl(struct lids_object_acl *o_acl)
{
	struct lids_object_acl *p;

	if (!o_acl)
		return;

	while (o_acl) {
		p = o_acl->next;
		kfree(o_acl);
		o_acl = p;
	}
}

void
lids_free_subject_acl(struct lids_subject_acl *s_acl)
{
	struct lids_object_acl *o_acl;

	if (!s_acl)
		return;

	o_acl = s_acl->o_acl;
	lids_free_object_acl(o_acl);
	kfree(s_acl);
}

void
lids_free_inode_acl(struct lids_inode_acl *i_acl)
{
	if (!i_acl)
		return;
	if (i_acl->magic != LIDS_MAGIC)
		return;

	lids_free_subject_acl(i_acl->s_acl);
	kfree(i_acl);
}

/*
 *	free the lids acl structure here
 */
void
lids_free_task_acl(struct lids_task_acl *task_acl)
{
	struct lids_subject_acl *s_acl;

	if (!task_acl)
		return;

	s_acl = task_acl->s_acl;
	lids_free_subject_acl(s_acl);
	kfree(task_acl);
	return;
}

/* 
 *	lids allocation routine
 */
static struct lids_task_acl *
lids_alloc_task_acl(struct task_struct *task)
{
	struct lids_task_acl *task_acl;

	task_acl = kmalloc(sizeof (struct lids_task_acl), GFP_ATOMIC);
	if (!task_acl) {
		printk(KERN_INFO "LIDS: kmalloc error for task_acl\n");
		return NULL;
	}
	task_acl->magic = LIDS_MAGIC;
	task_acl->task = task;
	task_acl->s_acl = NULL;
	INIT_LIST_HEAD(&task_acl->list);
	spin_lock_init(&task_acl->t_lock);

	task_lock(task);
	task->security = task_acl;
	task_unlock(task);

	return task_acl;
}

/***********************************************************************
 ***********************************************************************
 *
 * LIDS protection management
 *
 ***********************************************************************
 ***********************************************************************/

int
lids_local_off(void)
{
	struct task_struct *t;

	read_lock(&tasklist_lock);
	t = current;
	while (t && (t->pid > 1)) {
		if (t->pid == lids_local_pid) {
			read_unlock(&tasklist_lock);
			return 1;
		}
		t = t->parent;
	}
	read_unlock(&tasklist_lock);
	return 0;
}

/*
 *
 *	LIDS ACL Function 
 *
 */
static int
lids_check_acl_inode(struct inode *inode, int type)
{

	struct lids_task_acl *task_acl = current->security;
	struct lids_object_acl *o_acl; 
	struct lids_inode_acl *i_acl= inode->i_security;
	//time_t currenttime;
	int	i=0;


	if (!(task_acl && task_acl->s_acl))
		return -EPERM;
	
	while (i_acl->perm[i].sid != 0 && i< 64) {

		o_acl = task_acl->s_acl->o_acl;
		while(o_acl) {
			if ( i_acl->perm[i].sid == o_acl->sid &&
				i_acl->perm[i].oid == o_acl->oid ) {

			return type & i_acl->perm[i].type ? 0 : -EPERM;
			}
			o_acl = o_acl->next;
		}
		i++;
	}
	return -EPERM;
}
#if 0
void show_lids_sec(struct lids_inode_acl *i_acl)
{
	struct lids_subject_acl *s_acl;
	struct lids_object_acl *o_acl;
	
	if(!i_acl) return;
	printk("\tshow: inode: %s, %d\n",i_acl->name, i_acl->type);
	s_acl = i_acl->s_acl;
	if(!s_acl) return;
	printk("\t -- cap %x \n",s_acl->sys_cap);
	o_acl = s_acl->o_acl;
	if(!o_acl) return;
	while(o_acl) {
	printk("\t -- obj [%s] type %d inherit %d\n",o_acl->name,o_acl->type,o_acl->inherit);
	o_acl = o_acl->next;
	}
}
#endif
/*
*    ACLS computed routine.
*/
static int
lids_compute_inherit_acl(struct lids_subject_acl *current_s_acl,
			 struct lids_subject_acl *computed_s_acl)
{
	struct lids_object_acl *src_acl, *dst_acl;

	computed_s_acl->o_acl = NULL;
	src_acl = current_s_acl->o_acl;
	
	while (src_acl) {
		if (src_acl->inherit != 0) {
			dst_acl =
			    kmalloc(sizeof (struct lids_object_acl),
				    GFP_KERNEL);
			if (!dst_acl) {
				LIDS_DBG("kmalloc failed\n");
				lids_free_subject_acl(computed_s_acl);
				return -ENOMEM;
			}
			memcpy(dst_acl, src_acl,
			       sizeof (struct lids_object_acl));
			if (dst_acl->inherit > 0)
				dst_acl->inherit--;

			LIDS_DBG	
			    ("     +++ pid %i: 1 ACL inherited. remaining TTL : %i\n",
			     current->pid, dst_acl->inherit);

			dst_acl->next = computed_s_acl->o_acl;
			computed_s_acl->o_acl = dst_acl;

		} else {
			LIDS_DBG	
			    ("     + pid %i: 1 ACL not inherited: TTL elapsed.\n",
			     current->pid);
		}
		src_acl = src_acl->next;
	}
	LIDS_DBG("%s:     = pid %i: %s inherit acls\n", __FUNCTION__, current->pid,
		 computed_s_acl->o_acl ? "does" : "does not");
	return 0;
}

static void
lids_compute_inherit_cap(struct lids_subject_acl *current_s_acl,
			 struct lids_subject_acl *computed_s_acl)
{
	int i;

	computed_s_acl->ext_cap = current_s_acl->ext_cap;

	if (!current_s_acl->sys_cap)
		return;
	computed_s_acl->sys_cap = 0;
	/* reset the cap_inherit */
	for (i = 0; i < 32; i++) {
		if (test_bit(i, (void *)&current_s_acl->sys_cap) &&
		    (current_s_acl->cap_inherit[i].inherit != 0)) {
			set_bit(i, (void *)&computed_s_acl->sys_cap);
			memcpy(&computed_s_acl->cap_inherit[i],
			       &current_s_acl->cap_inherit[i],
			       sizeof (struct lids_cap));
			if (current_s_acl->cap_inherit[i].inherit > 0)
				computed_s_acl->cap_inherit[i].inherit--;
		}
	}
	LIDS_DBG("%s: %d current %x computed %x\n",__FUNCTION__, current->pid, current_s_acl->sys_cap, computed_s_acl->sys_cap);

}

static int
lids_compute_new_acl(struct lids_subject_acl *new_s_acl,
		     struct lids_subject_acl *computed_s_acl)
{
	struct lids_object_acl *src_acl, *dst_acl;

	src_acl = new_s_acl->o_acl;
	computed_s_acl->o_acl = NULL;
	while (src_acl) {
		LIDS_DBG("     + pid %i: getting a new fs ACL, %s type %d inherit %d\n", current->pid,src_acl->name, src_acl->type, src_acl->inherit);
		dst_acl = kmalloc(sizeof (struct lids_object_acl), GFP_KERNEL);
		if (!dst_acl) {
			LIDS_DBG("kmalloc failed\n");
			return -3;
		}
		memcpy(dst_acl, src_acl, sizeof (struct lids_object_acl));
		dst_acl->next = computed_s_acl->o_acl;
		computed_s_acl->o_acl = dst_acl;
		src_acl = src_acl->next;
	}
	return 0;
}

static void
lids_compute_new_cap(struct lids_subject_acl *new_s_acl,
		     struct lids_subject_acl *computed_s_acl)
{
	int i;

	computed_s_acl->sys_cap |= new_s_acl->sys_cap;

	/* 0. SOCKET inherit FIXME later */
	computed_s_acl->ext_cap |= new_s_acl->ext_cap;
	/* if its parent do not has mark, use its own */
	memcpy(computed_s_acl->port, new_s_acl->port, sizeof(computed_s_acl->port));

	for (i = 0; i < 32; i++) {
		/* Here we do an unsigned comparison for -1 to be the biggest number */
		if (test_bit(i, (void *)&new_s_acl->sys_cap) &&
		    ((u32) computed_s_acl->cap_inherit[i].inherit <
		     (u32) new_s_acl->cap_inherit[i].inherit)) {

			computed_s_acl->cap_inherit[i].inherit =
			    new_s_acl->cap_inherit[i].inherit;
		}
	}
}

/*
 *	lids_set_acls, this_sys_acl must be NOT NULL.
 *	protected if the flag for current process, if it is under protected, protected = 1, otherwise = 0
 */
int
lids_compute_acls(struct lids_subject_acl *current_s_acl,
		  struct lids_subject_acl *new_s_acl,
		  struct lids_subject_acl *computed_s_acl, int protect)
{

	memset(computed_s_acl, 0, sizeof (struct lids_subject_acl));

	if (current_s_acl) {

		LIDS_DBG("%s: + pid %i: inherit ACLs: %lx ext %lx port %d oacl %p\n",__FUNCTION__,
			 current->pid, current_s_acl->sys_cap,
			 current_s_acl->ext_cap,current_s_acl->port[0][0],
			current_s_acl->o_acl);

		lids_compute_inherit_cap(current_s_acl, computed_s_acl);

		if (protect) {
			if (lids_compute_inherit_acl
			    (current_s_acl, computed_s_acl) < 0)
				return -ENOMEM;
		}
	}

	if (new_s_acl && protect) {

		LIDS_DBG	
		    ("%s: + pid %i: getting new ACLs: cap %lx, ext_cap %lx computed %lx, ext %lx port %d oacl %p\n",__FUNCTION__,
		     current->pid, new_s_acl->sys_cap, new_s_acl->ext_cap,
		     new_s_acl->ext_cap, computed_s_acl->ext_cap,
		     new_s_acl->port[0][0],
		     new_s_acl->o_acl);

		lids_compute_new_cap(new_s_acl, computed_s_acl);

		if (lids_compute_new_acl(new_s_acl, computed_s_acl) < 0) {
			return -ENOMEM;
		}

	}

	LIDS_DBG("%s: = pid %i: final caps : %#lx ext_cap = %lx port %d o_acl = %p\n",
		 __FUNCTION__, current->pid, computed_s_acl->sys_cap,
		 computed_s_acl->ext_cap, computed_s_acl->port[0][0], computed_s_acl->o_acl);


	return 0;
}

/*
 * apply the acl to task->security
 */
void
lids_set_task_acl(struct lids_subject_acl *s_acl, struct task_struct *task)
{
	struct lids_task_acl *acl = task->security;

	if (!task || !s_acl) {
		printk(KERN_INFO "LIDS: %s:yee..bug!\n", __FUNCTION__);
		return;
	}
	if (s_acl) {
		/* check this acl, to see if it really contain an ACL */
		if (s_acl->sys_cap == 0 && s_acl->o_acl == NULL &&
		    s_acl->ext_cap == 0) {
			lids_free_subject_acl(s_acl);
			/* lock ?? */
			spin_lock(&acl->t_lock);
			acl->s_acl = NULL;
			spin_unlock(&acl->t_lock);
		} else {
			/* lock ?? */
			spin_lock(&acl->t_lock);
			acl->s_acl = s_acl;
			spin_unlock(&acl->t_lock);
			LIDS_DBG(" pid %i: set caps : %#lx\n", task->pid,
				 s_acl->sys_cap);
		}
	}
	return;
}

static struct lids_subject_acl *
lids_copy_subject_acl(struct lids_subject_acl *src)
{

	struct lids_object_acl *s, *d;
	struct lids_subject_acl *dst;

	dst = (struct lids_subject_acl *)
	    kmalloc(sizeof (struct lids_subject_acl), GFP_KERNEL);
	if (!dst) {
		LIDS_DBG("kmalloc error\n");
		return NULL;
	}
	memcpy(dst, src, sizeof (struct lids_subject_acl));

	/* 1 . copy lids_acl */
	dst->o_acl = NULL;
	s = src->o_acl;
	while (s) {
		d = kmalloc(sizeof (struct lids_object_acl), GFP_KERNEL);
		if (!d) {
			LIDS_DBG("kmalloc error\n");
			lids_free_subject_acl(dst);
			return NULL;
		}
		memcpy(d, s, sizeof (struct lids_object_acl));
		d->next = dst->o_acl;
		dst->o_acl = d;
		s = s->next;
	}

	return dst;
}

/***********************************************************************
 *
 *	lids_check_base();
 *	
 *	check if the base have been protected by the IDS system.
 *	use the base->d_parent 
 *	check if the requried access can be permitted
 */

int
lids_check_base(struct dentry *dentry, int flag)
{
	struct inode *inode = dentry->d_inode;
	struct lids_inode_acl *i_acl;
	int error;

	if (inode == NULL)
		return 0;

	error = lids_get_inode_security(dentry, inode);
	if (error) {
		printk("%s: yeee. [%s] error ??\n",__FUNCTION__, dentry->d_iname);
		return -EPERM;
	}

	i_acl = (struct lids_inode_acl *) inode->i_security;

	LIDS_DBG("%s: LIDS ACL: i_acl= %p, name=[%s], inode = %d\n",
		 __FUNCTION__, i_acl, dentry->d_iname, inode->i_ino);
	/* if it is a socket or link .. */
	if (i_acl == NULL) {
		return 0;
	}
	/* do not have any acl */
	if(i_acl->type == 0xffffffff) {
		return 0;
	}

	if (((i_acl->type) & flag) > 0)
		return 0;

	return lids_check_acl_inode(inode, flag);
}

static int
lids_get_task_acl(struct task_struct *task, struct lids_task_acl *task_acl,
		  struct lids_inode_acl *i_acl)
{
	struct lids_subject_acl *task_s_acl = NULL;
	struct lids_subject_acl *current_s_acl = NULL;
	struct lids_subject_acl *new_s_acl = NULL;
	struct lids_subject_acl *computed_s_acl = NULL;
	struct lids_task_acl *current_task_acl = task->security;
	int retval = 0;

	LIDS_DBG("%s:##### pid %i ppid %d  \n", __FUNCTION__, task->pid,
		 task->parent->pid);

	/* if no acl for itself and its parent has not acl */
	if (i_acl)
		new_s_acl = i_acl->s_acl;
	if (task_acl)
		task_s_acl = task_acl->s_acl;
	/* if no acl with this inode and not for its parent */
	if (!new_s_acl && !task_s_acl) {
		return 0;
	}

	if (current_task_acl == NULL) {
		current_task_acl = lids_alloc_task_acl(task);
		if (current_task_acl == NULL) {
			printk(KERN_INFO
			       "LIDS: kmalloc memeory for task acl\n");
			return -ENOMEM;
		}
	} else {
		if (current_task_acl->magic != LIDS_MAGIC) {
			printk
			    (KERN_INFO
			     "LIDS: Bug!! task security magic mismatch!\n");
			/* return 0? */
			return 0;
		}
	}
	current_s_acl = current_task_acl->s_acl;

	/* check if this program is protected or not */
	if (i_acl && (i_acl->type & LIDS_APPEND) == 0) {
		/* task and its parent do not have acl */
		computed_s_acl =
		    kmalloc(sizeof (struct lids_subject_acl), GFP_KERNEL);
		if (computed_s_acl == NULL) {
			printk(KERN_INFO
			       "LIDS: kmalloc error for computed acl\n");
			retval = -ENOMEM;
			goto out;
		}

		if (lids_compute_acls(task_s_acl, new_s_acl, computed_s_acl, 1)
		    < 0) {
			lids_free_subject_acl(computed_s_acl);
			retval = -EPERM;
			goto out;
		}

		LIDS_DBG("%s: LIDS: protected pid %d %d  get acl\n",
			 __FUNCTION__, task->pid, task->parent->pid);

		spin_lock(&current_task_acl->t_lock);
		current_task_acl->s_acl = computed_s_acl;
		spin_unlock(&current_task_acl->t_lock);
	} else {

		/* reset all the privileges  */
		/* maybe we need some locks here */
		if (task_s_acl) {

			computed_s_acl =
			    kmalloc(sizeof (struct lids_subject_acl),
				    GFP_KERNEL);
			if (computed_s_acl == NULL) {
				printk
				    (KERN_INFO
				     "LIDS: kmalloc error for computed acl\n");
				retval = -ENOMEM;
				goto out;
			}
			retval =
			    lids_compute_acls(task_s_acl, NULL, computed_s_acl,
					      0);
			if (retval < 0) {
				lids_free_subject_acl(computed_s_acl);
				retval = -EPERM;
				goto out;
			}
			/* restrict socket access found */
			/* remove the capability but leave socket */
			if (computed_s_acl->ext_cap != 0) {
				LIDS_DBG
				    ("LIDS: unprotected pid %d %d  get socket inheritance\n",
				     task->pid, task->parent->pid);
				lids_free_object_acl(computed_s_acl->o_acl);
				computed_s_acl->o_acl = NULL;
				computed_s_acl->sys_cap = 0;
				memset(computed_s_acl->cap_inherit, 0,
				       32 * sizeof (struct lids_cap));
				spin_lock(&current_task_acl->t_lock);
				current_task_acl->s_acl = computed_s_acl;
				spin_unlock(&current_task_acl->t_lock);
			} else {
				/* clean the task */
				task_lock(task);
				task->security = NULL;
				task_unlock(task);
				lids_free_subject_acl(computed_s_acl);
				spin_lock(&current_task_acl->t_lock);
				current_task_acl->s_acl = NULL;
				spin_unlock(&current_task_acl->t_lock);
			}
		}
	}
      out:
	lids_free_subject_acl(current_s_acl);

	current_s_acl = current_task_acl->s_acl;
	if (!current_s_acl) {
		task_lock(task);
		task->security = NULL;
		task_unlock(task);
		lids_free_task_acl(current_task_acl);
	}
	return retval;
}

/* lids task acl */
static LIST_HEAD(lids_init_head);
static spinlock_t lids_init_lock = SPIN_LOCK_UNLOCKED;

static int
lids_push_task_acl(struct task_struct *task)
{
	struct lids_task_acl *task_acl;

	/* init, ignore it */
	if (task->parent->pid == 0)
		return 0;

	LIDS_DBG("%s: pusing %d %d\n", __FUNCTION__, task->pid,
		 task->parent->pid);
	task_acl = task->security;
	/* it maybe have acl when switch within states */
	if (!task_acl) {
		/* can not sleep */
		task_acl = lids_alloc_task_acl(task);
		if (!task_acl) {
			return -ENOMEM;
		}
	}
	/* can I hold a lock */
	spin_lock(&lids_init_lock);
	if (list_empty(&task_acl->list))
		list_add(&task_acl->list, &lids_init_head);
	spin_unlock(&lids_init_lock);
	return 0;
}

/*
 *
 */
struct dentry *
lids_get_task_dentry(struct task_struct *task)
{
	struct dentry *dentry = NULL;
	struct vm_area_struct *vma;

	if (task->mm) {
		vma = task->mm->mmap;
		while (vma) {
			if ((vma->vm_flags & VM_EXECUTABLE) && vma->vm_file) {
				dentry = vma->vm_file->f_dentry;
				break;
			}
			vma = vma->vm_next;
		}
	}
	return dentry;
}

static int
lids_attach_task_acl(struct task_struct *task)
{
	struct lids_task_acl *task_acl;
	struct dentry *dentry = NULL;
	int retval;

	/* Get dentry of current process, if any */

	dentry = lids_get_task_dentry(task);
	if (!dentry)
		return 0;

	LIDS_DBG("attaching pid %d ppid %d %s\n", task->pid, task->parent->pid,
		 dentry->d_iname);

	retval = lids_get_inode_security(dentry, dentry->d_inode);
	if (retval < 0)
		return retval;

	task_acl = task->security;
	if (task_acl && task_acl->s_acl) {
		lids_free_subject_acl(task_acl->s_acl);
		/* FIXME lock ?? */
		task_acl->s_acl = NULL;
	}
	task_acl = task->parent->security;
	retval = lids_get_task_acl(task, task_acl, dentry->d_inode->i_security);

	return retval;
}

int
lids_setup_task_acl(int state)
{
	struct task_struct *p;

	read_lock(&tasklist_lock);
	for_each_process(p) {
		if (lids_push_task_acl(p) < 0)
			return -1;
	}
	read_unlock(&tasklist_lock);
	/* attach now */
	spin_lock(&lids_init_lock);
      next_task:
	if (!list_empty(&lids_init_head)) {
		struct lids_task_acl *task_acl;
		task_acl = list_entry(lids_init_head.next,
				      struct lids_task_acl, list);
		//struct inode *inode = isec->inode;
		spin_unlock(&lids_init_lock);

		lids_attach_task_acl(task_acl->task);

		spin_lock(&lids_init_lock);
		list_del_init(&task_acl->list);
		goto next_task;
	}
	spin_unlock(&lids_init_lock);

	return 0;
}

/* inode acl */


/* 
	inherit the acl from its parent/dir
*/

void
lids_do_inode_post_create(struct inode *dir_inode, struct dentry *dentry)
{
#if 0
	struct inode *inode = dentry->d_inode;
	struct lids_inode_acl *dir_i_acl = dir_inode->i_security;
	struct lids_inode_acl *i_acl;
	struct dentry *i_dentry = NULL;
	struct dentry *dir_dentry = NULL;
	int len, retval;

	if (!inode || !dir_i_acl)
		return;
	if (inode->i_op->getxattr == NULL || dir_i_acl->magic != LIDS_MAGIC)
		return;

	LIDS_DBG("%s: inode %ld\n", __FUNCTION__, inode->i_ino);

	i_dentry = dget(dentry);
	if (!i_dentry)
		return;
	dir_dentry = d_find_alias(dir_inode);
	if (!dir_dentry)
		return;

	i_acl = kmalloc(sizeof (struct lids_inode_acl), GFP_KERNEL);
	if (!i_acl) {
		printk(KERN_INFO "LIDS: kmalloc for lids_inode_acl failed\n");
		dput(i_dentry);
		return;
	}

	len = inode->i_op->getxattr(dir_dentry, lids_xattr_name[lids_state],
				    NULL, 0);
	if (len > 0) {
		if (len !=
		    (sizeof (struct lids_inode_acl) - sizeof (spinlock_t))) {
			printk
			    (KERN_INFO
			     "LIDS: dir getxattr return len(%d) mistmatch\n",
			     len);
			return;
		}
		if (len > LIDS_MAX_XATTR_LEN) {
			printk(KERN_INFO
			       "LIDS: xattr len %d larger than defined max len %d\n",
			       len, LIDS_MAX_XATTR_LEN);
			return;
		}
		retval =
		    inode->i_op->getxattr(dir_dentry,
					  lids_xattr_name[lids_state], i_acl,
					  len);
		/* we got it, so current acl is not the defualt one. */
		if (retval >= 0) {
			len =
			    inode->i_op->setxattr(i_dentry,
						  lids_xattr_name[lids_state],
						  i_acl, len, 0);
		}
		/* get the defualt acl */
	}
	retval = inode->i_op->getxattr(dir_dentry, XATTR_NAME_LIDS, i_acl,
				       sizeof (struct lids_inode_acl) -
				       sizeof (spinlock_t));

	dput(dir_dentry);

	if (retval >= 0) {
		len = inode->i_op->setxattr(i_dentry, XATTR_NAME_LIDS, i_acl,
					    sizeof (struct lids_inode_acl) -
					    sizeof (spinlock_t), 0);
	}
	/* set the xattr now */
	if (len < 0) {
		printk(KERN_INFO "LIDS:  setxattr failed for %s\n",
		       i_dentry->d_iname);
	}
	dput(i_dentry);
	kfree(i_acl);
#endif
	return;
}
static int 
lids_copy_inode_acl(struct lids_inode_acl *d_i_acl, struct lids_inode_acl *s_i_acl)
{
	struct lids_subject_acl *d_s_acl;
	struct lids_object_acl *d_o_acl,*s_o_acl;

	memcpy(d_i_acl, s_i_acl, sizeof(struct lids_inode_acl));

	LIDS_DBG("%s: inode name = %s, perm sid %d oid %d \n",__FUNCTION__,s_i_acl->name,s_i_acl->perm[0].sid, s_i_acl->perm[0].oid);

	if(s_i_acl->s_acl) {
		d_s_acl = kmalloc(sizeof(struct lids_subject_acl),GFP_KERNEL);
		if(!d_s_acl) {
			printk("%s: LIDS: kmalloc subjet acl error\n",__FUNCTION__);
			return -ENOMEM;
		}	
		memcpy(d_s_acl, s_i_acl->s_acl,sizeof(struct lids_subject_acl));

		d_i_acl->s_acl = d_s_acl;	
		d_s_acl->o_acl = NULL;

		s_o_acl = s_i_acl->s_acl->o_acl; 

		while(s_o_acl) {
			d_o_acl = kmalloc(sizeof(struct lids_object_acl),GFP_KERNEL);
			if(!d_o_acl) {
				printk("%s: LIDS: kmalloc object acl error\n",__FUNCTION__);
				return -ENOMEM;
			}	
			memcpy(d_o_acl, s_o_acl,sizeof(struct lids_object_acl));
			d_o_acl->next = d_s_acl->o_acl;
			d_s_acl->o_acl = d_o_acl;

			s_o_acl = s_o_acl->next;

		}
	}
	return 0;
}

static struct lids_inode_acl *
lids_set_inode_acl(struct inode *inode, struct lids_inode_acl *c_i_acl)
{
	struct lids_inode_acl *i_acl;

	i_acl = kmalloc(sizeof (struct lids_inode_acl), GFP_KERNEL);
	if (!i_acl) {
		printk(KERN_INFO "LIDS: kmalloc failed for inode_acl\n");
		return NULL;
	}
	if(!c_i_acl) {
		c_i_acl = kmalloc(sizeof(struct lids_inode_acl),GFP_KERNEL);	
		if(!c_i_acl) {
			printk("LIDS: fatal error c_i_acl kmalloc failed\n");
			return NULL;
		}
		memset(c_i_acl, 0, sizeof(struct lids_inode_acl));
		c_i_acl->version = lids_update_version; 
		c_i_acl->type = 0xffffffff;
		c_i_acl->magic = LIDS_MAGIC;
	}
	lids_copy_inode_acl(i_acl, c_i_acl);
	/* FIXME, if inode has security */
	
	spin_lock(&inode->i_lock);
	lids_free_inode_acl(inode->i_security);
	inode->i_security = i_acl;
	spin_unlock(&inode->i_lock);

	return c_i_acl;
}

static int
lids_get_inode_security(struct dentry *dentry, struct inode *inode)
{
	struct lids_inode_acl *i_acl, *c_i_acl;
	struct dentry *i_dentry = NULL;

	if (inode == NULL)
		return 0;

	if (!(S_ISREG(inode->i_mode) || S_ISDIR(inode->i_mode)
	      || S_ISLNK(inode->i_mode))) {
		return 0;
	}

	LIDS_DBG("%s: ##### inode: mode %d [%d:%d %ld]\n", __FUNCTION__,
		 inode->i_mode,
		 MAJOR(inode->i_sb->s_dev),
		 MINOR(inode->i_sb->s_dev), inode->i_ino);

	i_acl = inode->i_security;
	if(i_acl && i_acl->version == lids_update_version) { 
		return 0;
	}

	if (!dentry) {
		printk("%s: dentry is NULL, Bug? \n",__FUNCTION__);
		return 0;
	}
	i_dentry = dentry;
	while(1) {
		c_i_acl = lids_do_get_acl(i_dentry->d_inode);
		if(c_i_acl != NULL || i_dentry == i_dentry->d_parent ) {
			c_i_acl = lids_set_inode_acl(i_dentry->d_inode, c_i_acl);
			break;
		}
		i_dentry = i_dentry->d_parent;

		i_acl = i_dentry->d_inode->i_security;
		if(i_acl && i_acl->version == lids_update_version) {
			c_i_acl = i_acl;
			break;
		}
	}
	if(!c_i_acl) {
		return -ENOMEM;
	}
	while(dentry != i_dentry) {
		c_i_acl = lids_set_inode_acl(dentry->d_inode, c_i_acl);
		if(!c_i_acl) 
			return -ENOMEM;
		dentry = dentry->d_parent;
		/* it is root now */	
	}
	return 0;	
}

#if 0
static int
lids_check_shellcode(struct linux_binprm *bprm)
{
	struct page *page;
	int i, offset, err = 0;
	char *kaddr, *paddr, *p;
	struct dentry *dentry;
/*
 *	copy the userspace charctor into a buffer to hold all the 
 * 	envp here.. and check it.
 */
	offset = (bprm->p) % PAGE_SIZE;
	i = (bprm->p) / PAGE_SIZE;
	kaddr = p = kmalloc(MAX_ARG_PAGES * PAGE_SIZE - bprm->p, GFP_KERNEL);
	if (!p)
		return -ENOMEM;

	while (i < MAX_ARG_PAGES) {
		page = bprm->page[i];
		paddr = kmap(page);

		memcpy(kaddr, paddr + offset, PAGE_SIZE - offset);
		kaddr += PAGE_SIZE - offset;
		kunmap(page);
		offset = 0;
		i++;
	}
	kaddr = p;
	i = 0;
	while (i < bprm->envc + bprm->argc) {
		if (strlen(kaddr) > LIDS_SHELLCODE_LENGTH) {
			lids_security_alert
			    ("Found overlong %s when exec %s: length = %d",
			     i < bprm->argc ? "parameters" : "env",
			     bprm->filename, strlen(kaddr));
#ifdef LIDS_SHELLCODE_STRING
			if (strstr(kaddr, LIDS_SHELLCODE_STRING)) {
				err = -1;
				dentry = bprm->file->f_dentry;
				lids_security_alert
				    ("Shellcode detected when exec %s, program terminated!",
				     bprm->filename);
				break;
			}
#endif
		}
		kaddr = strchr(kaddr, '\0');
		kaddr = kaddr + 1;
		i++;
	}
	kfree(p);
	return err;
}
#endif

static int
lids_check_envp(struct linux_binprm *bprm)
{
	struct page *page;
	int err = 0;
	char *kaddr, *paddr, *p, *end_p;
	struct dentry *dentry;
	unsigned long offset;
	unsigned long i;
/*
 *	copy the userspace charctor into a buffer to hold all the 
 * 	envp here.. and check it.
 */
	offset = (bprm->p) % PAGE_SIZE;
	i = (bprm->p) / PAGE_SIZE;
	kaddr = p = kmalloc(MAX_ARG_PAGES * PAGE_SIZE - bprm->p, GFP_KERNEL);

	if (!p)
		return -1;

	end_p = p + MAX_ARG_PAGES * PAGE_SIZE - bprm->p; 
	while (i < MAX_ARG_PAGES) {
		page = bprm->page[i];
		paddr = kmap(page);
		/* make sure each time, the kaddr is not out of bound */
		if( kaddr + PAGE_SIZE - offset > end_p ) {
			kfree(p);
			return -1;
		} 
		memcpy(kaddr, paddr + offset, PAGE_SIZE - offset);
		kaddr += PAGE_SIZE - offset;
		kunmap(page);
		offset = 0;
		i++;
	}
	kaddr = p;
	i = 0;
	while (i < bprm->envc + bprm->argc) {
		if (i >= bprm->argc) {
			LIDS_DBG("%s:str[%d] = [%s]\n", __FUNCTION__, i, kaddr);
			if ((kaddr[0] == 'L' || kaddr[0] == 'l') &&
			    (kaddr[1] == 'D' || kaddr[1] == 'd') &&
			    (kaddr[2] == '_')) {
				err = -1;
				break;
			}
		}
		kaddr = strchr(kaddr, '\0');
		kaddr = kaddr + 1;
		/* overflow checking */
		if(kaddr > end_p) {
			kfree(p);
			return -1;
		} 
		i++;
	}

	if (err) {
		dentry = bprm->file->f_dentry;
		lids_security_alert
		    ("Attempt to give [%.128s] to privilegied program %.128s (dev %d:%d inode %ld)",
		     kaddr, bprm->filename, MAJOR(dentry->d_inode->i_sb->s_dev),
		     MINOR(dentry->d_inode->i_sb->s_dev),
		     dentry->d_inode->i_ino);
	}
	kfree(p);
	return err;
}
/*
 *	checking the envp 
 */
int
lids_execve_check_envp(struct linux_binprm *bprm)
{
	struct lids_task_acl *current_task_acl = current->security;
	struct lids_subject_acl *current_s_acl=NULL;

	if (current_task_acl && current_task_acl->s_acl) {
		current_s_acl = current_task_acl->s_acl;

		if ((current_s_acl->o_acl || current_s_acl->sys_cap)
		    && (lids_load && lids_local_load)) {
			if (!lids_check_capable(current, CAP_SYS_PTRACE, 0)) {
				if (lids_check_envp(bprm) < 0) {
					task_lock(current);
					current->security = NULL;
					task_unlock(current);
					lids_free_task_acl(current_task_acl);
				}
			}
		}
	}
	return 0;
}

/*
 *	the current->security  struct lids_sys_acl
 */
int
lids_execve(struct linux_binprm *bprm)
{
	struct lids_task_acl *current_task_acl = current->security;
	struct dentry *dentry,*t_dentry;
	struct lids_inode_acl *i_acl = NULL;
	int error;

	if (current->parent->pid == 0)
		return 0;

	if (!bprm || !bprm->file) {
		printk("LIDS: %s:BUG!\n", __FUNCTION__);
		return 0;
	}

	LIDS_DBG("@@@@@@ %s:##### pid %i ppid %i exec [%s]\n", __FUNCTION__,
		 current->pid, current->parent->pid, bprm->filename);
	dentry = bprm->file->f_dentry;
	/* check if this dentry is the same as this pid or not */
	/* LIDS_EXEC checking here */

	t_dentry = lids_get_task_dentry(current);

	if (t_dentry && lids_load && lids_local_load 
			&& lids_ext_capable(current, 15) < 0) {
		if (dentry->d_inode->i_ino != t_dentry->d_inode->i_ino ||
		    dentry->d_inode->i_sb->s_dev !=
		    t_dentry->d_inode->i_sb->s_dev 
		    ) {
			lids_security_alert
			    ("pid %i ppid %i, exec [%s] Denied\n",
			     current->pid, current->parent->pid,
			     bprm->filename);
			return -EPERM;
		}
	}

	if (dentry && dentry->d_inode) {
		error = lids_get_inode_security(dentry, dentry->d_inode);
		if (error < 0)
			return error;
		i_acl = dentry->d_inode->i_security;
	}

	error = lids_get_task_acl(current, current_task_acl, i_acl);
	if (error)
		return error;

	return 0;
}

/* copy the fork 
 */

int
lids_fork_task(struct task_struct *tsk)
{
	struct lids_subject_acl *src = NULL;
	struct lids_subject_acl *dst = NULL;
	struct lids_task_acl *task_acl, *current_task_acl;

	if (!tsk) {
		printk(KERN_INFO "LIDS: %s: BUG\n", __FUNCTION__);
		return 0;
	}

	if (!tsk->parent->pid)
		return 0;

	current_task_acl = current->security;

	if (!current_task_acl) {
		LIDS_DBG(KERN_WARNING "%s: current task is NULL\n",
			 __FUNCTION__);
		return 0;
	}

	src = current_task_acl->s_acl;

	if (!src) {
		return 0;
	}

	if ((dst = lids_copy_subject_acl(src)) == NULL) {
		LIDS_DBG("lids_copy_subject_acl error\n");
		return -1;
	}

	task_acl = lids_alloc_task_acl(tsk);
	if (!task_acl) {
		LIDS_DBG(KERN_WARNING "LIDS: kmalloc failed for task_acl\n");
		return -ENOMEM;
	}
	task_acl->s_acl = dst;
	return 0;
}

int
lids_check_task_kill(struct task_struct *p, struct siginfo *info, int sig)
{
	struct lids_task_acl *task_acl = p->security;
	struct lids_subject_acl *s_acl = task_acl->s_acl;

	if (s_acl && cap_raised(s_acl->sys_cap, CAP_PROTECTED)) {
		if (current->pid && (current->pid != p->pid)
		    && ((sig != SIGCHLD) || (current->parent->pid != p->pid))) {
			if (!capable(CAP_KILL_PROTECTED)) {
				lids_security_alert
				    ("Attempt to kill pid=%d with sig=%d",
				     p->pid, sig);
				lids_cap_log(CAP_PROTECTED);
				return LIDS_ERROR(-EPERM);
			}
		}
	}
	return 0;
}
