/*
 *  LIDS Capability functions 
 *
 *  Copyright (C) 2002 Huagang Xie <xie@lids.org>
 *  Copyright (C) 2002 Philippe Biondi <biondi@cartel-securite.fr>
 *
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2 of the License, or
 *	(at your option) any later version.
 */
#include <linux/config.h>
#include <linux/module.h>
#include <linux/init.h>
#include <linux/kernel.h>
#include <linux/errno.h>
#include <linux/sched.h>
#include <linux/security.h>
#include <linux/capability.h>
#include <linux/mm.h>
#include <linux/mman.h>
#include <linux/slab.h>
#include <linux/smp_lock.h>
#include <linux/spinlock.h>
#include <linux/file.h>
#include <linux/ext2_fs.h>
#include <net/ip.h>		/* for sysctl_local_port_range[] */
#include <asm/uaccess.h>
#include <asm/semaphore.h>
#include <asm/ioctls.h>
#include <linux/lids.h>
#include <linux/lidsext.h>
#include <linux/lidsif.h>
/*
 *	lids capability violate logging
 */

kernel_cap_t lids_cap_val = 0;

static char *lids_caps_desc[] = {
	"CAP_CHOWN",
	"CAP_DAC_OVERRIDE",
	"CAP_DAC_READ_SEARCH",
	"CAP_FOWNER",
	"CAP_FSETID",
	"CAP_KILL",
	"CAP_SETGID",
	"CAP_SETUID",
	"CAP_SETPCAP",
	"CAP_LINUX_IMMUTABLE",
	"CAP_NET_BIND_SERVICE",
	"CAP_NET_BROADCAST",
	"CAP_NET_ADMIN",
	"CAP_NET_RAW",
	"CAP_IPC_LOCK",
	"CAP_IPC_OWNER",
	"CAP_SYS_MODULE",
	"CAP_SYS_RAWIO",
	"CAP_SYS_CHROOT",
	"CAP_SYS_PTRACE",
	"CAP_SYS_PACCT",
	"CAP_SYS_ADMIN",
	"CAP_SYS_BOOT",
	"CAP_SYS_NICE",
	"CAP_SYS_RESOURCE",
	"CAP_SYS_TIME",
	"CAP_SYS_TTY_CONFIG",
	"CAP_MKNOD",
	"CAP_LEASE",
	"CAP_HIDDEN",
	"CAP_KILL_PROTECTED",
	"CAP_PROTECTED",
	NULL
};

/*
 *	check the CAP_NET_BIND_SERVICE to bind to specify port
 */
int lids_bind_checker(const int port)
{
	int i = 0;
	struct lids_task_acl *task_acl = current->security;
	struct lids_subject_acl *s_acl;

	/* if CAP_NET_BIND_SERVICE is enable global, return success */
//	if (capable(CAP_NET_BIND_SERVICE))
	if (cap_raised(cap_bset, CAP_NET_BIND_SERVICE)) 
		return 1;
	/* if the LIDS is disable , return success */
	/* check only port < 1024) */
	if (!(lids_load && lids_local_load) || port > 1023 )
		return 1;
	if (!( task_acl && task_acl->s_acl))
		return 1;
	s_acl = task_acl->s_acl;

	for (i = 0; i < LIDS_PORT_ITEM && s_acl->port[i][0] != -1; i++) {
		if (port <= s_acl->port[i][1]
		    && port >= s_acl->port[i][0])
			return 1;
	}
	return -1;
}


int
lids_check_capable(struct task_struct *tsk, int cap, int log)
{
	struct lids_task_acl *tsk_acl = tsk->security;

	if (cap_raised(cap_bset, cap)) {
		return 0;
	} else if (tsk_acl && tsk_acl->s_acl) {
		if ((cap_raised(tsk_acl->s_acl->sys_cap, cap)))
			return 0;
	}
	if (log)
		lids_cap_log(cap);
	return LIDS_ERROR(-EPERM);
}

void
lids_cap_log(int cap)
{
	if (!cap_raised(lids_cap_val, cap)) {
		lids_alert(LIDS_CAP, -1, cap, lids_caps_desc[cap],
			   lids_caps_desc[cap]);
	}
}

int
lids_ext_capable(struct task_struct *tsk, int type)
{
	struct lids_task_acl *task_acl = tsk->security;
	struct lids_subject_acl *s_acl;

	if (!task_acl) {
		return 0;
	}
	s_acl = task_acl->s_acl;
	if (s_acl && test_bit(type, (void *)&(s_acl->ext_cap))) {
		return -EPERM;
	}
	return 0;
}
