/*
 *  LIDS INIT functions 
 *
 *  Copyright (C) 2002-2003 Huagang Xie <xie@lids.org>
 *  Copyright (C) 2002 Philippe Biondi <biondi@cartel-securite.fr>
 *
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2 of the License, or
 *	(at your option) any later version.
 */
#include <linux/config.h>
#include <linux/module.h>
#include <linux/init.h>
#include <linux/kernel.h>
#include <linux/errno.h>
#include <linux/sched.h>
#include <linux/security.h>
#include <linux/capability.h>
#include <linux/mm.h>
#include <linux/mman.h>
#include <linux/slab.h>
#include <linux/smp_lock.h>
#include <linux/spinlock.h>
#include <linux/file.h>
#include <linux/namei.h>
#include <linux/ext2_fs.h>
#include <asm/uaccess.h>
#include <asm/semaphore.h>
#include <asm/ioctls.h>
#include <linux/lids.h>
#include <linux/lidsext.h>
#include <linux/lidsif.h>

static int lids_lock_init = 0;
static spinlock_t lids_lock;
int	lids_init_setup;
static int lids_u_size = 0;

static char lids_binary_acl_file[3][PATH_MAX] =
    { LIDS_BOOT_ACL_FILE, LIDS_POSTBOOT_ACL_FILE, LIDS_SHUTDOWN_ACL_FILE };

char lids_state_name[3][9] = { "BOOT\0", "POSTBOOT\0", "SHUTDOWN\0" };

#define LIDS_MAX_ACL_NUM 1024
static struct lids_inode_acl lids_acl[2][LIDS_MAX_ACL_NUM];
static int lids_last_acl[2];
static int lids_eft_set; 
int lids_update_version;

/* fast guessing table*/
static unsigned long lids_bittab[32] = {
	0x00000001, 0x00000002, 0x00000004, 0x00000008,
	0x00000010, 0x00000020, 0x00000040, 0x00000080,
	0x00000100, 0x00000200, 0x00000400, 0x00000800,
	0x00001000, 0x00002000, 0x00004000, 0x00008000,
	0x00010000, 0x00020000, 0x00040000, 0x00080000,
	0x00100000, 0x00200000, 0x00400000, 0x00800000,
	0x01000000, 0x02000000, 0x04000000, 0x08000000,
	0x10000000, 0x20000000, 0x40000000, 0x80000000
};
static int fastguess[2][2048];
static unsigned long lids_search_value[2];

#if 0
/*
 *	input 	: inode
 *	output	: the acl found in the lids_user
 */
static int lids_search_inode(unsigned long ino, int major, int minor)
{
	dev_t  s_dev;
	dev_t dev = MKDEV(major,minor); 
	long j;
	long i = (ino ^ dev) & 0xffff;

	/* when the 'hash' bit is not set we for sure don't have
	   a matching entry, so just signal 'nothing found' */

	if (!(fastguess[lids_eft_set][i >> 5] & lids_bittab[i & 31]))
		return -1;
	/* ok, in case of e.g. 256 entries we would require 256 comparisons
	   for a linear search with no match, the following reduces this
	   to 8 comparisons (table is sorted!) */

	for (j = i = lids_search_value[lids_eft_set];; j >>= 1) {
		s_dev = MKDEV(lids_acl[lids_eft_set][i].inode.dev.major, 
				lids_acl[lids_eft_set][i].inode.dev.minor);

		if (i >= lids_last_acl[lids_eft_set])
			i -= j;
		else if (s_dev < dev)
			i += j;
		else if (s_dev > dev)
			i -= j;
		else if (lids_acl[lids_eft_set][i].inode.ino < ino)
			i += j;
		else if (lids_acl[lids_eft_set][i].inode.ino > ino)
			i -= j;
		else {
			return i; // return type??
		}
		if (!j || i < 0)
			return -1;
	}
}
#endif 

struct lids_inode_acl *
lids_do_get_acl(struct inode *inode)
{
#if 0
	int retval;

	retval = lids_search_inode(inode->i_ino, MAJOR(inode->i_sb->s_dev),  MINOR(inode->i_sb->s_dev));
	if(retval >=0  ) 
		return &lids_acl[lids_eft_set][retval];
	return NULL;
#else
	int i;

	for(i=0;i<lids_last_acl[lids_eft_set];i++) {
		if(inode->i_ino == lids_acl[lids_eft_set][i].inode.ino &&
			MAJOR(inode->i_sb->s_dev) == lids_acl[lids_eft_set][i].inode.dev.major && 
			MINOR(inode->i_sb->s_dev) == lids_acl[lids_eft_set][i].inode.dev.minor ) { 
			return &lids_acl[lids_eft_set][i];
		}
	}	
	return NULL;
#endif
	
}

static int
lids_buffer_to_inode_acl(char *buffer, int len, struct lids_inode_acl *i_acl)
{
	int num;
	char *p = buffer;
	struct lids_subject_acl *s_acl;
	struct lids_object_acl *o_acl, *pre_acl;
	u32 eft_set = (lids_eft_set & 1) ^ 1;
	int i;


	if (len < sizeof (struct lids_inode_acl) - sizeof(char*) + lids_u_size ) {
		printk(KERN_INFO "LIDS: Inode ACL incorrect, len = %d\n", len);
		return -1;
	}
	/* we do not have the psinLock_t in the xattr */
	memcpy(i_acl, p, sizeof (struct lids_inode_acl) - sizeof(char *) -64 );
	memcpy(i_acl->name, p+sizeof (struct lids_inode_acl)-sizeof(char*)+lids_u_size-64, 64);

	i_acl->version = lids_update_version;	/* current version */
	i_acl->s_acl = NULL;


	if (i_acl->magic != LIDS_MAGIC) {	/* LIDS magic */
		printk(KERN_INFO "LIDS: magic code mistach %x\n",
		       i_acl->magic);
		return -1;
	}
	/* fastguesing */
	i = ((MKDEV(i_acl->inode.dev.major,i_acl->inode.dev.minor)) ^ (i_acl->inode.ino)) & 0xffff;
	fastguess[eft_set][i >> 5] |= lids_bittab[i & 31];

	if (len == (sizeof (struct lids_inode_acl) ) - sizeof(char *)+lids_u_size ) {
		return 0;
	}
	len -= (sizeof (struct lids_inode_acl)  - sizeof(char *)+lids_u_size );

	s_acl = kmalloc(sizeof (struct lids_subject_acl), GFP_KERNEL);
	if (!s_acl) {
		return -ENOMEM;
	}

	p += (sizeof (struct lids_inode_acl) -sizeof(char *) + lids_u_size  );
	memcpy(s_acl, p, sizeof (struct lids_subject_acl) - sizeof(char *) + lids_u_size);

	/* set it */
	i_acl->s_acl = s_acl;
	s_acl->o_acl = NULL;

	if (len == sizeof (struct lids_subject_acl) - sizeof(char *) + lids_u_size) {
		return 0;
	}
	len -= sizeof (struct lids_subject_acl) - sizeof(char *) + lids_u_size;

	if (len < 0) {
		printk(KERN_INFO "LIDS: Subject ACL incorrect, len = %d\n",
		       len);
		return -1;
	}
	num = (unsigned int) (len % ( sizeof (struct lids_object_acl) - sizeof(char*) + lids_u_size));
	if (num > 0) {
		printk(KERN_INFO "LIDS: Object ACLs incorrect, len = %d\n",
		       len);
		return -1;
	}
	num = (unsigned int) (len / (sizeof (struct lids_object_acl) - sizeof(char*) + lids_u_size ));

	p += sizeof (struct lids_subject_acl) - sizeof(char *) + lids_u_size;;
	o_acl = pre_acl = NULL;
	for (i = 0; i < num; i++) {
		o_acl = kmalloc(sizeof (struct lids_object_acl), GFP_KERNEL);
		if (!o_acl) {
			return -ENOMEM;
		}
		memcpy(o_acl, p, sizeof (struct lids_object_acl) - sizeof(char*) + lids_u_size -64 );
		o_acl->next = pre_acl;
		memcpy(o_acl->name, p+sizeof (struct lids_object_acl) - sizeof(char*) + lids_u_size-64, 64);

		p += sizeof (struct lids_object_acl) - sizeof(char*) + lids_u_size;
		pre_acl = o_acl;
	}
	/* the last one */
	s_acl->o_acl = o_acl;
	return 0;

}
/* 
 * 	translate buffer into acl 
 */
static int
lids_buffer_to_acl(char *buffer, int len)
{
	int err=0;
	char *p , *q;
	u32 num;
	u32 plen=0;
	u32 hlen;
	u32 eft_set = (lids_eft_set & 1) ^ 1;
	
	hlen = sizeof (struct lids_inode_acl) - sizeof(char*) + lids_u_size; 
	p = q = buffer;

	while(len >= hlen ) {
		hlen = sizeof (struct lids_inode_acl) - sizeof(char*) - 64; 
		num = *(u32 *)(p+12);

		plen =  sizeof (struct lids_inode_acl) - sizeof(char*) + lids_u_size; 

		if(num != 0) {
			hlen = plen + sizeof (struct lids_subject_acl) - sizeof(char*) + lids_u_size;
			if(hlen > len) {
				return len;
			}
			num = *(u32 *)(p+plen+12); 
			plen = hlen;

			if(num != 0) {
				plen+=num*( sizeof (struct lids_object_acl) - sizeof(char *) + lids_u_size);
				if(plen > len) {
					return len;
				}
			}
		}
		err = lids_buffer_to_inode_acl(p, plen, &lids_acl[eft_set][lids_last_acl[eft_set]]);

		if(err) return err;
		lids_last_acl[eft_set]++;

		len -= plen;
		hlen = sizeof (struct lids_inode_acl) - sizeof(char*) + lids_u_size; 
		p += plen;
	}

	return len;
}
void
lids_free_lids_set(int eft_set)
{
	int i;

	for(i=0;i<lids_last_acl[eft_set];i++) {
		lids_free_subject_acl((lids_acl[eft_set][i].s_acl));
	}
	memset(lids_acl[eft_set],0,sizeof(lids_acl[eft_set]));
}
/*
 * lids read capability from /etc/lids/lids.cap 
 */

static int
lids_read_acl(int state)
{
	struct file *filp = NULL;
	char *buffer;
	mm_segment_t oldfs;
	int bytes,rlen = 1024;
	int error = 0;
	u32 start = 0, finished = 0;
	struct lids_acl_header hdr;
	u32  eft_set = (lids_eft_set & 1) ^ 1;

	lids_update_version++;
	lids_last_acl[eft_set] = 0;
	
	/* using MUTEX to protect lids_acl[] */		
	/* FIXME, need to free the subject+acl if any*/

	lids_free_lids_set(eft_set);
	
	filp = filp_open(lids_binary_acl_file[state-1], O_RDONLY, O_RDONLY);

	if (IS_ERR(filp) || (filp == NULL)) {
		error = -1;
		printk
		    ("LIDS: Error opening ACLs file %s in state %d,  Does it exist?\n",
		     lids_binary_acl_file[state-1], state);
		/* FIXME: if (lids_load) goto err_panic;  */
		return error;
	}

	if (filp->f_op->read == NULL) {
		fput(filp);
		error = -3;
		printk("LIDS: The capabiltiy file can not be read [state %d]\n",
		       state);
		/*
		   if (lids_load) goto err_panic ; 
		 */
		return error;
	}
	/* read the LIDS cap and version */
	filp->f_pos = 0;
	oldfs = get_fs();
	set_fs(KERNEL_DS);
	bytes = filp->f_op->read(filp, (char *)&hdr, sizeof(hdr), &filp->f_pos);
	set_fs(oldfs);
	
	if (bytes != sizeof(hdr)) {
		printk("LIDS: %s format error\n", lids_binary_acl_file[state]);
		fput(filp);
		return -4;
	}
	lids_cap_val = hdr.sys_cap;
	lids_search_value[eft_set] = hdr.search;
	lids_u_size = hdr.u_size;
	printk("LIDS: user space is %d bit\n",lids_u_size*8);

	if (lids_state == LIDS_STATE_BOOT) {
		lids_acl_discovery = hdr.discovery;
		memcpy(&lidsadm, &(hdr.lidsadm), sizeof(struct lids_s_inode));
		printk("LIDS: lidsadm inode 0x%x dev 0x%x:%x\n", lidsadm.ino, lidsadm.dev.major, lidsadm.dev.minor);
	}

	start = sizeof(hdr) ; 

	while (!finished) {

		buffer = kmalloc(rlen,GFP_KERNEL); 
		memset(buffer,0,rlen);

		filp->f_pos = start;
		oldfs = get_fs();
		set_fs(KERNEL_DS);
		bytes = filp->f_op->read(filp, buffer, rlen, &filp->f_pos);
		set_fs(oldfs);

		if (bytes < rlen) {
			finished = 1;
		}
		error = lids_buffer_to_acl(buffer,bytes);

		kfree(buffer);

		if (error<0)
			break;
		/* we do not have enough room for the whole buffer */
		if(bytes == error) {
			if(finished) {
				printk("LIDS: Format error\n");
				error = -10;	
				break;
			}else {
				rlen +=rlen; /* enlarge the buffer*/
			}
		}else{
			rlen = 1024;
		}

		start += bytes-error;
	}
	/* Close the file */
	fput(filp);

	/* switch it */
	lids_eft_set = eft_set;
	//printk("LIDS: Using eft_set %d , ",lids_eft_set);
	printk("LIDS: ACL Discovery: %s, ",lids_acl_discovery?"ON":"OFF");
	printk("Effetive Capability: %x,",lids_cap_val);
	printk("Total ACLs Count: %d\n",lids_last_acl[eft_set]);

	return error;
}

/***********************************************************************
 *
 *	lids_init
 *
 *	initialize the vfs security system. read the config file .
 *	add the inode to the files.
 *
 */

int
lids_init(void)
{
	int error = 0;
	/* Get lidsadm dev/inode */

	LIDS_DBG("into lids_init_..\n");

	lids_local_on = 0;
	lids_local_pid = current->pid;

	if (!lids_lock_init) {
		spin_lock_init(&lids_lock);
		lids_lock_init = 1;
	}

	/* read global acl */
	/* Read the password now */
	if (lids_read_pw()) {
		printk("LIDS: Read password file error\n");
		error = -8;
		goto lids_panic;
	}
	/* read capability first based on state */
	printk(KERN_INFO "LIDS: Initializing LIDS ACLs\n");

	if (lids_read_acl(lids_state)) {
		printk("LIDS: Read ACL file error, state %d\n",
		       lids_state);
		error = -9;
		goto lids_panic;
	}

	if (lids_state == LIDS_STATE_BOOT) {
		cap_bset = lids_cap_val;
		printk(KERN_INFO
		       "LIDS: GLOBAL and %s state configuration files loaded\n",
		       lids_state_name[lids_state - 1]);
		printk(KERN_INFO "LIDS: Entering %s state\n",
		       lids_state_name[lids_state - 1]);
	}
	lids_local_on = 1;
	if (!error) return 0;
lids_panic:
	printk
		("LIDS_ERR: Cannot initialize the lids system, return code %d",
		     error);
	return error;
}

/*
	do_lids_setup 
 */
int
do_lids_setup(void)
{
	int err = 0;

	/* init the ids file system */
	struct file *filp;

	filp = filp_open(LIDS_PW_FILE, O_RDONLY, O_RDONLY);

	if (IS_ERR(filp) || (filp == NULL)) {
		return -1;
	}
	
	lids_init_setup = 1;
	lids_local_on = 1;
	lids_flags = 0;
	lids_state = LIDS_STATE_BOOT;
	lids_flag_raise(lids_flags, LIDS_FLAGS_LIDS_LOCAL_ON);
	lids_flag_raise(lids_flags, LIDS_FLAGS_INIT);

	if (lids_load)
		lids_flag_raise(lids_flags, LIDS_FLAGS_LIDS_ON);

	lids_update_version = (int)get_seconds();
	lids_eft_set = 0;
	memset(lids_last_acl,0,sizeof(lids_last_acl));
	memset(lids_acl,0,sizeof(lids_acl));


	lids_sysctl_init();
	/* load BOOT acl */
	/* make it read the configure file easier. */
	err = lids_init();

	printk(KERN_NOTICE "LIDS: Linux Intrusion Detection System %s %s \n", LIDS_VERSION,
	       lids_load == 1 ? "started" : "not started");


	return err;
}
