#ifndef LIDS_H
#define LIDS_H

/*
 * This file include everything needed for LIDS internals.
 * The biggest part is included from in lidsif.h
 *
 */
#include <linux/kernel.h>
#include <linux/sysctl.h>
#include <linux/slab.h>
#include <linux/tty.h>
#include <linux/binfmts.h>

#include "lidsext.h"
#include "lidsif.h"

#ifndef KERNEL_VERSION
#define KERNEL_VERSION(x,y,z) (((x)<<16)+((y)<<8)+(z))
#endif

#define LIDS_VERSION	"2.2.1pre2"

#define LIDS_ERROR(value)	lids_acl_discovery?0:value

#define LIDS_SHELLCODE_LENGTH 512
#ifdef CONFIG_X86
#define LIDS_SHELLCODE_STRING "\xcd\x80"	/* X86 int 80 */
#endif
#ifdef CONFIG_SPARC32
#define LIDS_SHELLCODE_STRING "\x91\xd0\x20"	/* SPARC,  ta */
#endif
#ifdef CONFIG_PPC
#define LIDS_SHELLCODE_STRING " \x44\xFF\xFF\x02\x7C\xE0\x3B\x78"	/* system call, PPC  */
#endif
#ifdef CONFIG_MIPS
#define LIDS_SHELLCODE_STRING "\x02\x04\x8d\x0c"	/* system call, MIPS from irix  */
#endif

extern kernel_cap_t lids_cap_val;
extern struct lids_s_inode lidsadm;
extern char lids_state_name[3][9];
extern int lids_load;		/* 1 = load ids protection , 0 = don't load */
extern int lids_init_setup;	/* 1 = init the seutp, 0 = do not */
extern lids_flags_t lids_flags;	/* 1 = load ids protection , 0 = don't load */
extern int lids_local_on;
extern lids_flags_t lids_flags;
extern int lids_acl_discovery;	/* 1 = in ACL DISCOVERY MODE, 0 = in normal mode */
extern int lids_update_version;

int _open_namei(const char *pathname, int flag, int mode, struct nameidata *nd);
struct file *_filp_open(const char *filename, int flags, int mode);

extern void lids_free_task_security(struct task_struct *tsk);

int lids_init_task_acl(struct lids_task_acl *acl);
int lids_compute_acls(struct lids_subject_acl *current_s_acl,
		      struct lids_subject_acl *new_s_acl,
		      struct lids_subject_acl *computed_s_acl, int protect);
void lids_set_task_acl(struct lids_subject_acl *acl, struct task_struct *task);
void lids_free_lids_task_acl(struct lids_task_acl *acl);
/*
int lids_get_inode_security(struct dentry *o_dentry,
				   struct inode *inode);
*/
extern struct lids_inode_acl * lids_do_get_acl(struct inode *inode); 


extern struct lids_sys_acl *lids_search_acl(unsigned long int ino, dev_t dev,
					    unsigned long lids_curr);
extern int lids_check_base(struct dentry *dentry, int flag);
extern int lids_check_hidden_inode(unsigned long int ino, dev_t dev);
extern int lids_bind_checker(const int);
extern int lids_local_off(void);
extern int lids_execve(struct linux_binprm *);
extern int lids_execve_check_envp(struct linux_binprm *bprm);
extern int lids_fork_task(struct task_struct *tsk);
extern int lids_sysctl_init(void);
extern void lids_sysctl_reset(void);
extern int lids_check_task_kill(struct task_struct *p, struct siginfo *info,
				int sig);
//extern struct lids_task_acl *lids_alloc_task_acl(struct task_struct *);
extern void lids_free_task_acl(struct lids_task_acl *);
extern void lids_free_inode_acl(struct lids_inode_acl *);
extern void lids_free_subject_acl(struct lids_subject_acl *s_acl);

extern void lids_do_inode_post_create(struct inode *inode,
				      struct dentry *dentry);
extern int lids_setup_task_acl(int state);
extern struct dentry *lids_get_task_dentry(struct task_struct *task);

extern int lids_check_capable(struct task_struct *tsk, int cap, int log);
extern int lids_ext_capable(struct task_struct *tsk, int type);
extern void lids_free_security(struct task_struct *p);

extern void lids_alert(int type, long dst, long dst2, char *name, char *action);
extern int lids_read_pw(void);
extern int  do_lids_setup(void);

#endif				/* LIDS_H */
